@@ -21,5 +21,6 @@
 
 #include "JSCValue.h"
 
-JSValueRef jscValueGetJSValue(JSCValue*);
+JS_EXPORT_PRIVATE JSValueRef jscValueGetJSValue(JSCValue*);
 JSCValue* jscValueCreate(JSCContext*, JSValueRef);
+void jscValueAddPropertyAccessor(JSCValue*, const char*, GType, GCallback, GCallback, gpointer, GDestroyNotify);
