@@ -23,19 +23,6 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  */
 
-@linkTimeConstant
-@constructor
-function RegExpStringIterator(regExp, string, global, fullUnicode)
-{
-    "use strict";
-
-    @putByIdDirectPrivate(this, "regExpStringIteratorRegExp", regExp);
-    @putByIdDirectPrivate(this, "regExpStringIteratorString", string);
-    @putByIdDirectPrivate(this, "regExpStringIteratorGlobal", global);
-    @putByIdDirectPrivate(this, "regExpStringIteratorUnicode", fullUnicode);
-    @putByIdDirectPrivate(this, "regExpStringIteratorDone", false);
-}
-
 @linkTimeConstant
 function advanceStringIndex(string, index, unicode)
 {
@@ -88,12 +75,35 @@ function hasObservableSideEffectsForRegExpMatch(regexp)
     if (regexpExec !== @regExpBuiltinExec)
         return true;
 
+    var regexpFlags = @tryGetById(regexp, "flags");
+    if (regexpFlags !== @regExpProtoFlagsGetter)
+        return true;
+
+    // These are accessed by the builtin flags getter.
     var regexpGlobal = @tryGetById(regexp, "global");
     if (regexpGlobal !== @regExpProtoGlobalGetter)
         return true;
+    var regexpHasIndices = @tryGetById(regexp, "hasIndices");
+    if (regexpHasIndices !== @regExpProtoHasIndicesGetter)
+        return true;
+    var regexpIgnoreCase = @tryGetById(regexp, "ignoreCase");
+    if (regexpIgnoreCase !== @regExpProtoIgnoreCaseGetter)
+        return true;
+    var regexpMultiline = @tryGetById(regexp, "multiline");
+    if (regexpMultiline !== @regExpProtoMultilineGetter)
+        return true;
+    var regexpSticky = @tryGetById(regexp, "sticky");
+    if (regexpSticky !== @regExpProtoStickyGetter)
+        return true;
+    var regexpDotAll = @tryGetById(regexp, "dotAll");
+    if (regexpDotAll !== @regExpProtoDotAllGetter)
+        return true;
     var regexpUnicode = @tryGetById(regexp, "unicode");
     if (regexpUnicode !== @regExpProtoUnicodeGetter)
         return true;
+    var regexpUnicodeSets = @tryGetById(regexp, "unicodeSets");
+    if (regexpUnicodeSets !== @regExpProtoUnicodeSetsGetter)
+        return true;
 
     return typeof regexp.lastIndex !== "number";
 }
@@ -103,10 +113,13 @@ function matchSlow(regexp, str)
 {
     "use strict";
 
-    if (!regexp.global)
+    var flags = @toString(regexp.flags);
+    var global = @stringIncludesInternal.@call(flags, "g");
+
+    if (!global)
         return @regExpExec(regexp, str);
     
-    var unicode = regexp.unicode;
+    var unicode = @stringIncludesInternal.@call(flags, "u") || @stringIncludesInternal.@call(flags, "v");
     regexp.lastIndex = 0;
     var resultList = [];
 
@@ -169,9 +182,9 @@ function matchAll(strArg)
     matcher.lastIndex = @toLength(regExp.lastIndex);
 
     var global = @stringIncludesInternal.@call(flags, "g");
-    var fullUnicode = @stringIncludesInternal.@call(flags, "u");
+    var fullUnicode = @stringIncludesInternal.@call(flags, "u") || @stringIncludesInternal.@call(flags, "v");
 
-    return new @RegExpStringIterator(matcher, string, global, fullUnicode);
+    return @regExpStringIteratorCreate(matcher, string, global, fullUnicode);
 }
 
 @linkTimeConstant
@@ -189,7 +202,7 @@ function getSubstitution(matched, str, position, captures, namedCaptures, replac
 
     for (var start = 0; start = @stringIndexOfInternal.@call(replacement, "$", lastStart), start !== -1; lastStart = start) {
         if (start - lastStart > 0)
-            result = result + @stringSubstringInternal.@call(replacement, lastStart, start);
+            result = result + @stringSubstring.@call(replacement, lastStart, start);
         start++;
         if (start >= replacementLength)
             result = result + "$";
@@ -207,12 +220,12 @@ function getSubstitution(matched, str, position, captures, namedCaptures, replac
                 break;
             case "`":
                 if (position > 0)
-                    result = result + @stringSubstringInternal.@call(str, 0, position);
+                    result = result + @stringSubstring.@call(str, 0, position);
                 start++;
                 break;
             case "'":
                 if (tailPos < stringLength)
-                    result = result + @stringSubstringInternal.@call(str, tailPos);
+                    result = result + @stringSubstring.@call(str, tailPos);
                 start++;
                 break;
             case "<":
@@ -220,7 +233,7 @@ function getSubstitution(matched, str, position, captures, namedCaptures, replac
                     var groupNameStartIndex = start + 1;
                     var groupNameEndIndex = @stringIndexOfInternal.@call(replacement, ">", groupNameStartIndex);
                     if (groupNameEndIndex !== -1) {
-                        var groupName = @stringSubstringInternal.@call(replacement, groupNameStartIndex, groupNameEndIndex);
+                        var groupName = @stringSubstring.@call(replacement, groupNameStartIndex, groupNameEndIndex);
                         var capture = namedCaptures[groupName];
                         if (capture !== @undefined)
                             result = result + @toString(capture);
@@ -241,7 +254,7 @@ function getSubstitution(matched, str, position, captures, namedCaptures, replac
 
                     var n = chCode - 0x30;
                     if (n > m) {
-                        result = result + @stringSubstringInternal.@call(replacement, originalStart, start);
+                        result = result + @stringSubstring.@call(replacement, originalStart, start);
                         break;
                     }
 
@@ -257,7 +270,7 @@ function getSubstitution(matched, str, position, captures, namedCaptures, replac
                     }
 
                     if (n == 0) {
-                        result = result + @stringSubstringInternal.@call(replacement, originalStart, start);
+                        result = result + @stringSubstring.@call(replacement, originalStart, start);
                         break;
                     }
 
@@ -271,7 +284,7 @@ function getSubstitution(matched, str, position, captures, namedCaptures, replac
         }
     }
 
-    return result + @stringSubstringInternal.@call(replacement, lastStart);
+    return result + @stringSubstring.@call(replacement, lastStart);
 }
 
 @overriddenName="[Symbol.replace]"
@@ -291,11 +304,12 @@ function replace(strArg, replace)
     if (!functionalReplace)
         replace = @toString(replace);
 
-    var global = regexp.global;
+    var flags = @toString(regexp.flags);
+    var global = @stringIncludesInternal.@call(flags, "g");
     var unicode = false;
 
     if (global) {
-        unicode = regexp.unicode;
+        unicode = @stringIncludesInternal.@call(flags, "u") || @stringIncludesInternal.@call(flags, "v");
         regexp.lastIndex = 0;
     }
 
@@ -368,7 +382,7 @@ function replace(strArg, replace)
         }
 
         if (position >= nextSourcePosition) {
-            accumulatedResult = accumulatedResult + @stringSubstringInternal.@call(str, nextSourcePosition, position) + replacement;
+            accumulatedResult = accumulatedResult + @stringSubstring.@call(str, nextSourcePosition, position) + replacement;
             nextSourcePosition = position + matchLength;
         }
     }
@@ -376,54 +390,7 @@ function replace(strArg, replace)
     if (nextSourcePosition >= stringLength)
         return  accumulatedResult;
 
-    return accumulatedResult + @stringSubstringInternal.@call(str, nextSourcePosition);
-}
-
-// 21.2.5.9 RegExp.prototype[@@search] (string)
-@overriddenName="[Symbol.search]"
-function search(strArg)
-{
-    "use strict";
-
-    var regexp = this;
-
-    // Check for observable side effects and call the fast path if there aren't any.
-    if (@isRegExpObject(regexp)
-        && @tryGetById(regexp, "exec") === @regExpBuiltinExec
-        && typeof regexp.lastIndex === "number")
-        return @regExpSearchFast.@call(regexp, strArg);
-
-    // 1. Let rx be the this value.
-    // 2. If Type(rx) is not Object, throw a TypeError exception.
-    if (!@isObject(this))
-        @throwTypeError("RegExp.prototype.@@search requires that |this| be an Object");
-
-    // 3. Let S be ? ToString(string).
-    var str = @toString(strArg)
-
-    // 4. Let previousLastIndex be ? Get(rx, "lastIndex").
-    var previousLastIndex = regexp.lastIndex;
-
-    // 5. If SameValue(previousLastIndex, 0) is false, then
-    // 5.a. Perform ? Set(rx, "lastIndex", 0, true).
-    if (!@sameValue(previousLastIndex, 0))
-        regexp.lastIndex = 0;
-
-    // 6. Let result be ? RegExpExec(rx, S).
-    var result = @regExpExec(regexp, str);
-
-    // 7. Let currentLastIndex be ? Get(rx, "lastIndex").
-    // 8. If SameValue(currentLastIndex, previousLastIndex) is false, then
-    // 8.a. Perform ? Set(rx, "lastIndex", previousLastIndex, true).
-    if (!@sameValue(regexp.lastIndex, previousLastIndex))
-        regexp.lastIndex = previousLastIndex;
-
-    // 9. If result is null, return -1.
-    if (result === null)
-        return -1;
-
-    // 10. Return ? Get(result, "index").
-    return result.index;
+    return accumulatedResult + @stringSubstring.@call(str, nextSourcePosition);
 }
 
 @linkTimeConstant
@@ -448,6 +415,9 @@ function hasObservableSideEffectsForRegExpSplit(regexp)
     var regexpGlobal = @tryGetById(regexp, "global");
     if (regexpGlobal !== @regExpProtoGlobalGetter)
         return true;
+    var regexpHasIndices = @tryGetById(regexp, "hasIndices");
+    if (regexpHasIndices !== @regExpProtoHasIndicesGetter)
+        return true;
     var regexpIgnoreCase = @tryGetById(regexp, "ignoreCase");
     if (regexpIgnoreCase !== @regExpProtoIgnoreCaseGetter)
         return true;
@@ -457,10 +427,16 @@ function hasObservableSideEffectsForRegExpSplit(regexp)
     var regexpSticky = @tryGetById(regexp, "sticky");
     if (regexpSticky !== @regExpProtoStickyGetter)
         return true;
+    var regexpDotAll = @tryGetById(regexp, "dotAll");
+    if (regexpDotAll !== @regExpProtoDotAllGetter)
+        return true;
     var regexpUnicode = @tryGetById(regexp, "unicode");
     if (regexpUnicode !== @regExpProtoUnicodeGetter)
         return true;
-    
+    var regexpUnicodeSets = @tryGetById(regexp, "unicodeSets");
+    if (regexpUnicodeSets !== @regExpProtoUnicodeSetsGetter)
+        return true;
+
     // These are accessed by the RegExp species constructor.
     var regexpSource = @tryGetById(regexp, "source");
     if (regexpSource !== @regExpProtoSourceGetter)
@@ -496,9 +472,9 @@ function split(string, limit)
     // 5. Let flags be ? ToString(? Get(rx, "flags")).
     var flags = @toString(regexp.flags);
 
-    // 6. If flags contains "u", var unicodeMatching be true.
+    // 6. If flags contains "u" or flags contains "v", var unicodeMatching be true.
     // 7. Else, let unicodeMatching be false.
-    var unicodeMatching = @stringIncludesInternal.@call(flags, "u");
+    var unicodeMatching = @stringIncludesInternal.@call(flags, "u") || @stringIncludesInternal.@call(flags, "v");
     // 8. If flags contains "y", var newFlags be flags.
     // 9. Else, let newFlags be the string that is the concatenation of flags and "y".
     var newFlags = @stringIncludesInternal.@call(flags, "y") ? flags : flags + "y";
@@ -564,7 +540,7 @@ function split(string, limit)
             // iv. Else e != p,
             else {
                 // 1. Let T be a String value equal to the substring of S consisting of the elements at indices p (inclusive) through q (exclusive).
-                var subStr = @stringSubstringInternal.@call(str, position, matchPosition);
+                var subStr = @stringSubstring.@call(str, position, matchPosition);
                 // 2. Perform ! CreateDataProperty(A, ! ToString(lengthA), T).
                 // 3. Let lengthA be lengthA + 1.
                 @arrayPush(result, subStr);
@@ -599,40 +575,10 @@ function split(string, limit)
         }
     }
     // 20. Let T be a String value equal to the substring of S consisting of the elements at indices p (inclusive) through size (exclusive).
-    var remainingStr = @stringSubstringInternal.@call(str, position, size);
+    var remainingStr = @stringSubstring.@call(str, position, size);
     // 21. Perform ! CreateDataProperty(A, ! ToString(lengthA), T).
     @arrayPush(result, remainingStr);
     // 22. Return A.
     return result;
 }
 
-// ES 21.2.5.13 RegExp.prototype.test(string)
-@intrinsic=RegExpTestIntrinsic
-function test(strArg)
-{
-    "use strict";
-
-    var regexp = this;
-
-    // Check for observable side effects and call the fast path if there aren't any.
-    if (@isRegExpObject(regexp)
-        && @tryGetById(regexp, "exec") === @regExpBuiltinExec
-        && typeof regexp.lastIndex === "number")
-        return @regExpTestFast.@call(regexp, strArg);
-
-    // 1. Let R be the this value.
-    // 2. If Type(R) is not Object, throw a TypeError exception.
-    if (!@isObject(regexp))
-        @throwTypeError("RegExp.prototype.test requires that |this| be an Object");
-
-    // 3. Let string be ? ToString(S).
-    var str = @toString(strArg);
-
-    // 4. Let match be ? RegExpExec(R, string).
-    var match = @regExpExec(regexp, str);
-
-    // 5. If match is not null, return true; else return false.
-    if (match !== null)
-        return true;
-    return false;
-}
