@@ -47,7 +47,7 @@ class VarargsForwardingPhase : public Phase {
     static constexpr bool verbose = false;
 public:
     VarargsForwardingPhase(Graph& graph)
-        : Phase(graph, "varargs forwarding")
+        : Phase(graph, "varargs forwarding"_s)
     {
     }
     
@@ -55,10 +55,7 @@ public:
     {
         DFG_ASSERT(m_graph, nullptr, m_graph.m_form != SSA);
         
-        if (verbose) {
-            dataLog("Graph before varargs forwarding:\n");
-            m_graph.dump();
-        }
+        dataLogIf(verbose, "Graph before varargs forwarding:\n", m_graph);
         
         m_changed = false;
         for (BasicBlock* block : m_graph.blocksInNaturalOrder())
@@ -87,8 +84,7 @@ private:
         // We expect calls into this function to be rare. So, this is written in a simple O(n) manner.
         
         Node* candidate = block->at(candidateNodeIndex);
-        if (verbose)
-            dataLog("Handling candidate ", candidate, "\n");
+        dataLogLnIf(verbose, "Handling candidate ", candidate);
         
         // We eliminate GetButterfly over CreateClonedArguments if the butterfly is only
         // used by a GetByOffset  that loads the CreateClonedArguments's length. We also
@@ -104,8 +100,7 @@ private:
 
             auto defaultEscape = [&] {
                 if (m_graph.uses(node, candidate)) {
-                    if (verbose)
-                        dataLog("    Escape at ", node, "\n");
+                    dataLogLnIf(verbose, "    Escape at ", node);
                     return true;
                 }
                 return false;
@@ -140,8 +135,7 @@ private:
                         }
                     });
                 if (sawEscape) {
-                    if (verbose)
-                        dataLog("    Escape at ", node, "\n");
+                    dataLogLnIf(verbose, "    Escape at ", node);
                     return;
                 }
                 break;
@@ -158,8 +152,7 @@ private:
             case TailCallVarargs:
             case TailCallVarargsInlinedCaller:
                 if (node->child1() == candidate || node->child2() == candidate) {
-                    if (verbose)
-                        dataLog("    Escape at ", node, "\n");
+                    dataLogLnIf(verbose, "    Escape at ", node);
                     return;
                 }
                 if (node->child2() == candidate)
@@ -168,8 +161,7 @@ private:
                 
             case SetLocal:
                 if (node->child1() == candidate && node->variableAccessData()->isLoadedFrom()) {
-                    if (verbose)
-                        dataLog("    Escape at ", node, "\n");
+                    dataLogLnIf(verbose, "    Escape at ", node);
                     return;
                 }
                 break;
@@ -205,7 +197,8 @@ private:
                 break;
 
             case GetByOffset: {
-                if (node->child1()->op() == GetButterfly
+                if (!node->hasDoubleResult()
+                    && node->child1()->op() == GetButterfly
                     && candidateButterflies.contains(node->child1().node())
                     && node->child2() == candidate
                     && node->storageAccessData().offset == clonedArgumentsLengthPropertyOffset) {
@@ -230,8 +223,7 @@ private:
             if (!validGetByOffset) {
                 for (Node* butterfly : candidateButterflies) {
                     if (m_graph.uses(node, butterfly)) {
-                        if (verbose)
-                            dataLog("    Butterfly escaped at ", node, "\n");
+                        dataLogLnIf(verbose, "    Butterfly escaped at ", node);
                         return;
                     }
                 }
@@ -240,8 +232,7 @@ private:
             forAllKilledOperands(
                 m_graph, node, block->tryAt(nodeIndex + 1),
                 [&] (Operand operand) {
-                    if (verbose)
-                        dataLog("    Killing ", operand, " while we are interested in ", listDump(relevantLocals), "\n");
+                    dataLogLnIf(verbose, "    Killing ", operand, " while we are interested in ", listDump(relevantLocals));
                     for (unsigned i = 0; i < relevantLocals.size(); ++i) {
                         if (operand == relevantLocals[i]) {
                             relevantLocals[i--] = relevantLocals.last();
@@ -253,11 +244,10 @@ private:
                     }
                 });
         }
-        if (verbose)
-            dataLog("Selected lastUserIndex = ", lastUserIndex, ", ", block->at(lastUserIndex), "\n");
+        dataLogLnIf(verbose, "Selected lastUserIndex = ", lastUserIndex, ", ", block->at(lastUserIndex));
 
         InlineCallFrame* startingInlineCallFrame = block->at(candidateNodeIndex)->origin.forExit.inlineCallFrame();
-        HashSet<InlineCallFrame*, WTF::DefaultHash<InlineCallFrame*>, WTF::NullableHashTraits<InlineCallFrame*>> seenInlineCallFrames;
+        UncheckedKeyHashSet<InlineCallFrame*, WTF::DefaultHash<InlineCallFrame*>, WTF::NullableHashTraits<InlineCallFrame*>> seenInlineCallFrames;
         
         // We're still in business. Determine if between the candidate and the last user there is any
         // effect that could interfere with sinking.
@@ -270,26 +260,24 @@ private:
             // what gets read on OSR exit.
             switch (node->op()) {
             case MovHint:
+            case ZombieHint:
             case KillStack:
                 if (argumentsInvolveStackSlot(candidate, node->unlinkedOperand())) {
-                    if (verbose)
-                        dataLog("    Interference at ", node, "\n");
+                    dataLogLnIf(verbose, "    Interference at ", node);
                     return;
                 }
                 break;
                 
             case PutStack:
                 if (argumentsInvolveStackSlot(candidate, node->stackAccessData()->operand)) {
-                    if (verbose)
-                        dataLog("    Interference at ", node, "\n");
+                    dataLogLnIf(verbose, "    Interference at ", node);
                     return;
                 }
                 break;
                 
             case SetLocal:
                 if (argumentsInvolveStackSlot(candidate, node->operand())) {
-                    if (verbose)
-                        dataLog("    Interference at ", node, "\n");
+                    dataLogLnIf(verbose, "    Interference at ", node);
                     return;
                 }
                 break;
@@ -309,8 +297,7 @@ private:
                     },
                     NoOpClobberize());
                 if (doesInterfere) {
-                    if (verbose)
-                        dataLog("    Interference at ", node, "\n");
+                    dataLogLnIf(verbose, "    Interference at ", node);
                     return;
                 }
             } }
@@ -335,8 +322,7 @@ private:
         }
         
         // We can make this work.
-        if (verbose)
-            dataLog("    Will do forwarding!\n");
+        dataLogLnIf(verbose, "    Will do forwarding!");
         m_changed = true;
         
         // Transform the program.
