@@ -22,8 +22,9 @@ class TSymbolUniqueId
     POOL_ALLOCATOR_NEW_DELETE
     explicit TSymbolUniqueId(const TSymbol &symbol);
     constexpr TSymbolUniqueId(const TSymbolUniqueId &) = default;
-    TSymbolUniqueId &operator                          =(const TSymbolUniqueId &);
+    TSymbolUniqueId &operator=(const TSymbolUniqueId &);
     bool operator==(const TSymbolUniqueId &) const;
+    bool operator!=(const TSymbolUniqueId &) const;
 
     constexpr int get() const { return mId; }
 
@@ -37,7 +38,7 @@ class TSymbolUniqueId
     int mId;
 };
 
-enum class SymbolType
+enum class SymbolType : uint8_t
 {
     BuiltIn,
     UserDefined,
@@ -45,7 +46,7 @@ enum class SymbolType
     Empty  // Meaning symbol without a name.
 };
 
-enum class SymbolClass
+enum class SymbolClass : uint8_t
 {
     Function,
     Variable,
