@@ -77,7 +77,7 @@ class Renderbuffer final : public RefCountObject<RenderbufferID>,
 
     void onDestroy(const Context *context) override;
 
-    void setLabel(const Context *context, const std::string &label) override;
+    angle::Result setLabel(const Context *context, const std::string &label) override;
     const std::string &getLabel() const override;
 
     angle::Result setStorage(const Context *context,
@@ -144,9 +144,10 @@ class Renderbuffer final : public RefCountObject<RenderbufferID>,
     bool isRenderable(const Context *context,
                       GLenum binding,
                       const ImageIndex &imageIndex) const override;
+    bool isEGLImageSource() const;
 
-    void onAttach(const Context *context, rx::Serial framebufferSerial) override;
-    void onDetach(const Context *context, rx::Serial framebufferSerial) override;
+    void onAttach(const Context *context, rx::UniqueSerial framebufferSerial) override;
+    void onDetach(const Context *context, rx::UniqueSerial framebufferSerial) override;
     GLuint getId() const override;
 
     InitState initState(GLenum binding, const ImageIndex &imageIndex) const override;
