@@ -68,10 +68,10 @@ class TextureStorage11 : public TextureStorage
 
     static DWORD GetTextureBindFlags(GLenum internalFormat,
                                      const Renderer11DeviceCaps &renderer11DeviceCaps,
-                                     bool renderTarget);
+                                     BindFlags flags);
     static DWORD GetTextureMiscFlags(GLenum internalFormat,
                                      const Renderer11DeviceCaps &renderer11DeviceCaps,
-                                     bool renderTarget,
+                                     BindFlags flags,
                                      int levels);
 
     UINT getBindFlags() const;
@@ -80,6 +80,7 @@ class TextureStorage11 : public TextureStorage
     angle::Result getSRVLevels(const gl::Context *context,
                                GLint baseLevel,
                                GLint maxLevel,
+                               bool forceLinearSampler,
                                const d3d11::SharedSRV **outSRV);
     angle::Result generateSwizzles(const gl::Context *context,
                                    const gl::TextureState &textureState);
@@ -104,6 +105,9 @@ class TextureStorage11 : public TextureStorage
     bool isManaged() const override;
     bool supportsNativeMipmapFunction() const override;
     int getLevelCount() const override;
+    bool isUnorderedAccess() const override { return mBindFlags & D3D11_BIND_UNORDERED_ACCESS; }
+    bool isMultiplanar(const gl::Context *context) override;
+    bool requiresTypelessTextureFormat() const;
     angle::Result generateMipmap(const gl::Context *context,
                                  const gl::ImageIndex &sourceIndex,
                                  const gl::ImageIndex &destIndex) override;
@@ -240,7 +244,11 @@ class TextureStorage11 : public TextureStorage
     struct SamplerKey
     {
         SamplerKey();
-        SamplerKey(int baseLevel, int mipLevels, bool swizzle, bool dropStencil);
+        SamplerKey(int baseLevel,
+                   int mipLevels,
+                   bool swizzle,
+                   bool dropStencil,
+                   bool forceLinearSampler);
 
         bool operator<(const SamplerKey &rhs) const;
 
@@ -248,6 +256,7 @@ class TextureStorage11 : public TextureStorage
         int mipLevels;
         bool swizzle;
         bool dropStencil;
+        bool forceLinearSampler;
     };
 
     angle::Result getCachedOrCreateSRVForSampler(const gl::Context *context,
@@ -292,7 +301,7 @@ class TextureStorage11_2D : public TextureStorage11
     TextureStorage11_2D(Renderer11 *renderer, SwapChain11 *swapchain, const std::string &label);
     TextureStorage11_2D(Renderer11 *renderer,
                         GLenum internalformat,
-                        bool renderTarget,
+                        BindFlags bindFlags,
                         GLsizei width,
                         GLsizei height,
                         int levels,
@@ -491,6 +500,8 @@ class TextureStorage11_EGLImage final : public TextureStorage11ImmutableBase
                               const std::string &label);
     ~TextureStorage11_EGLImage() override;
 
+    angle::Result onDestroy(const gl::Context *context) override;
+
     angle::Result getSubresourceIndex(const gl::Context *context,
                                       const gl::ImageIndex &index,
                                       UINT *outSubresourceIndex) const override;
@@ -517,6 +528,13 @@ class TextureStorage11_EGLImage final : public TextureStorage11ImmutableBase
                                                 bool useLevelZeroTexture) override;
     void onLabelUpdate() override;
 
+    void associateImage(Image11 *image, const gl::ImageIndex &index) override;
+    void disassociateImage(const gl::ImageIndex &index, Image11 *expectedImage) override;
+    void verifyAssociatedImageValid(const gl::ImageIndex &index, Image11 *expectedImage) override;
+    angle::Result releaseAssociatedImage(const gl::Context *context,
+                                         const gl::ImageIndex &index,
+                                         Image11 *incomingImage) override;
+
   protected:
     angle::Result getSwizzleTexture(const gl::Context *context,
                                     const TextureHelper11 **outTexture) override;
@@ -544,6 +562,8 @@ class TextureStorage11_EGLImage final : public TextureStorage11ImmutableBase
     // Swizzle-related variables
     TextureHelper11 mSwizzleTexture;
     std::vector<d3d11::RenderTargetView> mSwizzleRenderTargets;
+
+    Image11 *mAssociatedImage;
 };
 
 class TextureStorage11_Cube : public TextureStorage11
@@ -551,7 +571,7 @@ class TextureStorage11_Cube : public TextureStorage11
   public:
     TextureStorage11_Cube(Renderer11 *renderer,
                           GLenum internalformat,
-                          bool renderTarget,
+                          BindFlags bindFlags,
                           int size,
                           int levels,
                           bool hintLevelZeroOnly,
@@ -647,7 +667,7 @@ class TextureStorage11_3D : public TextureStorage11
   public:
     TextureStorage11_3D(Renderer11 *renderer,
                         GLenum internalformat,
-                        bool renderTarget,
+                        BindFlags bindFlags,
                         GLsizei width,
                         GLsizei height,
                         GLsizei depth,
@@ -720,7 +740,7 @@ class TextureStorage11_2DArray : public TextureStorage11
   public:
     TextureStorage11_2DArray(Renderer11 *renderer,
                              GLenum internalformat,
-                             bool renderTarget,
+                             BindFlags bindFlags,
                              GLsizei width,
                              GLsizei height,
                              GLsizei depth,
