@@ -36,6 +36,7 @@ class Blit9;
 class Context9;
 class IndexDataManager;
 class ProgramD3D;
+class ProgramExecutableD3D;
 class RenderTarget9;
 class StreamingIndexBufferInterface;
 class StaticIndexBufferInterface;
@@ -129,7 +130,7 @@ class Renderer9 : public RendererD3D
     angle::Result applyRenderTarget(const gl::Context *context,
                                     const RenderTarget9 *colorRenderTarget,
                                     const RenderTarget9 *depthStencilRenderTarget);
-    void applyUniforms(ProgramD3D *programD3D);
+    void applyUniforms(ProgramExecutableD3D *executableD3D);
     bool applyPrimitiveType(gl::PrimitiveMode primitiveType,
                             GLsizei elementCount,
                             bool usesPointSize);
@@ -290,27 +291,27 @@ class Renderer9 : public RendererD3D
                                                  const std::string &label) override;
 
     TextureStorage *createTextureStorage2D(GLenum internalformat,
-                                           bool renderTarget,
+                                           BindFlags bindFlags,
                                            GLsizei width,
                                            GLsizei height,
                                            int levels,
                                            const std::string &label,
                                            bool hintLevelZeroOnly) override;
     TextureStorage *createTextureStorageCube(GLenum internalformat,
-                                             bool renderTarget,
+                                             BindFlags bindFlags,
                                              int size,
                                              int levels,
                                              bool hintLevelZeroOnly,
                                              const std::string &label) override;
     TextureStorage *createTextureStorage3D(GLenum internalformat,
-                                           bool renderTarget,
+                                           BindFlags bindFlags,
                                            GLsizei width,
                                            GLsizei height,
                                            GLsizei depth,
                                            int levels,
                                            const std::string &label) override;
     TextureStorage *createTextureStorage2DArray(GLenum internalformat,
-                                                bool renderTarget,
+                                                BindFlags bindFlags,
                                                 GLsizei width,
                                                 GLsizei height,
                                                 GLsizei depth,
@@ -446,7 +447,8 @@ class Renderer9 : public RendererD3D
     void generateCaps(gl::Caps *outCaps,
                       gl::TextureCapsMap *outTextureCaps,
                       gl::Extensions *outExtensions,
-                      gl::Limitations *outLimitations) const override;
+                      gl::Limitations *outLimitations,
+                      ShPixelLocalStorageOptions *outPLSOptions) const override;
 
     void initializeFeatures(angle::FeaturesD3D *features) const override;
 
