@@ -9,7 +9,7 @@
 #ifndef LIBANGLE_RENDERER_GL_CGL_DISPLAYCGL_H_
 #define LIBANGLE_RENDERER_GL_CGL_DISPLAYCGL_H_
 
-#include <thread>
+#include <unordered_set>
 
 #include "libANGLE/renderer/gl/DisplayGL.h"
 
@@ -22,7 +22,16 @@ typedef _CGLPixelFormatObject *CGLPixelFormatObj;
 namespace rx
 {
 
-class WorkerContext;
+struct EnsureCGLContextIsCurrent : angle::NonCopyable
+{
+  public:
+    EnsureCGLContextIsCurrent(CGLContextObj context);
+    ~EnsureCGLContextIsCurrent();
+
+  private:
+    CGLContextObj mOldContext;
+    bool mResetContext;
+};
 
 class DisplayCGL : public DisplayGL
 {
@@ -32,8 +41,11 @@ class DisplayCGL : public DisplayGL
 
     egl::Error initialize(egl::Display *display) override;
     void terminate() override;
+    egl::Error prepareForCall() override;
+    egl::Error releaseThread() override;
 
-    egl::Error makeCurrent(egl::Surface *drawSurface,
+    egl::Error makeCurrent(egl::Display *display,
+                           egl::Surface *drawSurface,
                            egl::Surface *readSurface,
                            gl::Context *context) override;
 
@@ -69,9 +81,8 @@ class DisplayCGL : public DisplayGL
 
     DeviceImpl *createDevice() override;
 
-    std::string getVendorString() const override;
-
     egl::Error waitClient(const gl::Context *context) override;
+    egl::Error waitUntilWorkScheduled() override;
     egl::Error waitNative(const gl::Context *context, EGLint engine) override;
 
     gl::Version getMaxSupportedESVersion() const override;
@@ -79,17 +90,18 @@ class DisplayCGL : public DisplayGL
     CGLContextObj getCGLContext() const;
     CGLPixelFormatObj getCGLPixelFormat() const;
 
-    WorkerContext *createWorkerContext(std::string *infoLog);
-
     void initializeFrontendFeatures(angle::FrontendFeatures *features) const override;
 
     void populateFeatureList(angle::FeatureList *features) override;
 
+    RendererGL *getRenderer() const override;
+
     // Support for dual-GPU MacBook Pros. Used only by ContextCGL. The use of
     // these entry points is gated by the presence of dual GPUs.
     egl::Error referenceDiscreteGPU();
     egl::Error unreferenceDiscreteGPU();
     egl::Error handleGPUSwitch() override;
+    egl::Error forceGPUSwitch(EGLint gpuIDHigh, EGLint gpuIDLow) override;
 
   private:
     egl::Error makeCurrentSurfaceless(gl::Context *context) override;
@@ -98,12 +110,13 @@ class DisplayCGL : public DisplayGL
     void generateCaps(egl::Caps *outCaps) const override;
 
     void checkDiscreteGPUStatus();
+    void setContextToGPU(uint64_t gpuID, GLint virtualScreen);
 
     std::shared_ptr<RendererGL> mRenderer;
 
     egl::Display *mEGLDisplay;
     CGLContextObj mContext;
-    std::unordered_map<std::thread::id, CGLContextObj> mCurrentContexts;
+    std::unordered_set<uint64_t> mThreadsWithCurrentContext;
     CGLPixelFormatObj mPixelFormat;
     bool mSupportsGPUSwitching;
     uint64_t mCurrentGPUID;
@@ -113,6 +126,7 @@ class DisplayCGL : public DisplayGL
     // is unref'd for the last time, this is set to the time of that last unref. If it isn't
     // activated again in 10 seconds, the discrete GPU pixel format is deleted.
     double mLastDiscreteGPUUnrefTime;
+    bool mDeviceContextIsVolatile = false;
 };
 
 }  // namespace rx
