@@ -11,12 +11,12 @@
 
 #include "libANGLE/Context.h"
 #include "libANGLE/Query.h"
-#include "libANGLE/renderer/glslang_wrapper_utils.h"
 #include "libANGLE/renderer/vulkan/BufferVk.h"
 #include "libANGLE/renderer/vulkan/ContextVk.h"
 #include "libANGLE/renderer/vulkan/FramebufferVk.h"
 #include "libANGLE/renderer/vulkan/ProgramVk.h"
 #include "libANGLE/renderer/vulkan/QueryVk.h"
+#include "libANGLE/renderer/vulkan/ShaderInterfaceVariableInfoMap.h"
 
 #include "common/debug.h"
 
@@ -32,6 +32,7 @@ TransformFeedbackVk::TransformFeedbackVk(const gl::TransformFeedbackState &state
       mCounterBufferHandles{},
       mCounterBufferOffsets{}
 {
+    mBufferObserverBindings.reserve(gl::IMPLEMENTATION_MAX_TRANSFORM_FEEDBACK_BUFFERS);
     for (angle::SubjectIndex bufferIndex = 0;
          bufferIndex < gl::IMPLEMENTATION_MAX_TRANSFORM_FEEDBACK_BUFFERS; ++bufferIndex)
     {
@@ -44,16 +45,14 @@ TransformFeedbackVk::~TransformFeedbackVk() {}
 void TransformFeedbackVk::onDestroy(const gl::Context *context)
 {
     ContextVk *contextVk   = vk::GetImpl(context);
-    RendererVk *rendererVk = contextVk->getRenderer();
-
-    releaseCounterBuffers(rendererVk);
+    releaseCounterBuffers(contextVk);
 }
 
-void TransformFeedbackVk::releaseCounterBuffers(RendererVk *renderer)
+void TransformFeedbackVk::releaseCounterBuffers(vk::Context *context)
 {
     for (vk::BufferHelper &bufferHelper : mCounterBufferHelpers)
     {
-        bufferHelper.release(renderer);
+        bufferHelper.release(context);
     }
     for (VkBuffer &buffer : mCounterBufferHandles)
     {
@@ -113,9 +112,10 @@ angle::Result TransformFeedbackVk::begin(const gl::Context *context,
             if (mCounterBufferHandles[bufferIndex] == VK_NULL_HANDLE)
             {
                 vk::BufferHelper &bufferHelper = mCounterBufferHelpers[bufferIndex];
-                ANGLE_TRY(bufferHelper.initSuballocation(
-                    contextVk, contextVk->getRenderer()->getDeviceLocalMemoryTypeIndex(), 16,
-                    contextVk->getRenderer()->getDefaultBufferAlignment()));
+                ANGLE_TRY(contextVk->initBufferAllocation(
+                    &bufferHelper, contextVk->getRenderer()->getDeviceLocalMemoryTypeIndex(), 16,
+                    contextVk->getRenderer()->getDefaultBufferAlignment(),
+                    BufferUsageType::Static));
                 mCounterBufferHandles[bufferIndex] = bufferHelper.getBuffer().getHandle();
                 mCounterBufferOffsets[bufferIndex] = bufferHelper.getOffset();
             }
@@ -152,7 +152,7 @@ angle::Result TransformFeedbackVk::end(const gl::Context *context)
 
     contextVk->onEndTransformFeedback();
 
-    releaseCounterBuffers(contextVk->getRenderer());
+    releaseCounterBuffers(contextVk);
 
     return angle::Result::Continue;
 }
@@ -192,29 +192,6 @@ angle::Result TransformFeedbackVk::bindIndexedBuffer(
     return angle::Result::Continue;
 }
 
-void TransformFeedbackVk::updateDescriptorSetLayout(
-    ContextVk *contextVk,
-    const ShaderInterfaceVariableInfoMap &variableInfoMap,
-    size_t xfbBufferCount,
-    vk::DescriptorSetLayoutDesc *descSetLayoutOut) const
-{
-    if (!contextVk->getFeatures().emulateTransformFeedback.enabled)
-    {
-        return;
-    }
-
-    for (uint32_t bufferIndex = 0; bufferIndex < xfbBufferCount; ++bufferIndex)
-    {
-        const ShaderInterfaceVariableInfo &info =
-            variableInfoMap.getTransformFeedbackInfo(gl::ShaderType::Vertex, bufferIndex);
-
-        ASSERT(info.binding != std::numeric_limits<uint32_t>::max());
-
-        descSetLayoutOut->update(info.binding, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1,
-                                 VK_SHADER_STAGE_VERTEX_BIT, nullptr);
-    }
-}
-
 void TransformFeedbackVk::getBufferOffsets(ContextVk *contextVk,
                                            GLint drawCallFirstVertex,
                                            int32_t *offsetsOut,
@@ -271,59 +248,50 @@ void TransformFeedbackVk::onSubjectStateChange(angle::SubjectIndex index,
         ASSERT(bufferVk->isBufferValid());
         mBufferHelpers[index] = &bufferVk->getBuffer();
         mBufferOffsets[index] = binding.getOffset() + mBufferHelpers[index]->getOffset();
-        mBufferSizes[index]   = gl::GetBoundBufferAvailableSize(binding);
+        mBufferSizes[index]   = std::min<VkDeviceSize>(gl::GetBoundBufferAvailableSize(binding),
+                                                       mBufferHelpers[index]->getSize());
         mBufferObserverBindings[index].bind(bufferVk);
         mBufferHandles[index] = mBufferHelpers[index]->getBuffer().getHandle();
     }
 }
 
-void TransformFeedbackVk::writeDescriptorSet(vk::Context *context,
-                                             UpdateDescriptorSetsBuilder *updateBuilder,
-                                             const ShaderInterfaceVariableInfoMap &variableInfoMap,
-                                             size_t xfbBufferCount,
-                                             VkDescriptorBufferInfo *bufferInfo,
-                                             VkDescriptorSet descSet) const
-{
-    ASSERT(context->getRenderer()->getFeatures().emulateTransformFeedback.enabled);
-
-    const ShaderInterfaceVariableInfo &info =
-        variableInfoMap.getTransformFeedbackInfo(gl::ShaderType::Vertex, 0);
-
-    VkWriteDescriptorSet &writeDescriptorInfo = updateBuilder->allocWriteDescriptorSet();
-    writeDescriptorInfo.sType                 = VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET;
-    writeDescriptorInfo.dstSet                = descSet;
-    writeDescriptorInfo.dstBinding            = info.binding;
-    writeDescriptorInfo.dstArrayElement       = 0;
-    writeDescriptorInfo.descriptorCount       = static_cast<uint32_t>(xfbBufferCount);
-    writeDescriptorInfo.descriptorType        = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER;
-    writeDescriptorInfo.pImageInfo            = nullptr;
-    writeDescriptorInfo.pBufferInfo           = bufferInfo;
-    writeDescriptorInfo.pTexelBufferView      = nullptr;
-}
-
 void TransformFeedbackVk::updateTransformFeedbackDescriptorDesc(
     const vk::Context *context,
     const gl::ProgramExecutable &executable,
     const ShaderInterfaceVariableInfoMap &variableInfoMap,
+    const vk::WriteDescriptorDescs &writeDescriptorDescs,
     const vk::BufferHelper &emptyBuffer,
     bool activeUnpaused,
     vk::DescriptorSetDescBuilder *builder) const
 {
     size_t xfbBufferCount = executable.getTransformFeedbackBufferCount();
-    builder->updateTransformFeedbackWrite(variableInfoMap, static_cast<uint32_t>(xfbBufferCount));
-
     for (uint32_t bufferIndex = 0; bufferIndex < xfbBufferCount; ++bufferIndex)
     {
         if (mBufferHelpers[bufferIndex] && activeUnpaused)
         {
-            builder->updateTransformFeedbackBuffer(
-                context, variableInfoMap, bufferIndex, *mBufferHelpers[bufferIndex],
-                mBufferOffsets[bufferIndex], mBufferSizes[bufferIndex]);
+            builder->updateTransformFeedbackBuffer(context, variableInfoMap, writeDescriptorDescs,
+                                                   bufferIndex, *mBufferHelpers[bufferIndex],
+                                                   mBufferOffsets[bufferIndex],
+                                                   mBufferSizes[bufferIndex]);
         }
         else
         {
-            builder->updateTransformFeedbackBuffer(context, variableInfoMap, bufferIndex,
-                                                   emptyBuffer, 0, emptyBuffer.getSize());
+            builder->updateTransformFeedbackBuffer(context, variableInfoMap, writeDescriptorDescs,
+                                                   bufferIndex, emptyBuffer, 0,
+                                                   emptyBuffer.getSize());
+        }
+    }
+}
+
+void TransformFeedbackVk::onNewDescriptorSet(const gl::ProgramExecutable &executable,
+                                             const vk::SharedDescriptorSetCacheKey &sharedCacheKey)
+{
+    size_t xfbBufferCount = executable.getTransformFeedbackBufferCount();
+    for (uint32_t bufferIndex = 0; bufferIndex < xfbBufferCount; ++bufferIndex)
+    {
+        if (mBufferHelpers[bufferIndex])
+        {
+            mBufferHelpers[bufferIndex]->onNewDescriptorSet(sharedCacheKey);
         }
     }
 }
