@@ -32,6 +32,10 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <wtf/ASCIICType.h>
 #include <wtf/HashMap.h>
 #include <wtf/Lock.h>
+#include <wtf/StdLibExtras.h>
+#include <wtf/ZippedRange.h>
+#include <wtf/text/AdaptiveStringSearcher.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/StringToIntegerConversion.h>
 #include <wtf/text/TextBreakIterator.h>
 #include <wtf/unicode/icu/ICUHelpers.h>
@@ -58,7 +62,7 @@ size_t StringView::findIgnoringASCIICase(StringView matchString, unsigned startO
     return ::WTF::findIgnoringASCIICase(*this, matchString, startOffset);
 }
 
-bool StringView::startsWith(UChar character) const
+bool StringView::startsWith(char16_t character) const
 {
     return m_length && (*this)[0] == character;
 }
@@ -73,7 +77,7 @@ bool StringView::startsWithIgnoringASCIICase(StringView prefix) const
     return ::WTF::startsWithIgnoringASCIICase(*this, prefix);
 }
 
-bool StringView::endsWith(UChar character) const
+bool StringView::endsWith(char16_t character) const
 {
     return m_length && (*this)[m_length - 1] == character;
 }
@@ -88,18 +92,18 @@ bool StringView::endsWithIgnoringASCIICase(StringView suffix) const
     return ::WTF::endsWithIgnoringASCIICase(*this, suffix);
 }
 
-Expected<CString, UTF8ConversionError> StringView::tryGetUtf8(ConversionMode mode) const
+Expected<CString, UTF8ConversionError> StringView::tryGetUTF8(ConversionMode mode) const
 {
     if (isNull())
-        return CString("", 0);
+        return CString { ""_span };
     if (is8Bit())
-        return StringImpl::utf8ForCharacters(characters8(), length());
-    return StringImpl::utf8ForCharacters(characters16(), length(), mode);
+        return StringImpl::utf8ForCharacters(span8());
+    return StringImpl::utf8ForCharacters(span16(), mode);
 }
 
 CString StringView::utf8(ConversionMode mode) const
 {
-    auto expectedString = tryGetUtf8(mode);
+    auto expectedString = tryGetUTF8(mode);
     RELEASE_ASSERT(expectedString);
     return expectedString.value();
 }
@@ -109,6 +113,58 @@ size_t StringView::find(StringView matchString, unsigned start) const
     return findCommon(*this, matchString, start);
 }
 
+size_t StringView::find(AdaptiveStringSearcherTables& tables, StringView matchString, unsigned start) const
+{
+    unsigned subjectLength = length();
+    unsigned matchLength = matchString.length();
+
+    if (start > subjectLength)
+        return notFound;
+
+    if (!matchLength)
+        return start;
+
+    if (subjectLength > INT32_MAX || matchLength > INT32_MAX) [[unlikely]]
+        return find(matchString, start);
+
+    if (is8Bit()) {
+        if (matchString.is8Bit())
+            return searchString(tables, span8(), matchString.span8(), start);
+        return searchString(tables, span8(), matchString.span16(), start);
+    }
+
+    if (matchString.is8Bit())
+        return searchString(tables, span16(), matchString.span8(), start);
+    return searchString(tables, span16(), matchString.span16(), start);
+}
+
+size_t StringView::find(std::span<const LChar> match, unsigned start) const
+{
+    ASSERT(!match.empty());
+    auto length = this->length();
+    if (start > length)
+        return notFound;
+
+    unsigned searchLength = length - start;
+    if (match.size() > searchLength)
+        return notFound;
+
+    if (is8Bit())
+        return findInner(span8().subspan(start), match, start);
+    return findInner(span16().subspan(start), match, start);
+}
+
+size_t StringView::reverseFind(std::span<const LChar> match, unsigned start) const
+{
+    ASSERT(!match.empty());
+    if (match.size() > length())
+        return notFound;
+
+    if (is8Bit())
+        return reverseFindInner(span8(), match, start);
+    return reverseFindInner(span16(), match, start);
+}
+
 void StringView::SplitResult::Iterator::findNextSubstring()
 {
     for (size_t separatorPosition; (separatorPosition = m_result.m_string.find(m_result.m_separator, m_position)) != notFound; ++m_position) {
@@ -136,7 +192,7 @@ auto StringView::SplitResult::Iterator::operator++() -> Iterator&
 }
 
 class StringView::GraphemeClusters::Iterator::Impl {
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_DEPRECATED_MAKE_FAST_ALLOCATED(Impl);
 public:
     Impl(StringView stringView, std::optional<NonSharedCharacterBreakIterator>&& iterator, unsigned index)
         : m_stringView(stringView)
@@ -155,9 +211,7 @@ public:
 
     StringView operator*() const
     {
-        if (m_stringView.is8Bit())
-            return StringView(m_stringView.characters8() + m_index, m_indexEnd - m_index);
-        return StringView(m_stringView.characters16() + m_index, m_indexEnd - m_index);
+        return m_stringView.substring(m_index, m_indexEnd - m_index);
     }
 
     bool operator==(const Impl& other) const
@@ -190,9 +244,7 @@ StringView::GraphemeClusters::Iterator::Iterator(StringView stringView, unsigned
 {
 }
 
-StringView::GraphemeClusters::Iterator::~Iterator()
-{
-}
+StringView::GraphemeClusters::Iterator::~Iterator() = default;
 
 StringView::GraphemeClusters::Iterator::Iterator(Iterator&& other)
     : m_impl(WTFMove(other.m_impl))
@@ -215,88 +267,79 @@ bool StringView::GraphemeClusters::Iterator::operator==(const Iterator& other) c
     return *m_impl == *(other.m_impl);
 }
 
-bool StringView::GraphemeClusters::Iterator::operator!=(const Iterator& other) const
-{
-    return !(*this == other);
-}
-
 enum class ASCIICase { Lower, Upper };
 
 template<ASCIICase type, typename CharacterType>
-String convertASCIICase(const CharacterType* input, unsigned length)
+String convertASCIICase(std::span<const CharacterType> input)
 {
     if (!input)
         return { };
 
-    CharacterType* characters;
-    auto result = String::createUninitialized(length, characters);
-    for (unsigned i = 0; i < length; ++i)
-        characters[i] = type == ASCIICase::Lower ? toASCIILower(input[i]) : toASCIIUpper(input[i]);
+    std::span<CharacterType> characters;
+    auto result = String::createUninitialized(input.size(), characters);
+    size_t i = 0;
+    for (auto character : input)
+        characters[i++] = type == ASCIICase::Lower ? toASCIILower(character) : toASCIIUpper(character);
     return result;
 }
 
 String StringView::convertToASCIILowercase() const
 {
     if (m_is8Bit)
-        return convertASCIICase<ASCIICase::Lower>(static_cast<const LChar*>(m_characters), m_length);
-    return convertASCIICase<ASCIICase::Lower>(static_cast<const UChar*>(m_characters), m_length);
+        return convertASCIICase<ASCIICase::Lower>(span8());
+    return convertASCIICase<ASCIICase::Lower>(span16());
 }
 
 String StringView::convertToASCIIUppercase() const
 {
     if (m_is8Bit)
-        return convertASCIICase<ASCIICase::Upper>(static_cast<const LChar*>(m_characters), m_length);
-    return convertASCIICase<ASCIICase::Upper>(static_cast<const UChar*>(m_characters), m_length);
+        return convertASCIICase<ASCIICase::Upper>(span8());
+    return convertASCIICase<ASCIICase::Upper>(span16());
 }
 
 template<typename CharacterType>
-static AtomString convertASCIILowercaseAtom(const CharacterType* input, unsigned length)
+static AtomString convertASCIILowercaseAtom(std::span<const CharacterType> input)
 {
-    for (unsigned i = 0; i < length; ++i) {
-        if (UNLIKELY(isASCIIUpper(input[i]))) {
-            CharacterType* characters;
-            auto result = String::createUninitialized(length, characters);
-            StringImpl::copyCharacters(characters, input, i);
-            for (; i < length; ++i)
-                characters[i] = toASCIILower(input[i]);
-            return result;
-        }
+    for (auto character : input) {
+        if (isASCIIUpper(character)) [[unlikely]]
+            return makeAtomString(asASCIILowercase(input));
     }
     // Fast path when the StringView is already all lowercase.
-    return AtomString(input, length);
+    return input;
 }
 
 AtomString StringView::convertToASCIILowercaseAtom() const
 {
     if (m_is8Bit)
-        return convertASCIILowercaseAtom(characters8(), m_length);
-    return convertASCIILowercaseAtom(characters16(), m_length);
+        return convertASCIILowercaseAtom(span8());
+    return convertASCIILowercaseAtom(span16());
 }
 
+
 template<typename DestinationCharacterType, typename SourceCharacterType>
-void getCharactersWithASCIICaseInternal(StringView::CaseConvertType type, DestinationCharacterType* destination, const SourceCharacterType* source, unsigned length)
+void getCharactersWithASCIICaseInternal(StringView::CaseConvertType type, std::span<DestinationCharacterType> destination, std::span<const SourceCharacterType> source)
 {
-    static_assert(std::is_same<SourceCharacterType, LChar>::value || std::is_same<SourceCharacterType, UChar>::value);
-    static_assert(std::is_same<DestinationCharacterType, LChar>::value || std::is_same<DestinationCharacterType, UChar>::value);
+    static_assert(std::is_same<SourceCharacterType, LChar>::value || std::is_same<SourceCharacterType, char16_t>::value);
+    static_assert(std::is_same<DestinationCharacterType, LChar>::value || std::is_same<DestinationCharacterType, char16_t>::value);
     static_assert(sizeof(DestinationCharacterType) >= sizeof(SourceCharacterType));
     auto caseConvert = (type == StringView::CaseConvertType::Lower) ? toASCIILower<SourceCharacterType> : toASCIIUpper<SourceCharacterType>;
-    for (unsigned i = 0; i < length; ++i)
-        destination[i] = caseConvert(source[i]);
+    for (auto [destinationCharacter, character] : zippedRange(destination, source))
+        destinationCharacter = caseConvert(character);
 }
 
-void StringView::getCharactersWithASCIICase(CaseConvertType type, LChar* destination) const
+void StringView::getCharactersWithASCIICase(CaseConvertType type, std::span<LChar> destination) const
 {
     ASSERT(is8Bit());
-    getCharactersWithASCIICaseInternal(type, destination, characters8(), m_length);
+    getCharactersWithASCIICaseInternal(type, destination, span8());
 }
 
-void StringView::getCharactersWithASCIICase(CaseConvertType type, UChar* destination) const
+void StringView::getCharactersWithASCIICase(CaseConvertType type, std::span<char16_t> destination) const
 {
     if (is8Bit()) {
-        getCharactersWithASCIICaseInternal(type, destination, characters8(), m_length);
+        getCharactersWithASCIICaseInternal(type, destination, span8());
         return;
     }
-    getCharactersWithASCIICaseInternal(type, destination, characters16(), m_length);
+    getCharactersWithASCIICaseInternal(type, destination, span16());
 }
 
 StringViewWithUnderlyingString normalizedNFC(StringView string)
@@ -310,18 +353,19 @@ StringViewWithUnderlyingString normalizedNFC(StringView string)
     ASSERT(U_SUCCESS(status));
 
     // No need to normalize if already normalized.
-    UBool checkResult = unorm2_isNormalized(normalizer, string.characters16(), string.length(), &status);
+    auto span = string.span16();
+    UBool checkResult = unorm2_isNormalized(normalizer, span.data(), span.size(), &status);
     if (checkResult)
         return { string, { } };
 
-    unsigned normalizedLength = unorm2_normalize(normalizer, string.characters16(), string.length(), nullptr, 0, &status);
+    unsigned normalizedLength = unorm2_normalize(normalizer, span.data(), span.size(), nullptr, 0, &status);
     ASSERT(needsToGrowToProduceBuffer(status));
 
-    UChar* characters;
+    std::span<char16_t> characters;
     String result = String::createUninitialized(normalizedLength, characters);
 
     status = U_ZERO_ERROR;
-    unorm2_normalize(normalizer, string.characters16(), string.length(), characters, normalizedLength, &status);
+    unorm2_normalize(normalizer, span.data(), span.size(), characters.data(), characters.size(), &status);
     ASSERT(U_SUCCESS(status));
 
     StringView view { result };
@@ -349,23 +393,102 @@ bool equalRespectingNullity(StringView a, StringView b)
     return equalCommon(a, b);
 }
 
-bool StringView::contains(const char* string) const
+size_t StringView::reverseFind(StringView matchString, unsigned start) const
 {
-    return find(string) != notFound;
+    if (isNull())
+        return notFound;
+
+    if (matchString.length())
+        return std::min(start, length());
+
+    // Check start & matchLength are in range.
+    if (matchString.length() > length())
+        return notFound;
+
+    if (is8Bit()) {
+        if (matchString.is8Bit())
+            return reverseFindInner(span8(), matchString.span8(), start);
+        return reverseFindInner(span8(), matchString.span16(), start);
+    }
+
+    if (matchString.is8Bit())
+        return reverseFindInner(span16(), matchString.span8(), start);
+    return reverseFindInner(span16(), matchString.span16(), start);
 }
 
-int codePointCompare(StringView lhs, StringView rhs)
+String makeStringByReplacingAll(StringView string, char16_t target, char16_t replacement)
+{
+    if (string.is8Bit()) {
+        if (!isLatin1(target)) {
+            // Looking for a 16-bit character in an 8-bit string, so we're done.
+            return string.toString();
+        }
+
+        auto characters = string.span8();
+        size_t i;
+        unsigned length = string.length();
+        for (i = 0; i != characters.size(); ++i) {
+            if (characters[i] == target)
+                break;
+        }
+        if (i == length)
+            return string.toString();
+        return StringImpl::createByReplacingInCharacters(characters, target, replacement, i);
+    }
+
+    auto characters = string.span16();
+    size_t i;
+    unsigned length = string.length();
+    for (i = 0; i != characters.size(); ++i) {
+        if (characters[i] == target)
+            break;
+    }
+    if (i == length)
+        return string.toString();
+    return StringImpl::createByReplacingInCharacters(characters, target, replacement, i);
+}
+
+std::strong_ordering codePointCompare(StringView lhs, StringView rhs)
 {
     bool lhsIs8Bit = lhs.is8Bit();
     bool rhsIs8Bit = rhs.is8Bit();
     if (lhsIs8Bit) {
         if (rhsIs8Bit)
-            return codePointCompare(lhs.characters8(), lhs.length(), rhs.characters8(), rhs.length());
-        return codePointCompare(lhs.characters8(), lhs.length(), rhs.characters16(), rhs.length());
+            return codePointCompare(lhs.span8(), rhs.span8());
+        return codePointCompare(lhs.span8(), rhs.span16());
     }
     if (rhsIs8Bit)
-        return codePointCompare(lhs.characters16(), lhs.length(), rhs.characters8(), rhs.length());
-    return codePointCompare(lhs.characters16(), lhs.length(), rhs.characters16(), rhs.length());
+        return codePointCompare(lhs.span16(), rhs.span8());
+    return codePointCompare(lhs.span16(), rhs.span16());
+}
+
+template<typename CharacterType> static String makeStringBySimplifyingNewLinesSlowCase(const String& string, unsigned firstCarriageReturn)
+{
+    unsigned length = string.length();
+    unsigned resultLength = firstCarriageReturn;
+    auto characters = string.span<CharacterType>();
+    std::span<CharacterType> resultCharacters;
+    auto result = String::createUninitialized(length, resultCharacters);
+    memcpySpan(resultCharacters, characters.first(firstCarriageReturn));
+    for (unsigned i = firstCarriageReturn; i < length; ++i) {
+        if (characters[i] != '\r')
+            resultCharacters[resultLength++] = characters[i];
+        else {
+            resultCharacters[resultLength++] = '\n';
+            if (i + 1 < length && characters[i + 1] == '\n')
+                ++i;
+        }
+    }
+    if (resultLength < length)
+        result = StringImpl::createSubstringSharingImpl(Ref { *result.impl() }, 0, resultLength);
+    return result;
+}
+
+String makeStringBySimplifyingNewLinesSlowCase(const String& string, unsigned firstCarriageReturn)
+{
+    if (string.is8Bit())
+        return makeStringBySimplifyingNewLinesSlowCase<LChar>(string, firstCarriageReturn);
+    return makeStringBySimplifyingNewLinesSlowCase<char16_t>(string, firstCarriageReturn);
 }
 
 #if CHECK_STRINGVIEW_LIFETIME
@@ -373,10 +496,10 @@ int codePointCompare(StringView lhs, StringView rhs)
 // Manage reference count manually so UnderlyingString does not need to be defined in the header.
 
 struct StringView::UnderlyingString {
-    WTF_MAKE_STRUCT_FAST_ALLOCATED;
+    WTF_DEPRECATED_MAKE_STRUCT_FAST_ALLOCATED(UnderlyingString);
     std::atomic_uint refCount { 1u };
     bool isValid { true };
-    const StringImpl& string;
+    SUPPRESS_UNCOUNTED_MEMBER const StringImpl& string;
     explicit UnderlyingString(const StringImpl&);
 };
 
@@ -406,7 +529,7 @@ void StringView::invalidate(const StringImpl& stringToBeDestroyed)
     underlyingString->isValid = false;
 }
 
-bool StringView::underlyingStringIsValid() const
+bool StringView::underlyingStringIsValidImpl() const
 {
     return !m_underlyingString || m_underlyingString->isValid;
 }
@@ -425,7 +548,7 @@ void StringView::adoptUnderlyingString(UnderlyingString* underlyingString)
     m_underlyingString = underlyingString;
 }
 
-void StringView::setUnderlyingString(const StringImpl* string)
+void StringView::setUnderlyingStringImpl(const StringImpl* string)
 {
     UnderlyingString* underlyingString;
     if (!string)
@@ -442,7 +565,7 @@ void StringView::setUnderlyingString(const StringImpl* string)
     adoptUnderlyingString(underlyingString);
 }
 
-void StringView::setUnderlyingString(const StringView& otherString)
+void StringView::setUnderlyingStringImpl(const StringView& otherString)
 {
     UnderlyingString* underlyingString = otherString.m_underlyingString;
     if (underlyingString) {
@@ -456,6 +579,34 @@ void StringView::setUnderlyingString(const StringView& otherString)
     adoptUnderlyingString(underlyingString);
 }
 
-#endif // CHECK_STRINGVIEW_LIFETIME
+#else
+
+bool StringView::underlyingStringIsValidImpl() const
+{
+    return true;
+}
+
+void StringView::setUnderlyingStringImpl(const StringImpl*)
+{
+}
+
+void StringView::setUnderlyingStringImpl(const StringView&)
+{
+}
+
+#endif // not CHECK_STRINGVIEW_LIFETIME
+
+#ifndef NDEBUG
+void StringView::show() const
+{
+    toStringWithoutCopying().show();
+}
+#endif
+
+#if !defined(NDEBUG)
+namespace Detail {
+std::atomic<int> wtfStringCopyCount;
+}
+#endif
 
 } // namespace WTF
