@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
+ * Copyright (C) 2021-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -59,7 +59,7 @@ private:
     }
 
     Ref<WebGPU::CommandBuffer> m_backing;
-    Ref<GPUCommandEncoder> m_encoder;
+    const Ref<GPUCommandEncoder> m_encoder;
 };
 
 }
