@@ -26,14 +26,13 @@
 #include "config.h"
 #include "LegacyCDMSessionClearKey.h"
 
-#include "JSExecState.h"
 #include "Logging.h"
-#include <pal/text/TextEncoding.h>
 #include "WebKitMediaKeyError.h"
-#include <JavaScriptCore/JSGlobalObjectInlines.h>
-#include <JavaScriptCore/JSLock.h>
-#include <JavaScriptCore/JSONObject.h>
-#include <JavaScriptCore/VM.h>
+#include <JavaScriptCore/GenericTypedArrayViewInlines.h>
+#include <JavaScriptCore/TypedArrayAdaptors.h>
+#include <pal/text/TextEncoding.h>
+#include <wtf/JSONValues.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/UUID.h>
 #include <wtf/text/Base64.h>
 
@@ -41,15 +40,10 @@
 
 
 namespace WebCore {
-using namespace JSC;
 
-static VM& clearKeyVM()
-{
-    static VM& vm = VM::create().leakRef();
-    return vm;
-}
+WTF_MAKE_TZONE_ALLOCATED_IMPL(CDMSessionClearKey);
 
-CDMSessionClearKey::CDMSessionClearKey(LegacyCDMSessionClient* client)
+CDMSessionClearKey::CDMSessionClearKey(LegacyCDMSessionClient& client)
     : m_client(client)
     , m_sessionId(createVersion4UUIDString())
 {
@@ -70,7 +64,7 @@ RefPtr<Uint8Array> CDMSessionClearKey::generateKeyRequest(const String& mimeType
     m_initData = initData;
 
     bool sawError = false;
-    String keyID = PAL::UTF8Encoding().decode(static_cast<char*>(m_initData->baseAddress()), m_initData->byteLength(), true, sawError);
+    String keyID = PAL::UTF8Encoding().decode(initData->span(), true, sawError);
     if (sawError) {
         errorCode = WebKitMediaKeyError::MEDIA_KEYERR_CLIENT;
         return nullptr;
@@ -84,38 +78,32 @@ void CDMSessionClearKey::releaseKeys()
     m_cachedKeys.clear();
 }
 
-bool CDMSessionClearKey::update(Uint8Array* rawKeysData, RefPtr<Uint8Array>& nextMessage, unsigned short& errorCode, uint32_t& systemCode)
+bool CDMSessionClearKey::update(JSC::Uint8Array* rawKeysData, RefPtr<JSC::Uint8Array>& nextMessage, unsigned short& errorCode, uint32_t& systemCode)
 {
     UNUSED_PARAM(nextMessage);
     UNUSED_PARAM(systemCode);
     ASSERT(rawKeysData);
 
     do {
-        auto rawKeysString = String::fromUTF8(rawKeysData->data(), rawKeysData->length());
+        auto rawKeysString = String::fromUTF8(rawKeysData->span());
         if (rawKeysString.isEmpty())  {
             LOG(Media, "CDMSessionClearKey::update(%p) - failed: empty message", this);
-            continue;
+            break;
         }
 
-        auto& vm = clearKeyVM();
-        JSLockHolder lock(vm);
-        auto scope = DECLARE_THROW_SCOPE(vm);
-        auto* globalObject = JSGlobalObject::create(vm, JSGlobalObject::createStructure(vm, jsNull()));
-        auto& lexicalGlobalObject = *globalObject;
-
-        auto keysDataValue = JSONParse(&lexicalGlobalObject, rawKeysString);
-        if (scope.exception() || !keysDataValue.isObject()) {
+        auto keysDataValue = JSON::Value::parseJSON(rawKeysString);
+        if (!keysDataValue || !keysDataValue->asObject()) {
             LOG(Media, "CDMSessionClearKey::update(%p) - failed: invalid JSON", this);
             break;
         }
 
-        auto keysArrayValue = asObject(keysDataValue)->get(&lexicalGlobalObject, Identifier::fromString(vm, "keys"));
-        if (scope.exception() || !isJSArray(keysArrayValue)) {
+        auto keysDataObject = keysDataValue->asObject();
+        auto keysArray = keysDataObject->getArray("keys"_s);
+        if (!keysArray) {
             LOG(Media, "CDMSessionClearKey::update(%p) - failed: keys array missing or empty", this);
             break;
         }
 
-        auto keysArray = asArray(keysArrayValue);
         auto length = keysArray->length();
         if (!length) {
             LOG(Media, "CDMSessionClearKey::update(%p) - failed: keys array missing or empty", this);
@@ -124,46 +112,39 @@ bool CDMSessionClearKey::update(Uint8Array* rawKeysData, RefPtr<Uint8Array>& nex
 
         bool foundValidKey = false;
         for (unsigned i = 0; i < length; ++i) {
-            auto keyValue = keysArray->getIndex(&lexicalGlobalObject, i);
+            auto keyObject = keysArray->get(i)->asObject();
 
-            if (scope.exception() || !keyValue.isObject()) {
+            if (!keyObject) {
                 LOG(Media, "CDMSessionClearKey::update(%p) - failed: null keyDictionary", this);
                 continue;
             }
 
-            auto keyObject = asObject(keyValue);
-
-            auto getStringProperty = [&scope, &lexicalGlobalObject, &keyObject, &vm](const char* name) -> String {
-                auto value = keyObject->get(&lexicalGlobalObject, Identifier::fromString(vm, name));
-                if (scope.exception() || !value.isString())
+            auto getStringProperty = [&keyObject](ASCIILiteral name) -> String {
+                String string;
+                if (!keyObject->getString(name, string))
                     return { };
-
-                auto string = asString(value)->value(&lexicalGlobalObject);
-                if (scope.exception())
-                    return { };
-                
                 return string;
             };
 
-            auto algorithm = getStringProperty("alg");
-            if (!equalLettersIgnoringASCIICase(algorithm, "a128kw")) {
+            auto algorithm = getStringProperty("alg"_s);
+            if (!equalLettersIgnoringASCIICase(algorithm, "a128kw"_s)) {
                 LOG(Media, "CDMSessionClearKey::update(%p) - failed: algorithm unsupported", this);
                 continue;
             }
 
-            auto keyType = getStringProperty("kty");
-            if (!equalLettersIgnoringASCIICase(keyType, "oct")) {
+            auto keyType = getStringProperty("kty"_s);
+            if (!equalLettersIgnoringASCIICase(keyType, "oct"_s)) {
                 LOG(Media, "CDMSessionClearKey::update(%p) - failed: keyType unsupported", this);
                 continue;
             }
 
-            auto keyId = getStringProperty("kid");
+            auto keyId = getStringProperty("kid"_s);
             if (keyId.isEmpty()) {
                 LOG(Media, "CDMSessionClearKey::update(%p) - failed: keyId missing or empty", this);
                 continue;
             }
 
-            auto rawKeyData = getStringProperty("k");
+            auto rawKeyData = getStringProperty("k"_s);
             if (rawKeyData.isEmpty())  {
                 LOG(Media, "CDMSessionClearKey::update(%p) - failed: key missing or empty", this);
                 continue;
@@ -188,13 +169,13 @@ bool CDMSessionClearKey::update(Uint8Array* rawKeysData, RefPtr<Uint8Array>& nex
     return false;
 }
 
-RefPtr<ArrayBuffer> CDMSessionClearKey::cachedKeyForKeyID(const String& keyId) const
+RefPtr<JSC::ArrayBuffer> CDMSessionClearKey::cachedKeyForKeyID(const String& keyId) const
 {
     if (!m_cachedKeys.contains(keyId))
         return nullptr;
 
     auto keyData = m_cachedKeys.get(keyId);
-    auto keyDataArray = Uint8Array::create(keyData.data(), keyData.size());
+    auto keyDataArray = JSC::Uint8Array::create(keyData.span());
     return keyDataArray->unsharedBuffer();
 }
 
