@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2009, 2010, 2011 Apple Inc. All Rights Reserved.
+ * Copyright (C) 2008, 2009, 2010, 2011 Apple Inc. All rights reserved.
  * Copyright (C) 2009 Torch Mobile, Inc.
  * Copyright 2010, The Android Open Source Project
  *
@@ -31,21 +31,25 @@
 #if ENABLE(GEOLOCATION)
 
 #include "Document.h"
+#include "DocumentInlines.h"
 #include "EventLoop.h"
-#include "FeaturePolicy.h"
-#include "Frame.h"
 #include "GeoNotifier.h"
 #include "GeolocationController.h"
 #include "GeolocationCoordinates.h"
 #include "GeolocationError.h"
 #include "GeolocationPosition.h"
 #include "GeolocationPositionData.h"
+#include "LocalFrame.h"
 #include "Navigator.h"
 #include "Page.h"
-#include "RuntimeApplicationChecks.h"
+#include "PermissionsPolicy.h"
+#include "ScriptExecutionContextInlines.h"
 #include "SecurityOrigin.h"
-#include <wtf/IsoMallocInlines.h>
+#include <JavaScriptCore/ConsoleTypes.h>
 #include <wtf/Ref.h>
+#include <wtf/RuntimeApplicationChecks.h>
+#include <wtf/TZoneMallocInlines.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
@@ -55,7 +59,7 @@ static const ASCIILiteral failedToStartServiceErrorMessage { "Failed to start Ge
 static const ASCIILiteral framelessDocumentErrorMessage { "Geolocation cannot be used in frameless documents"_s };
 static const ASCIILiteral originCannotRequestGeolocationErrorMessage { "Origin does not have permission to use Geolocation service"_s };
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(Geolocation);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(Geolocation);
 
 static RefPtr<GeolocationPosition> createGeolocationPosition(std::optional<GeolocationPositionData>&& position)
 {
@@ -156,9 +160,15 @@ SecurityOrigin* Geolocation::securityOrigin() const
     return scriptExecutionContext()->securityOrigin();
 }
 
+RefPtr<SecurityOrigin> Geolocation::protectedSecurityOrigin() const
+{
+    return securityOrigin();
+}
+
 Page* Geolocation::page() const
 {
-    return document() ? document()->page() : nullptr;
+    RefPtr document = this->document();
+    return document ? document->page() : nullptr;
 }
     
 void Geolocation::suspend(ReasonForSuspension reason)
@@ -227,7 +237,7 @@ void Geolocation::resumeTimerFired()
     }
 
     if (m_errorWaitingForResume) {
-        handleError(*m_errorWaitingForResume);
+        handleError(m_errorWaitingForResume.releaseNonNull());
         m_errorWaitingForResume = nullptr;
     }
 }
@@ -240,12 +250,12 @@ void Geolocation::resetAllGeolocationPermission()
     }
 
     if (m_allowGeolocation == InProgress) {
-        Page* page = this->page();
+        RefPtr page = this->page();
         if (page)
-            GeolocationController::from(page)->cancelPermissionRequest(*this);
+            GeolocationController::from(page.get())->cancelPermissionRequest(*this);
 
         // This return is not technically correct as GeolocationController::cancelPermissionRequest() should have cleared the active request.
-        // Neither iOS nor OS X supports cancelPermissionRequest() (https://bugs.webkit.org/show_bug.cgi?id=89524), so we workaround that and let ongoing requests complete. :(
+        // Neither iOS nor macOS supports cancelPermissionRequest() (https://bugs.webkit.org/show_bug.cgi?id=89524), so we workaround that and let ongoing requests complete. :(
         return;
     }
 
@@ -268,11 +278,16 @@ void Geolocation::resetAllGeolocationPermission()
         startRequest(watcher.get());
 }
 
+Document* Geolocation::document() const
+{
+    return downcast<Document>(scriptExecutionContext());
+}
+
 void Geolocation::stop()
 {
-    Page* page = this->page();
+    RefPtr page = this->page();
     if (page && m_allowGeolocation == InProgress)
-        GeolocationController::from(page)->cancelPermissionRequest(*this);
+        GeolocationController::from(page.get())->cancelPermissionRequest(*this);
     // The frame may be moving to a new page and we want to get the permissions from the new page's client.
     resetIsAllowed();
     cancelAllRequests();
@@ -282,28 +297,27 @@ void Geolocation::stop()
     m_pendingForPermissionNotifiers.clear();
 }
 
-const char* Geolocation::activeDOMObjectName() const
-{
-    return "Geolocation";
-}
-
 GeolocationPosition* Geolocation::lastPosition()
 {
-    Page* page = this->page();
+    RefPtr page = this->page();
     if (!page)
         return nullptr;
 
-    m_lastPosition = createGeolocationPosition(GeolocationController::from(page)->lastPosition());
+    m_lastPosition = createGeolocationPosition(GeolocationController::from(page.get())->lastPosition());
 
     return m_lastPosition.get();
 }
 
 void Geolocation::getCurrentPosition(Ref<PositionCallback>&& successCallback, RefPtr<PositionErrorCallback>&& errorCallback, PositionOptions&& options)
 {
-    if (!document() || !document()->isFullyActive()) {
-        if (errorCallback && errorCallback->scriptExecutionContext()) {
-            errorCallback->scriptExecutionContext()->eventLoop().queueTask(TaskSource::Geolocation, [errorCallback] {
-                errorCallback->handleEvent(GeolocationPositionError::create(GeolocationPositionError::POSITION_UNAVAILABLE, "Document is not fully active"_s));
+    RefPtr document = this->document();
+    if (!document || !document->isFullyActive()) {
+        if (!errorCallback)
+            return;
+
+        if (RefPtr context = errorCallback->scriptExecutionContext()) {
+            context->checkedEventLoop()->queueTask(TaskSource::Geolocation, [errorCallback = WTFMove(errorCallback)] {
+                errorCallback->invoke(GeolocationPositionError::create(GeolocationPositionError::POSITION_UNAVAILABLE, "Document is not fully active"_s));
             });
         }
         return;
@@ -317,10 +331,14 @@ void Geolocation::getCurrentPosition(Ref<PositionCallback>&& successCallback, Re
 
 int Geolocation::watchPosition(Ref<PositionCallback>&& successCallback, RefPtr<PositionErrorCallback>&& errorCallback, PositionOptions&& options)
 {
-    if (!document() || !document()->isFullyActive()) {
-        if (errorCallback && errorCallback->scriptExecutionContext()) {
-            errorCallback->scriptExecutionContext()->eventLoop().queueTask(TaskSource::Geolocation, [errorCallback] {
-                errorCallback->handleEvent(GeolocationPositionError::create(GeolocationPositionError::POSITION_UNAVAILABLE, "Document is not fully active"_s));
+    RefPtr document = this->document();
+    if (!document || !document->isFullyActive()) {
+        if (!errorCallback)
+            return 0;
+
+        if (RefPtr context = errorCallback->scriptExecutionContext()) {
+            context->checkedEventLoop()->queueTask(TaskSource::Geolocation, [errorCallback = WTFMove(errorCallback)] {
+                errorCallback->invoke(GeolocationPositionError::create(GeolocationPositionError::POSITION_UNAVAILABLE, "Document is not fully active"_s));
             });
         }
         return 0;
@@ -332,51 +350,37 @@ int Geolocation::watchPosition(Ref<PositionCallback>&& successCallback, RefPtr<P
     int watchID;
     // Keep asking for the next id until we're given one that we don't already have.
     do {
-        watchID = scriptExecutionContext()->circularSequentialID();
+        watchID = protectedScriptExecutionContext()->circularSequentialID();
     } while (!m_watchers.add(watchID, notifier.copyRef()));
     return watchID;
 }
 
 static void logError(const String& target, const bool isSecure, const bool isMixedContent, Document* document)
 {
-    StringBuilder message;
-    message.append("[blocked] Access to geolocation was blocked over");
-    
-    if (!isSecure)
-        message.append(" insecure connection to ");
-    else if (isMixedContent)
-        message.append(" secure connection with mixed content to ");
-    else
+    if (isSecure && !isMixedContent)
         return;
-    
-    message.append(target);
-    message.append(".\n");
-    document->addConsoleMessage(MessageSource::Security, MessageLevel::Error, message.toString());
-}
 
-// FIXME: remove this function when rdar://problem/32137821 is fixed.
-static bool isRequestFromIBooks()
-{
-#if PLATFORM(COCOA)
-    return CocoaApplication::isIBooks();
-#endif
-    return false;
+    auto message = makeString("[blocked] Access to geolocation was blocked over"_s,
+        isSecure ? " secure connection with mixed content to "_s : " insecure connection to "_s,
+        target, ".\n"_s);
+    document->addConsoleMessage(MessageSource::Security, MessageLevel::Error, WTFMove(message));
 }
     
 bool Geolocation::shouldBlockGeolocationRequests()
 {
-    if (!isFeaturePolicyAllowedByDocumentAndAllOwners(FeaturePolicy::Type::Geolocation, *document(), LogFeaturePolicyFailure::Yes))
+    RefPtr document = this->document();
+    if (!PermissionsPolicy::isFeatureEnabled(PermissionsPolicy::Feature::Geolocation, *document))
         return true;
 
-    bool isSecure = SecurityOrigin::isSecure(document()->url()) || document()->isSecureContext();
-    bool hasMixedContent = !document()->foundMixedContent().isEmpty();
+    bool isSecure = SecurityOrigin::isSecure(document->url()) || document->isSecureContext();
+    bool hasMixedContent = !document->foundMixedContent().isEmpty();
     bool isLocalOrigin = securityOrigin()->isLocal();
-    if (document()->canAccessResource(ScriptExecutionContext::ResourceType::Geolocation) != ScriptExecutionContext::HasResourceAccess::No) {
-        if (isLocalOrigin || (isSecure && !hasMixedContent) || isRequestFromIBooks())
+    if (document->canAccessResource(ScriptExecutionContext::ResourceType::Geolocation) != ScriptExecutionContext::HasResourceAccess::No) {
+        if (isLocalOrigin || (isSecure && !hasMixedContent))
             return false;
     }
 
-    logError(securityOrigin()->toString(), isSecure, hasMixedContent, document());
+    logError(protectedSecurityOrigin()->toString(), isSecure, hasMixedContent, document.get());
     return true;
 }
 
@@ -442,10 +446,11 @@ void Geolocation::makeCachedPositionCallbacks()
     // All modifications to m_requestsAwaitingCachedPosition are done
     // asynchronously, so we don't need to worry about it being modified from
     // the callbacks.
+    RefPtr lastPosition = this->lastPosition();
     for (auto& notifier : m_requestsAwaitingCachedPosition) {
         // FIXME: This seems wrong, since makeCachedPositionCallbacks() is called in a branch where
         // lastPosition() is known to be null in Geolocation::setIsAllowed().
-        notifier->runSuccessCallback(lastPosition());
+        notifier->runSuccessCallback(lastPosition.get());
 
         // If this is a one-shot request, stop it. Otherwise, if the watch still
         // exists, start the service to get updates.
@@ -474,7 +479,7 @@ void Geolocation::requestTimedOut(GeoNotifier* notifier)
 
 bool Geolocation::haveSuitableCachedPosition(const PositionOptions& options)
 {
-    auto* cachedPosition = lastPosition();
+    RefPtr cachedPosition = lastPosition();
     if (!cachedPosition)
         return false;
     if (!options.maximumAge)
@@ -488,8 +493,8 @@ void Geolocation::clearWatch(int watchID)
     if (watchID <= 0)
         return;
 
-    if (GeoNotifier* notifier = m_watchers.find(watchID))
-        m_pendingForPermissionNotifiers.remove(notifier);
+    if (RefPtr notifier = m_watchers.find(watchID))
+        m_pendingForPermissionNotifiers.remove(notifier.get());
     m_watchers.remove(watchID);
     
     if (!hasListeners())
@@ -644,14 +649,15 @@ void Geolocation::requestPermission()
     if (m_allowGeolocation > Unknown)
         return;
 
-    Page* page = this->page();
+    RefPtr page = this->page();
     if (!page)
         return;
 
     m_allowGeolocation = InProgress;
+    m_hasBeenRequested = true;
 
     // Ask the embedder: it maintains the geolocation challenge policy itself.
-    GeolocationController::from(page)->requestPermission(*this);
+    GeolocationController::from(page.get())->requestPermission(*this);
 }
 
 void Geolocation::revokeAuthorizationTokenIfNecessary()
@@ -659,17 +665,18 @@ void Geolocation::revokeAuthorizationTokenIfNecessary()
     if (m_authorizationToken.isNull())
         return;
 
-    Page* page = this->page();
+    RefPtr page = this->page();
     if (!page)
         return;
 
-    GeolocationController::from(page)->revokeAuthorizationToken(std::exchange(m_authorizationToken, String()));
+    GeolocationController::from(page.get())->revokeAuthorizationToken(std::exchange(m_authorizationToken, String()));
 }
 
 void Geolocation::resetIsAllowed()
 {
     m_allowGeolocation = Unknown;
     revokeAuthorizationTokenIfNecessary();
+    m_hasBeenRequested = false;
 }
 
 void Geolocation::makeSuccessCallbacks(GeolocationPosition& position)
@@ -723,21 +730,21 @@ void Geolocation::setError(GeolocationError& error)
 
 bool Geolocation::startUpdating(GeoNotifier* notifier)
 {
-    Page* page = this->page();
+    RefPtr page = this->page();
     if (!page)
         return false;
 
-    GeolocationController::from(page)->addObserver(*this, notifier->options().enableHighAccuracy);
+    GeolocationController::from(page.get())->addObserver(*this, notifier->options().enableHighAccuracy);
     return true;
 }
 
 void Geolocation::stopUpdating()
 {
-    Page* page = this->page();
+    RefPtr page = this->page();
     if (!page)
         return;
 
-    GeolocationController::from(page)->removeObserver(*this);
+    GeolocationController::from(page.get())->removeObserver(*this);
 }
 
 void Geolocation::handlePendingPermissionNotifiers()
@@ -762,7 +769,7 @@ Navigator* Geolocation::navigator()
     return m_navigator.get();
 }
 
-Frame* Geolocation::frame() const
+LocalFrame* Geolocation::frame() const
 {
     return m_navigator ? m_navigator->frame() : nullptr;
 }
