@@ -45,7 +45,7 @@ IDBConnectionToClient::IDBConnectionToClient(IDBConnectionToClientDelegate& dele
 IDBConnectionIdentifier IDBConnectionToClient::identifier() const
 {
     ASSERT(m_delegate);
-    return m_delegate->identifier();
+    return *m_delegate->identifier();
 }
 
 void IDBConnectionToClient::didDeleteDatabase(const IDBResultData& result)
@@ -162,6 +162,12 @@ void IDBConnectionToClient::fireVersionChangeEvent(UniqueIDBDatabaseConnection&
         m_delegate->fireVersionChangeEvent(connection, requestIdentifier, requestedVersion);
 }
 
+void IDBConnectionToClient::generateIndexKeyForRecord(const IDBResourceIdentifier& requestIdentifier, const IDBIndexInfo& indexInfo, const std::optional<IDBKeyPath>& keyPath, const IDBKeyData& key, const IDBValue& value, std::optional<int64_t> recordID)
+{
+    if (m_delegate)
+        m_delegate->generateIndexKeyForRecord(requestIdentifier, indexInfo, keyPath, key, value, recordID);
+}
+
 void IDBConnectionToClient::didStartTransaction(const IDBResourceIdentifier& transactionIdentifier, const IDBError& error)
 {
     if (m_delegate)
@@ -202,8 +208,8 @@ void IDBConnectionToClient::connectionToClientClosed()
     m_isClosed = true;
     auto databaseConnections = m_databaseConnections;
 
-    for (auto* connection : databaseConnections) {
-        ASSERT(m_databaseConnections.contains(connection));
+    for (RefPtr connection : databaseConnections) {
+        ASSERT(m_databaseConnections.contains(connection.get()));
         connection->connectionClosedFromClient();
     }
 
