@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 Apple Inc. All Rights Reserved.
+ * Copyright (C) 2016 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -23,13 +23,59 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+const maxNonLiveDuration = 604800; // 604800 seconds == 1 week
+
 class MediaController
 {
+    static EmptyRanges = {
+        get length() { return 0; },
+        start: function(index) { return undefined; },
+        end: function(index) { return undefined; },
+    }
+
+    static NullMedia = {
+        get audioTracks() { return []; },
+        get autoplay() { return false; },
+        get buffered() { return EmptyRanges; },
+        get controls() { return false; },
+        get currentTime() { return 0; },
+        set currentTime(time) { },
+        get defaultPlaybackRate() { return 1; },
+        set defaultPlaybackRate(rate) { },
+        get duration() { return 0; },
+        get error() { return null; },
+        get muted() { return false; },
+        set muted(muted) { },
+        get networkState() { return HTMLMediaElement.NETWORK_NO_SOURCE; },
+        get paused() { return false; },
+        get playbackRate() { return 1; },
+        set playbackRate(rate) { },
+        get played() { return EmptyRanges; },
+        get readyState() { return HTMLMediaElement.HAVE_NOTHING; },
+        get seekable() { return EmptyRanges; },
+        get textTracks() { return []; },
+        get videoTracks() { return []; },
+        get volume() { return 1; },
+        set volume(volume) { },
+        get webkitCurrentPlaybackTargetIsWireless() { return false; },
+        get webkitPresentationMode() { return "inline"; },
+        get webkitSupportsFullscreen() { return false; },
+
+        pause: function() { },
+        play: function() { },
+        fastSeek: function(time) { },
+        requestPictureInPicture: function() { },
+        webkitEnterFullscreen: function() { },
+        webkitExitFullscreen: function() { },
+        webkitSetPresentationMode: function(mode) { },
+        webkitShowPlaybackTargetPicker: function() { },
+        webkitSupportsPresentationMode: function(mode) { return false; },
+    };
 
     constructor(shadowRoot, media, host)
     {
-        this.shadowRoot = shadowRoot;
-        this.media = media;
+        this.shadowRootWeakRef = new WeakRef(shadowRoot);
+        this.mediaWeakRef = new WeakRef(media);
         this.host = host;
 
         this.fullscreenChangeEventType = media.webkitSupportsPresentationMode ? "webkitpresentationmodechanged" : "webkitfullscreenchange";
@@ -52,12 +98,16 @@ class MediaController
         scheduler.flushScheduledLayoutCallbacks();
 
         shadowRoot.addEventListener("resize", this);
+        shadowRoot.addEventListener("fullscreenchange", this);
 
         media.videoTracks.addEventListener("addtrack", this);
         media.videoTracks.addEventListener("removetrack", this);
 
         media.addEventListener("play", this);
         media.addEventListener(this.fullscreenChangeEventType, this);
+        media.addEventListener("keydown", this);
+        media.addEventListener("keyup", this);
+        media.addEventListener("click", this);
 
         window.addEventListener("keydown", this);
 
@@ -65,6 +115,16 @@ class MediaController
     }
 
     // Public
+    get media()
+    {
+        return this.mediaWeakRef.deref() ?? MediaController.NullMedia;
+    }
+
+    get shadowRoot()
+    {
+
+        return this.shadowRootWeakRef ? this.shadowRootWeakRef.deref() : null;
+    }
 
     get isAudio()
     {
@@ -91,7 +151,10 @@ class MediaController
 
     get isFullscreen()
     {
-        return this.media.webkitSupportsPresentationMode ? this.media.webkitPresentationMode === "fullscreen" : this.media.webkitDisplayingFullscreen;
+        if (!this.media)
+            return false;
+
+        return this.media.webkitSupportsPresentationMode ? this.media.webkitPresentationMode === "fullscreen" || (this.host && this.host.inWindowFullscreen) : this.media.webkitDisplayingFullscreen;
     }
 
     get layoutTraits()
@@ -170,7 +233,7 @@ class MediaController
     macOSControlsBackgroundWasClicked()
     {
         // Toggle playback when clicking on the video but not on any controls on macOS.
-        if (this.media.controls)
+        if (this.media.controls || (this.host && this.host.shouldForceControlsDisplay))
             this.togglePlayback();
     }
 
@@ -194,33 +257,121 @@ class MediaController
             this._updateControlsIfNeeded();
             // We must immediately perform layouts so that we don't lag behind the media layout size.
             scheduler.flushScheduledLayoutCallbacks();
-        } else if (event.currentTarget === this.media) {
-            if (event.type === "play")
-                this.hasPlayed = true;
-            this._updateControlsIfNeeded();
+        } else if (event.type === "fullscreenchange" && event.currentTarget === this.shadowRoot) {
             this._updateControlsAvailability();
         } else if (event.type === "keydown" && this.isFullscreen && event.key === " ") {
             this.togglePlayback();
             event.preventDefault();
+            event.stopPropagation();
+        } else if (event.type === "keyup" && this.isFullscreen && event.key === " ") {
+            event.preventDefault();
+            event.stopPropagation();
+        } else if (event.type === "dragstart" && this.isFullscreen)
+            event.preventDefault();
+        else if (event.type === this.fullscreenChangeEventType)
+            this.host?.presentationModeChanged?.();
+        else if (event.type === "click" && event.target === this.media) {
+            // If the <video> receives a click event, and the <video> is also the target
+            // of this event, this means we have clicked outside the border of the
+            // media controls <div>, which covers the video content.
+            if (this.host && this.host.inWindowFullscreen) {
+                this.media.webkitExitFullscreen();
+                event.stopPropagation();
+            }
+        }
+
+        if (event.currentTarget === this.media) {
+            if (event.type === "play")
+                this.hasPlayed = true;
+            this._updateControlsIfNeeded();
+            this._updateControlsAvailability();
         }
     }
 
-    // Private
+    // HTMLMediaElement
 
-    _supportingObjectClasses()
+    setShowingStats(shouldShowStats)
     {
-        let overridenSupportingObjectClasses = this.layoutTraits.overridenSupportingObjectClasses();
-        if (overridenSupportingObjectClasses)
-            return overridenSupportingObjectClasses;
+        if (!(this.media instanceof HTMLVideoElement))
+            return false;
 
-        let classes = [AudioSupport, CloseSupport, ControlsVisibilitySupport, FullscreenSupport, MuteSupport, OverflowSupport, PiPSupport, PlacardSupport, PlaybackSupport, ScrubbingSupport, SeekBackwardSupport, SeekForwardSupport, SkipBackSupport, SkipForwardSupport, StartSupport, StatusSupport, TimeControlSupport, TracksSupport, VolumeSupport];
+        if (!shouldShowStats) {
+            this._statsContainer?.remove();
+            this._statsContainer = null;
+            return false;
+        }
 
-        if (this.layoutTraits.supportsAirPlay())
-            classes.push(AirplaySupport);
+        if (this._statsContainer)
+            return true;
 
-        return classes;
+        this._statsContainer = this.container.appendChild(document.createElement("div"))
+        this._statsContainer.className = "stats-container";
+
+        let table = this._statsContainer.appendChild(document.createElement("table"));
+
+        function createRow(label) {
+            let rowElement = table.appendChild(document.createElement("tr"));
+
+            let labelElement = rowElement.appendChild(document.createElement("th"));
+            labelElement.textContent = label;
+
+            let valueElement = rowElement.appendChild(document.createElement("td"));
+            return valueElement;
+        }
+        let sourceValueElement = createRow(UIString("Source"));
+        let viewportValueElement = createRow(UIString("Viewport"));
+        let framesValueElement = createRow(UIString("Frames"));
+        let resolutionValueElement = createRow(UIString("Resolution"));
+        let codecsValueElement = createRow(UIString("Codecs"));
+        let colorValueElement = createRow(UIString("Color"));
+
+        let update = () => {
+            if (!this._statsContainer)
+                return;
+
+            let quality = this.media.getVideoPlaybackQuality();
+            let videoTrack = this.media.videoTracks.item(this.media.videoTracks.selectedIndex);
+            let videoTrackConfiguration = videoTrack.configuration;
+            let videoColorSpace = videoTrackConfiguration?.colorSpace;
+
+            sourceValueElement.textContent = UIString(this.host.sourceType ?? "none");
+            viewportValueElement.textContent = UIString("%s\u00d7%s", this.controls.width, this.controls.height) + (window.devicePixelRatio !== 1 ? " " + UIString("(%s)", UIString("%s\u00d7", window.devicePixelRatio)) : "");
+            framesValueElement.textContent = UIString("%s dropped of %s", quality.droppedVideoFrames, quality.totalVideoFrames);
+            resolutionValueElement.textContent = UIString("%s\u00d7%s", videoTrackConfiguration?.width, videoTrackConfiguration?.height) + " " + UIString("(%s)", UIString("%sfps", Math.round(videoTrackConfiguration?.framerate * 1000) / 1000));
+            codecsValueElement.textContent = videoTrackConfiguration?.codec;
+            colorValueElement.textContent = UIString("%s / %s / %s", videoColorSpace?.primaries, videoColorSpace?.transfer, videoColorSpace?.matrix);
+
+            window.requestAnimationFrame(update);
+        };
+        update();
+
+        return true;
     }
 
+    deinitialize()
+    {
+        this.shadowRoot.removeChild(this.container);
+        window.removeEventListener("keydown", this);
+        if (this.controls)
+            this.controls.disable();
+        return true;
+    }
+
+    reinitialize(shadowRoot, media, host)
+    {
+        iconService.shadowRoot = shadowRoot;
+        this.shadowRootWeakRef = new WeakRef(shadowRoot);
+        this.mediaWeakRef = new WeakRef(media);
+        this.host = host;
+        shadowRoot.appendChild(this.container);
+        window.addEventListener("keydown", this);
+        if (this.controls)
+            this.controls.reenable();
+        return true;
+    }
+
+    // Private
+
     _updateControlsIfNeeded()
     {
         const layoutTraits = this.layoutTraits;
@@ -239,6 +390,9 @@ class MediaController
                 supportingObject.disable();
         }
 
+        if (previousControls)
+            previousControls.disable();
+
         this.controls = new ControlsClass;
         this.controls.delegate = this;
 
@@ -255,11 +409,35 @@ class MediaController
         this._updateTextTracksClassList();
         this._updateControlsSize();
 
-        this._supportingObjects = this._supportingObjectClasses().map(SupportClass => new SupportClass(this), this);
+        this._supportingObjects = layoutTraits.supportingObjectClasses().map(SupportClass => new SupportClass(this), this);
 
         this.controls.shouldUseSingleBarLayout = this.controls instanceof InlineMediaControls && this.isYouTubeEmbedWithTitle;
 
+        if (this.controls instanceof MacOSFullscreenMediaControls)
+            window.addEventListener("dragstart", this, true);
+        else
+            window.removeEventListener("dragstart", this, true);
+
+        if (this.host && this.host.inWindowFullscreen) {
+            this._stopPropagationOnClickEvents();
+            if (!this.host.supportsRewind)
+                this.controls.rewindButton.dropped = true;
+        }
+
         this._updateControlsAvailability();
+
+        if (this.host?.needsChromeMediaControlsPseudoElement)
+            this.controls.element.setAttribute('useragentpart', '-webkit-media-controls');
+    }
+
+    _stopPropagationOnClickEvents()
+    {
+        let clickEvents = ["click", "mousedown", "mouseup", "pointerdown", "pointerup"];
+        for (let clickEvent of clickEvents) {
+            this.controls.element.addEventListener(clickEvent, (event) => {
+                event.stopPropagation();
+            });
+        }
     }
 
     _updateControlsSize()
@@ -307,8 +485,6 @@ class MediaController
         // Finally, we factor in the scale factor of the controls themselves, which reflects the page's scale factor.
         this.controls.width = Math.round((maxX - minX) * this.controls.scaleFactor);
         this.controls.height = Math.round((maxY - minY) * this.controls.scaleFactor);
-
-        this.controls.shouldCenterControlsVertically = this.isAudio;
     }
 
     _updateTextTracksClassList()
@@ -332,11 +508,11 @@ class MediaController
 
     _shouldControlsBeAvailable()
     {
-        if (this.layoutTraits.controlsAlwaysAvailable())
-            return true;
-
         if (this.layoutTraits.controlsNeverAvailable())
             return false;
+        
+        if (this.controls instanceof IOSInlineMediaControls && this.media.webkitPresentationMode === "picture-in-picture")
+            return false;
 
         // Otherwise, for controls to be available, the controls attribute must be present on the media element
         // or the MediaControlsHost must indicate that controls are forced.
