@@ -51,7 +51,7 @@ PushManager& ServiceWorkerRegistrationPushAPI::pushManager(ServiceWorkerRegistra
 PushManager& ServiceWorkerRegistrationPushAPI::pushManager()
 {
     if (!m_pushManager)
-        m_pushManager = makeUniqueWithoutRefCountedCheck<PushManager>(m_serviceWorkerRegistration);
+        lazyInitialize(m_pushManager, makeUniqueWithoutRefCountedCheck<PushManager>(m_serviceWorkerRegistration));
 
     return *m_pushManager;
 }
