@@ -30,8 +30,10 @@
 #include "DatabaseTask.h"
 
 #include "Database.h"
+#include "ExceptionOr.h"
 #include "Logging.h"
 #include "SQLTransaction.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
@@ -53,6 +55,8 @@ void DatabaseTaskSynchronizer::taskCompleted()
     m_synchronousCondition.notifyOne();
 }
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(DatabaseTask);
+
 DatabaseTask::DatabaseTask(Database& database, DatabaseTaskSynchronizer* synchronizer)
     : m_database(database)
     , m_synchronizer(synchronizer)
@@ -69,7 +73,7 @@ void DatabaseTask::performTask()
     // Database tasks are meant to be used only once, so make sure this one hasn't been performed before.
     ASSERT(!m_complete);
 
-    LOG(StorageAPI, "Performing %s %p\n", debugTaskName(), this);
+    LOG(StorageAPI, "Performing %s %p\n", debugTaskName().characters(), this);
 
     m_database.resetAuthorizer();
 
@@ -95,14 +99,14 @@ DatabaseOpenTask::DatabaseOpenTask(Database& database, bool setVersionInNewDatab
 
 void DatabaseOpenTask::doPerformTask()
 {
-    m_result = isolatedCopy(database().performOpenAndVerify(m_setVersionInNewDatabase));
+    m_result = crossThreadCopy(database().performOpenAndVerify(m_setVersionInNewDatabase));
 }
 
 #if !LOG_DISABLED
 
-const char* DatabaseOpenTask::debugTaskName() const
+ASCIILiteral DatabaseOpenTask::debugTaskName() const
 {
-    return "DatabaseOpenTask";
+    return "DatabaseOpenTask"_s;
 }
 
 #endif
@@ -122,9 +126,9 @@ void DatabaseCloseTask::doPerformTask()
 
 #if !LOG_DISABLED
 
-const char* DatabaseCloseTask::debugTaskName() const
+ASCIILiteral DatabaseCloseTask::debugTaskName() const
 {
-    return "DatabaseCloseTask";
+    return "DatabaseCloseTask"_s;
 }
 
 #endif
@@ -161,9 +165,9 @@ void DatabaseTransactionTask::doPerformTask()
 
 #if !LOG_DISABLED
 
-const char* DatabaseTransactionTask::debugTaskName() const
+ASCIILiteral DatabaseTransactionTask::debugTaskName() const
 {
-    return "DatabaseTransactionTask";
+    return "DatabaseTransactionTask"_s;
 }
 
 #endif
@@ -185,9 +189,9 @@ void DatabaseTableNamesTask::doPerformTask()
 
 #if !LOG_DISABLED
 
-const char* DatabaseTableNamesTask::debugTaskName() const
+ASCIILiteral DatabaseTableNamesTask::debugTaskName() const
 {
-    return "DatabaseTableNamesTask";
+    return "DatabaseTableNamesTask"_s;
 }
 
 #endif
