@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023 Apple Inc. All rights reserved.
+ * Copyright (C) 2023-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,18 +29,17 @@
 #include "AnimationTimelinesController.h"
 #include "CSSNumericFactory.h"
 #include "CSSPropertyParserConsumer+Timeline.h"
+#include "CSSValuePair.h"
 #include "Document.h"
 #include "Element.h"
 #include "LegacyRenderSVGModelObject.h"
 #include "RenderBlock.h"
-#include "RenderBox.h"
-#include "RenderBoxInlines.h"
-#include "RenderInline.h"
+#include "RenderBoxModelObject.h"
+#include "RenderElementInlines.h"
 #include "RenderLayerScrollableArea.h"
 #include "RenderSVGModelObject.h"
 #include "ScrollAnchoringController.h"
 #include "ScrollingConstraints.h"
-#include "StyleBuilderConverter.h"
 #include "StyleScrollPadding.h"
 #include "WebAnimation.h"
 
@@ -72,9 +71,9 @@ ExceptionOr<Ref<ViewTimeline>> ViewTimeline::create(Document& document, ViewTime
     return viewTimeline;
 }
 
-Ref<ViewTimeline> ViewTimeline::create(const AtomString& name, ScrollAxis axis, ViewTimelineInsets&& insets)
+Ref<ViewTimeline> ViewTimeline::create(const AtomString& name, ScrollAxis axis, const ViewTimelineInsetItem& insetItem)
 {
-    return adoptRef(*new ViewTimeline(name, axis, WTFMove(insets)));
+    return adoptRef(*new ViewTimeline(name, axis, insetItem));
 }
 
 ViewTimeline::ViewTimeline(ScrollAxis axis)
@@ -82,9 +81,9 @@ ViewTimeline::ViewTimeline(ScrollAxis axis)
 {
 }
 
-ViewTimeline::ViewTimeline(const AtomString& name, ScrollAxis axis, ViewTimelineInsets&& insets)
+ViewTimeline::ViewTimeline(const AtomString& name, ScrollAxis axis, const ViewTimelineInsetItem& insetItem)
     : ScrollTimeline(name, axis)
-    , m_insets(WTFMove(insets))
+    , m_insets(insetItem)
 {
 }
 
@@ -99,17 +98,14 @@ ExceptionOr<ViewTimeline::SpecifiedViewTimelineInsets> ViewTimeline::validateSpe
     if (auto* insetString = std::get_if<String>(&inset)) {
         if (insetString->isEmpty())
             return Exception { ExceptionCode::TypeError };
-        CSSTokenizer tokenizer(*insetString);
-        auto tokenRange = tokenizer.tokenRange();
-        tokenRange.consumeWhitespace();
-        auto consumedInset = CSSPropertyParserHelpers::consumeViewTimelineInsetListItem(tokenRange, Ref { document }->cssParserContext());
+        auto consumedInset = CSSPropertyParserHelpers::parseSingleViewTimelineInsetItem(*insetString, Ref { document }->cssParserContext());
         if (!consumedInset)
             return Exception { ExceptionCode::TypeError };
 
-        if (auto insetPair = dynamicDowncast<CSSValuePair>(consumedInset)) {
+        if (RefPtr insetPair = dynamicDowncast<CSSValuePair>(consumedInset)) {
             return { {
-                dynamicDowncast<CSSPrimitiveValue>(insetPair->protectedFirst()),
-                dynamicDowncast<CSSPrimitiveValue>(insetPair->protectedSecond())
+                RefPtr { dynamicDowncast<CSSPrimitiveValue>(insetPair->first()) },
+                RefPtr { dynamicDowncast<CSSPrimitiveValue>(insetPair->second()) }
             } };
         } else
             return { { dynamicDowncast<CSSPrimitiveValue>(consumedInset), nullptr } };
@@ -305,32 +301,27 @@ void ViewTimeline::cacheCurrentTime()
             return { };
 
         CheckedPtr sourceRenderer = sourceScrollerRenderer();
-        auto* sourceScrollableArea = scrollableAreaForSourceRenderer(sourceRenderer.get(), subject->element.document());
+        CheckedPtr sourceScrollableArea = scrollableAreaForSourceRenderer(sourceRenderer.get(), subject->element.document());
         if (!sourceScrollableArea)
             return { };
 
         auto scrollDirection = resolvedScrollDirection();
-        if (!scrollDirection)
-            return { };
-
-        float scrollOffset = scrollDirection->isVertical ? sourceScrollableArea->scrollOffset().y() : sourceScrollableArea->scrollOffset().x();
-        float scrollContainerSize = scrollDirection->isVertical ? sourceScrollableArea->visibleHeight() : sourceScrollableArea->visibleWidth();
+        float scrollOffset = scrollDirection.isVertical ? sourceScrollableArea->scrollOffset().y() : sourceScrollableArea->scrollOffset().x();
+        float scrollContainerSize = scrollDirection.isVertical ? sourceScrollableArea->visibleHeight() : sourceScrollableArea->visibleWidth();
 
         // https://drafts.csswg.org/scroll-animations-1/#view-timelines-ranges
         // Transforms and sticky position offsets are ignored, but relative and absolute positioning are accounted for.
         OptionSet<MapCoordinatesMode> options { IgnoreStickyOffsets };
         auto subjectOffsetFromSource = subjectRenderer->localToContainerPoint(pointForLocalToContainer(*sourceScrollableArea), sourceRenderer.get(), options);
-        float subjectOffset = scrollDirection->isVertical ? subjectOffsetFromSource.y() : subjectOffsetFromSource.x();
+        float subjectOffset = scrollDirection.isVertical ? subjectOffsetFromSource.y() : subjectOffsetFromSource.x();
 
         // Ensure borders are subtracted.
         auto scrollerPaddingBoxOrigin = sourceRenderer->paddingBoxRect().location();
-        subjectOffset -= scrollDirection->isVertical ? scrollerPaddingBoxOrigin.y() : scrollerPaddingBoxOrigin.x();
+        subjectOffset -= scrollDirection.isVertical ? scrollerPaddingBoxOrigin.y() : scrollerPaddingBoxOrigin.x();
 
         auto subjectBounds = [&] -> FloatSize {
-            if (CheckedPtr subjectRenderBox = dynamicDowncast<RenderBox>(subjectRenderer.get()))
-                return subjectRenderBox->contentBoxRect().size();
-            if (CheckedPtr subjectRenderInline = dynamicDowncast<RenderInline>(subjectRenderer.get()))
-                return subjectRenderInline->borderBoundingBox().size();
+            if (CheckedPtr subjectRenderBoxModelObject = dynamicDowncast<RenderBoxModelObject>(subjectRenderer.get()))
+                return subjectRenderBoxModelObject->borderBoundingBox().size();
             if (CheckedPtr subjectRenderSVGModelObject = dynamicDowncast<RenderSVGModelObject>(subjectRenderer.get()))
                 return subjectRenderSVGModelObject->borderBoxRectEquivalent().size();
             if (is<LegacyRenderSVGModelObject>(subjectRenderer.get()))
@@ -338,7 +329,7 @@ void ViewTimeline::cacheCurrentTime()
             return { };
         }();
 
-        auto subjectSize = scrollDirection->isVertical ? subjectBounds.height() : subjectBounds.width();
+        auto subjectSize = scrollDirection.isVertical ? subjectBounds.height() : subjectBounds.width();
 
         if (m_specifiedInsets) {
             RefPtr subjectElement { &subject->element };
@@ -354,8 +345,8 @@ void ViewTimeline::cacheCurrentTime()
         auto scrollPadding = [&](PaddingEdge edge) {
             auto& style = sourceRenderer->style();
             if (edge == PaddingEdge::Start)
-                return scrollDirection->isVertical ? style.scrollPaddingTop() : style.scrollPaddingLeft();
-            return scrollDirection->isVertical ? style.scrollPaddingBottom() : style.scrollPaddingRight();
+                return scrollDirection.isVertical ? style.scrollPaddingTop() : style.scrollPaddingLeft();
+            return scrollDirection.isVertical ? style.scrollPaddingBottom() : style.scrollPaddingRight();
         };
 
         bool hasInsetsStart = m_insets.start.has_value();
@@ -382,7 +373,7 @@ void ViewTimeline::cacheCurrentTime()
                 insetEnd = insetStart; 
             }
         } else if (hasInsetsEnd) {
-            insetStart = Style::evaluate(scrollPadding(PaddingEdge::Start), scrollContainerSize);\
+            insetStart = Style::evaluate(scrollPadding(PaddingEdge::Start), scrollContainerSize);
 
             if (m_insets.end->isAuto())
                 insetEnd = Style::evaluate(scrollPadding(PaddingEdge::End), scrollContainerSize);
@@ -398,7 +389,7 @@ void ViewTimeline::cacheCurrentTime()
             FloatRect constrainingRect = stickyContainer->constrainingRectForStickyPosition();
             StickyPositionViewportConstraints constraints;
             stickyContainer->computeStickyPositionConstraints(constraints, constrainingRect);
-            stickyData = StickinessAdjustmentData::computeStickinessAdjustmentData(constraints, *scrollDirection, scrollContainerSize, subjectSize, subjectOffset);
+            stickyData = StickinessAdjustmentData::computeStickinessAdjustmentData(constraints, scrollDirection, scrollContainerSize, subjectSize, subjectOffset);
         }
 
         return {
@@ -438,6 +429,13 @@ TimelineRange ViewTimeline::defaultRange() const
     return TimelineRange::defaultForViewTimeline();
 }
 
+Element* ViewTimeline::bindingsSource() const
+{
+    if (auto subject = m_subject.styleable())
+        subject->element.protectedDocument()->updateStyleIfNeeded();
+    return ScrollTimeline::bindingsSource();
+}
+
 Element* ViewTimeline::source() const
 {
     if (CheckedPtr sourceRender = sourceScrollerRenderer())
@@ -495,44 +493,56 @@ ScrollTimeline::Data ViewTimeline::computeTimelineData() const
 
 std::pair<double, double> ViewTimeline::intervalForTimelineRangeName(const ScrollTimeline::Data& data, const SingleTimelineRange::Name name) const
 {
-    auto subjectRangeStart = [&]() {
+    auto subjectRangeStart = [&]() -> double {
         switch (name) {
         case SingleTimelineRange::Name::Normal:
         case SingleTimelineRange::Name::Omitted:
         case SingleTimelineRange::Name::Cover:
-        case SingleTimelineRange::Name::Entry:
         case SingleTimelineRange::Name::EntryCrossing:
             return data.rangeStart;
+        case SingleTimelineRange::Name::Entry:
+            // https://drafts.csswg.org/scroll-animations-1/#valdef-animation-timeline-range-entry
+            // 0% is equivalent to 0% of the cover range.
+            return intervalForTimelineRangeName(data, SingleTimelineRange::Name::Cover).first;
         case SingleTimelineRange::Name::Contain:
             return data.rangeStart + m_cachedCurrentTimeData.subjectSize + m_cachedCurrentTimeData.stickinessData.entryDistanceAdjustment();
         case SingleTimelineRange::Name::Exit:
+            // https://drafts.csswg.org/scroll-animations-1/#valdef-animation-timeline-range-exit
+            // 0% is equivalent to 100% of the contain range.
+            return intervalForTimelineRangeName(data, SingleTimelineRange::Name::Contain).second;
         case SingleTimelineRange::Name::ExitCrossing:
             return data.rangeEnd - m_cachedCurrentTimeData.subjectSize - m_cachedCurrentTimeData.stickinessData.exitDistanceAdjustment();
         default:
             break;
         }
         ASSERT_NOT_REACHED();
-        return 0.f;
+        return 0.0;
     }();
 
-    auto subjectRangeEnd = [&]() {
+    auto subjectRangeEnd = [&]() -> double {
         switch (name) {
         case SingleTimelineRange::Name::Normal:
         case SingleTimelineRange::Name::Omitted:
         case SingleTimelineRange::Name::Cover:
-        case SingleTimelineRange::Name::Exit:
         case SingleTimelineRange::Name::ExitCrossing:
             return data.rangeEnd;
+        case SingleTimelineRange::Name::Exit:
+            // https://drafts.csswg.org/scroll-animations-1/#valdef-animation-timeline-range-exit
+            // 100% is equivalent to 100% of the cover range.
+            return intervalForTimelineRangeName(data, SingleTimelineRange::Name::Cover).second;
         case SingleTimelineRange::Name::Contain:
             return data.rangeEnd - m_cachedCurrentTimeData.subjectSize - m_cachedCurrentTimeData.stickinessData.exitDistanceAdjustment();
         case SingleTimelineRange::Name::Entry:
+            // https://drafts.csswg.org/scroll-animations-1/#valdef-animation-timeline-range-entry
+            // 100% is equivalent to 0% of the contain range.
+            return intervalForTimelineRangeName(data, SingleTimelineRange::Name::Contain).first;
         case SingleTimelineRange::Name::EntryCrossing:
             return data.rangeStart + m_cachedCurrentTimeData.subjectSize + m_cachedCurrentTimeData.stickinessData.entryDistanceAdjustment();
         default:
             break;
         }
         ASSERT_NOT_REACHED();
-        return 0.f;
+        return 0.0;
     }();
 
     if (subjectRangeEnd < subjectRangeStart)
@@ -614,20 +624,25 @@ Ref<CSSNumericValue> ViewTimeline::endOffset() const
 
 WTF::TextStream& operator<<(WTF::TextStream& ts, const StickinessAdjustmentData& stickiness)
 {
-    ts << "[ TopOrLeftAdjustment: " << stickiness.stickyTopOrLeftAdjustment << ", TopOrLeftLocation: " << stickiness.topOrLeftAdjustmentLocation << ", BottomOrRightAdjustment: " << stickiness.stickyBottomOrRightAdjustment << ", BottomOrRightLocation: " << stickiness.bottomOrRightAdjustmentLocation << " ]";
+    ts << "[ TopOrLeftAdjustment: "_s << stickiness.stickyTopOrLeftAdjustment << ", TopOrLeftLocation: "_s << stickiness.topOrLeftAdjustmentLocation << ", BottomOrRightAdjustment: "_s << stickiness.stickyBottomOrRightAdjustment << ", BottomOrRightLocation: "_s << stickiness.bottomOrRightAdjustmentLocation << " ]"_s;
     return ts;
 }
 
 WTF::TextStream& operator<<(WTF::TextStream& ts, const StickinessAdjustmentData::StickinessLocation& stickiness)
 {
     switch (stickiness) {
-    case StickinessAdjustmentData::StickinessLocation::BeforeEntry: ts << "BeforeEntry"; break;
-    case StickinessAdjustmentData::StickinessLocation::DuringEntry: ts << "DuringEntry"; break;
-    case StickinessAdjustmentData::StickinessLocation::WhileContained: ts << "WhileContained"; break;
-    case StickinessAdjustmentData::StickinessLocation::DuringExit: ts << "DuringExit"; break;
-    case StickinessAdjustmentData::StickinessLocation::AfterExit: ts << "AfterExit"; break;
+    case StickinessAdjustmentData::StickinessLocation::BeforeEntry: ts << "BeforeEntry"_s; break;
+    case StickinessAdjustmentData::StickinessLocation::DuringEntry: ts << "DuringEntry"_s; break;
+    case StickinessAdjustmentData::StickinessLocation::WhileContained: ts << "WhileContained"_s; break;
+    case StickinessAdjustmentData::StickinessLocation::DuringExit: ts << "DuringExit"_s; break;
+    case StickinessAdjustmentData::StickinessLocation::AfterExit: ts << "AfterExit"_s; break;
     }
     return ts;
 }
 
+TextStream& operator<<(TextStream& ts, const ViewTimeline& timeline)
+{
+    return ts << timeline.name() << ' ' << timeline.axis() << ' ' << timeline.insets();
+}
+
 } // namespace WebCore
