@@ -63,6 +63,43 @@ public:
     }
 
     StringView source() const final
+    {
+        return sourceImpl(Locker { m_lock });
+    }
+
+    void clearDecodedData() final
+    {
+        Locker locker { m_lock };
+        m_cachedScriptString = String();
+    }
+
+    void tryReplaceScriptBuffer(const ScriptBuffer& scriptBuffer) final
+    {
+        // If this new file-mapped script buffer is identical to the one we have, then replace
+        // ours to save dirty memory.
+        Locker locker { m_lock };
+        if (m_scriptBuffer != scriptBuffer)
+            return;
+
+        m_scriptBuffer = scriptBuffer;
+        m_contiguousBuffer = nullptr;
+    }
+
+    JSC::CodeBlockHash codeBlockHashConcurrently(int startOffset, int endOffset, JSC::CodeSpecializationKind kind) override
+    {
+        Locker locker { m_lock };
+        auto view = sourceImpl(locker);
+        return JSC::CodeBlockHash { view.substring(startOffset, endOffset - startOffset), view, kind };
+    }
+
+private:
+    ScriptBufferSourceProvider(const ScriptBuffer& scriptBuffer, const JSC::SourceOrigin& sourceOrigin, String&& sourceURL, String&& preRedirectURL, const TextPosition& startPosition, JSC::SourceProviderSourceType sourceType)
+        : JSC::SourceProvider(sourceOrigin, WTFMove(sourceURL), WTFMove(preRedirectURL), JSC::SourceTaintedOrigin::Untainted, startPosition, sourceType)
+        , m_scriptBuffer(scriptBuffer)
+    {
+    }
+
+    StringView sourceImpl(const AbstractLocker&) const
     {
         if (m_scriptBuffer.isEmpty())
             return emptyString();
@@ -86,34 +123,12 @@ public:
         return m_cachedScriptString;
     }
 
-    void clearDecodedData() final
-    {
-        m_cachedScriptString = String();
-    }
-
-    void tryReplaceScriptBuffer(const ScriptBuffer& scriptBuffer) final
-    {
-        // If this new file-mapped script buffer is identical to the one we have, then replace
-        // ours to save dirty memory.
-        if (m_scriptBuffer != scriptBuffer)
-            return;
-
-        m_scriptBuffer = scriptBuffer;
-        m_contiguousBuffer = nullptr;
-    }
-
-private:
-    ScriptBufferSourceProvider(const ScriptBuffer& scriptBuffer, const JSC::SourceOrigin& sourceOrigin, String&& sourceURL, String&& preRedirectURL, const TextPosition& startPosition, JSC::SourceProviderSourceType sourceType)
-        : JSC::SourceProvider(sourceOrigin, WTFMove(sourceURL), WTFMove(preRedirectURL), JSC::SourceTaintedOrigin::Untainted, startPosition, sourceType)
-        , m_scriptBuffer(scriptBuffer)
-    {
-    }
-
     ScriptBuffer m_scriptBuffer;
     mutable RefPtr<SharedBuffer> m_contiguousBuffer;
     mutable unsigned m_scriptHash { 0 };
     mutable String m_cachedScriptString;
     mutable std::optional<bool> m_containsOnlyASCII;
+    mutable Lock m_lock;
 };
 
 } // namespace WebCore
