@@ -29,6 +29,7 @@
 #include "CSSPrimitiveNumericTypes.h"
 #include "CSSPropertyParserConsumer+MetaConsumerDefinitions.h"
 #include "CSSPropertyParserOptions.h"
+#include "CSSPropertyParserState.h"
 #include "StylePrimitiveNumericTypes.h"
 #include <optional>
 #include <type_traits>
@@ -46,7 +47,7 @@ namespace CSSPropertyParserHelpers {
 
 /// The result of a meta consume.
 /// To be used with a list of `CSS` types (e.g. `ConsumeResult<CSS::Angle<Range>, CSS::Percentage<Range>, CSS::Keyword::None>`), which will yield a
-/// result type of either a std::variant of those types (e.g.`std::variant<CSS::Angle<Range>, CSS::Percentage<Range>, CSS::Keyword::None>`) or the type
+/// result type of either a Variant of those types (e.g.`Variant<CSS::Angle<Range>, CSS::Percentage<Range>, CSS::Keyword::None>`) or the type
 /// itself if only a single type was specified.
 template<typename... Ts>
 struct MetaConsumeResult {
@@ -118,13 +119,13 @@ struct MetaConsumerDispatcher<IdentToken, Consumer, typename std::void_t<typenam
 template<typename... Ts>
 struct MetaConsumerUnroller {
     template<CSSParserTokenType, typename ResultType>
-    static std::nullopt_t consume(CSSParserTokenRange&, const CSSParserContext&, CSSCalcSymbolsAllowed, CSSPropertyParserOptions)
+    static std::nullopt_t consume(CSSParserTokenRange&, CSS::PropertyParserState&, CSSCalcSymbolsAllowed, CSSPropertyParserOptions)
     {
         return std::nullopt;
     }
 
     template<CSSParserTokenType, typename ResultType, typename F>
-    static std::nullopt_t consume(CSSParserTokenRange&, const CSSParserContext&, CSSCalcSymbolsAllowed, CSSPropertyParserOptions, NOESCAPE F&&)
+    static std::nullopt_t consume(CSSParserTokenRange&, CSS::PropertyParserState&, CSSCalcSymbolsAllowed, CSSPropertyParserOptions, NOESCAPE F&&)
     {
         return std::nullopt;
     }
@@ -136,25 +137,25 @@ struct MetaConsumerUnroller {
 template<typename T, typename... Ts>
 struct MetaConsumerUnroller<T, Ts...> {
     template<CSSParserTokenType tokenType, typename ResultType>
-    static std::optional<ResultType> consume(CSSParserTokenRange& range, const CSSParserContext& context, CSSCalcSymbolsAllowed symbolsAllowed, CSSPropertyParserOptions options)
+    static std::optional<ResultType> consume(CSSParserTokenRange& range, CSS::PropertyParserState& state, CSSCalcSymbolsAllowed symbolsAllowed, CSSPropertyParserOptions options)
     {
         using Consumer = MetaConsumerDispatcher<tokenType, ConsumerDefinition<T>>;
         if constexpr (Consumer::supported) {
-            if (auto result = Consumer::consume(range, context, symbolsAllowed, options))
+            if (auto result = Consumer::consume(range, state, symbolsAllowed, options))
                 return { T { *result } };
         }
-        return MetaConsumerUnroller<Ts...>::template consume<tokenType, ResultType>(range, context, symbolsAllowed, options);
+        return MetaConsumerUnroller<Ts...>::template consume<tokenType, ResultType>(range, state, symbolsAllowed, options);
     }
 
     template<CSSParserTokenType tokenType, typename ResultType, typename F>
-    static std::optional<ResultType> consume(CSSParserTokenRange& range, const CSSParserContext& context, CSSCalcSymbolsAllowed symbolsAllowed, CSSPropertyParserOptions options, NOESCAPE F&& functor)
+    static std::optional<ResultType> consume(CSSParserTokenRange& range, CSS::PropertyParserState& state, CSSCalcSymbolsAllowed symbolsAllowed, CSSPropertyParserOptions options, NOESCAPE F&& functor)
     {
         using Consumer = MetaConsumerDispatcher<tokenType, ConsumerDefinition<T>>;
         if constexpr (Consumer::supported) {
-            if (auto result = Consumer::consume(range, context, symbolsAllowed, options))
+            if (auto result = Consumer::consume(range, state, symbolsAllowed, options))
                 return std::make_optional(functor(T { *result }));
         }
-        return MetaConsumerUnroller<Ts...>::template consume<tokenType, ResultType>(range, context, symbolsAllowed, options, std::forward<F>(functor));
+        return MetaConsumerUnroller<Ts...>::template consume<tokenType, ResultType>(range, state, symbolsAllowed, options, std::forward<F>(functor));
     }
 };
 
@@ -180,37 +181,59 @@ struct MetaConsumer {
     using Unroller = MetaConsumerUnroller<T, Ts...>;
 
     template<typename... F>
-    static decltype(auto) consume(CSSParserTokenRange& range, const CSSParserContext& context, CSSCalcSymbolsAllowed symbolsAllowed, CSSPropertyParserOptions options, F&&... f)
+    static decltype(auto) consume(CSSParserTokenRange& range, CSS::PropertyParserState& state, CSSCalcSymbolsAllowed symbolsAllowed, CSSPropertyParserOptions options, F&&... f)
     {
         auto visitor = WTF::makeVisitor(std::forward<F>(f)...);
         using ResultType = decltype(visitor(std::declval<T>()));
 
         switch (range.peek().type()) {
         case FunctionToken:
-            return Unroller::template consume<FunctionToken, ResultType>(range, context, WTFMove(symbolsAllowed), options, visitor);
+            return Unroller::template consume<FunctionToken, ResultType>(range, state, WTFMove(symbolsAllowed), options, visitor);
 
         case NumberToken:
-            return Unroller::template consume<NumberToken, ResultType>(range, context, WTFMove(symbolsAllowed), options, visitor);
+            return Unroller::template consume<NumberToken, ResultType>(range, state, WTFMove(symbolsAllowed), options, visitor);
 
         case PercentageToken:
-            return Unroller::template consume<PercentageToken, ResultType>(range, context, WTFMove(symbolsAllowed), options, visitor);
+            return Unroller::template consume<PercentageToken, ResultType>(range, state, WTFMove(symbolsAllowed), options, visitor);
 
         case DimensionToken:
-            return Unroller::template consume<DimensionToken, ResultType>(range, context, WTFMove(symbolsAllowed), options, visitor);
+            return Unroller::template consume<DimensionToken, ResultType>(range, state, WTFMove(symbolsAllowed), options, visitor);
 
         case IdentToken:
-            return Unroller::template consume<IdentToken, ResultType>(range, context, WTFMove(symbolsAllowed), options, visitor);
+            return Unroller::template consume<IdentToken, ResultType>(range, state, WTFMove(symbolsAllowed), options, visitor);
 
         default:
             return std::optional<ResultType> { };
         }
     }
 
-    static decltype(auto) consume(CSSParserTokenRange& range, const CSSParserContext& context, CSSCalcSymbolsAllowed symbolsAllowed, CSSPropertyParserOptions options)
+    // Overloaded with the `CSSPropertyParserOptions` parameter removed so it can be defaulted when using the continuation functor parameters.
+    template<typename... F>
+    static decltype(auto) consume(CSSParserTokenRange& range, CSS::PropertyParserState& state, CSSCalcSymbolsAllowed symbolsAllowed, F&&... f)
+    {
+        return consume(range, state, WTFMove(symbolsAllowed), { }, std::forward<F>(f)...);
+    }
+
+    // Overloaded with the `CSSCalcSymbolsAllowed` parameter removed so it can be defaulted when using the continuation functor parameters.
+    template<typename... F>
+    static decltype(auto) consume(CSSParserTokenRange& range, CSS::PropertyParserState& state, CSSPropertyParserOptions options, F&&... f)
+    {
+        return consume(range, state, { }, options, std::forward<F>(f)...);
+    }
+
+    // Overloaded with the `CSSPropertyParserOptions` and `CSSCalcSymbolsAllowed` parameters removed so they can be defaulted when using the continuation functor parameters.
+    template<typename... F>
+    static decltype(auto) consume(CSSParserTokenRange& range, CSS::PropertyParserState& state, F&&... f)
+    {
+        return consume(range, state, { }, { }, std::forward<F>(f)...);
+    }
+
+    // Overloaded with no continuation functor parameters allowing a for simplified interface when returning a single value / or Variant is acceptable.
+    static decltype(auto) consume(CSSParserTokenRange& range, CSS::PropertyParserState& state, CSSCalcSymbolsAllowed symbolsAllowed = { }, CSSPropertyParserOptions options = { })
     {
         using ResultType = typename MetaConsumeResult<T, Ts...>::type;
 
-        return consume(range, context, WTFMove(symbolsAllowed), options,
+        return consume(range, state, WTFMove(symbolsAllowed), options,
             [](auto&& value) {
                 return ResultType { WTFMove(value) };
             }
