@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005, 2006, 2008 Apple Inc. All rights reserved.
+ * Copyright (C) 2005-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -47,13 +47,12 @@ private:
     void executeApply();
 
     RefPtr<HTMLElement> protectedDummySpan() const { return m_dummySpan; }
-    Ref<Element> protectedElement() const { return m_element; }
 
 #ifndef NDEBUG
     void getNodesInCommand(NodeSet&) override;
 #endif
 
-    Ref<Element> m_element;
+    const Ref<Element> m_element;
     RefPtr<HTMLElement> m_dummySpan;
 };
 
