@@ -27,6 +27,7 @@
 #include "NetworkSendQueue.h"
 
 #include "BlobLoader.h"
+#include "ContextDestructionObserverInlines.h"
 #include "ScriptExecutionContext.h"
 
 namespace WebCore {
@@ -53,16 +54,15 @@ void NetworkSendQueue::enqueue(CString&& utf8)
 void NetworkSendQueue::enqueue(const JSC::ArrayBuffer& binaryData, unsigned byteOffset, unsigned byteLength)
 {
     if (m_queue.isEmpty()) {
-        auto* data = static_cast<const uint8_t*>(binaryData.data());
-        m_writeRawData(Span { data + byteOffset, byteLength });
+        m_writeRawData(binaryData.span().subspan(byteOffset, byteLength));
         return;
     }
-    m_queue.append(SharedBuffer::create(static_cast<const uint8_t*>(binaryData.data()) + byteOffset, byteLength));
+    m_queue.append(SharedBuffer::create(binaryData.span().subspan(byteOffset, byteLength)));
 }
 
 void NetworkSendQueue::enqueue(WebCore::Blob& blob)
 {
-    auto* context = scriptExecutionContext();
+    RefPtr context = scriptExecutionContext();
     if (!context)
         return;
 
@@ -78,14 +78,12 @@ void NetworkSendQueue::enqueue(WebCore::Blob& blob)
     });
     auto* blobLoaderPtr = &blobLoader.get();
     m_queue.append(WTFMove(blobLoader));
-    blobLoaderPtr->start(blob, context, FileReaderLoader::ReadAsArrayBuffer);
+    blobLoaderPtr->start(blob, context.get(), FileReaderLoader::ReadAsArrayBuffer);
 }
 
 void NetworkSendQueue::clear()
 {
-    // Do not call m_queue.clear() here since destroying a BlobLoader will cause its completion
-    // handler to get called, which will call processMessages() to iterate over m_queue.
-    std::exchange(m_queue, { });
+    m_queue.clear();
 }
 
 void NetworkSendQueue::processMessages()
@@ -98,13 +96,13 @@ void NetworkSendQueue::processMessages()
             data->forEachSegment(m_writeRawData);
         }, [this, &shouldStopProcessing](UniqueRef<BlobLoader>& loader) {
             auto errorCode = loader->errorCode();
-            if (loader->isLoading() || (errorCode && errorCode.value() == AbortError)) {
+            if (loader->isLoading() || (errorCode && errorCode.value() == ExceptionCode::AbortError)) {
                 shouldStopProcessing = true;
                 return;
             }
 
             if (const auto& result = loader->arrayBufferResult()) {
-                m_writeRawData(Span { static_cast<const uint8_t*>(result->data()), result->byteLength() });
+                m_writeRawData(result->span());
                 return;
             }
             ASSERT(errorCode);
