@@ -35,12 +35,12 @@ class HTMLButtonElement final : public HTMLFormControlElement {
 public:
     static Ref<HTMLButtonElement> create(const QualifiedName&, Document&, HTMLFormElement*);
     static Ref<HTMLButtonElement> create(Document&);
-
-    WEBCORE_EXPORT void setType(const AtomString&);
     
     const AtomString& value() const;
+    const AtomString& command() const;
 
     RefPtr<Element> commandForElement() const;
+    CommandType commandType() const;
 
     bool willRespondToMouseClickEventsWithEditability(Editability) const final;
 
@@ -65,7 +65,6 @@ private:
     bool hasPresentationalHintsForAttribute(const QualifiedName&) const final;
     void defaultEventHandler(Event&) final;
 
-    CommandType commandType() const;
     void handleCommand();
 
     bool appendFormData(DOMFormData&) final;
@@ -88,6 +87,8 @@ private:
 
     bool isSubmitButton() const final;
 
+    void computeType(const AtomString& typeAttrValue);
+
     Type m_type;
     bool m_isActivatedSubmit;
 };
