@@ -26,12 +26,13 @@ namespace WebCore {
 
 class HTMLImageLoader;
 
-enum class CreatePlugins { No, Yes };
+enum class CreatePlugins : bool { No, Yes };
 
 // Base class for HTMLEmbedElement and HTMLObjectElement.
 // FIXME: This is the only class that derives from HTMLPlugInElement, so we could merge the two classes.
 class HTMLPlugInImageElement : public HTMLPlugInElement {
-    WTF_MAKE_ISO_ALLOCATED(HTMLPlugInImageElement);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(HTMLPlugInImageElement);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(HTMLPlugInImageElement);
 public:
     virtual ~HTMLPlugInImageElement();
 
@@ -44,6 +45,8 @@ public:
 
     bool needsWidgetUpdate() const { return m_needsWidgetUpdate; }
     void setNeedsWidgetUpdate(bool needsWidgetUpdate) { m_needsWidgetUpdate = needsWidgetUpdate; }
+
+    bool shouldBypassCSPForPDFPlugin(const String& contentType) const;
     
 protected:
     HTMLPlugInImageElement(const QualifiedName& tagName, Document&);
@@ -67,14 +70,13 @@ protected:
 private:
     bool isPlugInImageElement() const final { return true; }
 
-    bool shouldBypassCSPForPDFPlugin(const String&) const;
     bool canLoadPlugInContent(const String& relativeURL, const String& mimeType) const;
     bool canLoadURL(const URL&) const;
 
     RenderPtr<RenderElement> createElementRenderer(RenderStyle&&, const RenderTreePosition&) override;
     bool childShouldCreateRenderer(const Node&) const override;
-    void willRecalcStyle(Style::Change) final;
-    void didRecalcStyle(Style::Change) final;
+    void willRecalcStyle(OptionSet<Style::Change>) final;
+    void didRecalcStyle(OptionSet<Style::Change>) final;
     void didAttachRenderers() final;
     void willDetachRenderers() final;
 
@@ -85,7 +87,7 @@ private:
 
     bool m_needsWidgetUpdate { false };
     bool m_needsDocumentActivationCallbacks { false };
-    std::unique_ptr<HTMLImageLoader> m_imageLoader;
+    const std::unique_ptr<HTMLImageLoader> m_imageLoader;
     bool m_needsImageReload { false };
     bool m_hasUpdateScheduledForAfterStyleResolution { false };
 };
@@ -94,5 +96,9 @@ private:
 
 SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::HTMLPlugInImageElement)
     static bool isType(const WebCore::HTMLPlugInElement& element) { return element.isPlugInImageElement(); }
-    static bool isType(const WebCore::Node& node) { return is<WebCore::HTMLPlugInElement>(node) && isType(downcast<WebCore::HTMLPlugInElement>(node)); }
+    static bool isType(const WebCore::Node& node)
+    {
+        auto* pluginElement = dynamicDowncast<WebCore::HTMLPlugInElement>(node);
+        return pluginElement && isType(*pluginElement);
+    }
 SPECIALIZE_TYPE_TRAITS_END()
