@@ -26,16 +26,17 @@
 #pragma once
 
 #include "WebGLExtension.h"
+#include "WebGLRenderingContextBase.h"
+#include <wtf/Noncopyable.h>
+#include <wtf/TZoneMalloc.h>
 
 namespace WebCore {
 
-class OESTextureFloat final : public WebGLExtension {
-    WTF_MAKE_ISO_ALLOCATED(OESTextureFloat);
+class OESTextureFloat final : public WebGLExtension<WebGLRenderingContextBase> {
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(OESTextureFloat);
 public:
     explicit OESTextureFloat(WebGLRenderingContextBase&);
-    virtual ~OESTextureFloat();
-
-    ExtensionName getName() const override;
+    ~OESTextureFloat();
 
     static bool supported(GraphicsContextGL&);
 };
