@@ -35,8 +35,9 @@
 #include "MHTMLArchive.h"
 
 #include "Document.h"
-#include "Frame.h"
 #include "LegacySchemeRegistry.h"
+#include "LocalFrame.h"
+#include "LocalFrameInlines.h"
 #include "MHTMLParser.h"
 #include "MIMETypeRegistry.h"
 #include "Page.h"
@@ -57,18 +58,18 @@
 
 namespace WebCore {
 
-const char* const quotedPrintable = "quoted-printable";
-const char* const base64 = "base64";
+constexpr auto quotedPrintable = "quoted-printable"_s;
+constexpr auto base64 = "base64"_s;
 
 static String generateRandomBoundary()
 {
     // Trying to generate random boundaries similar to IE/UnMHT (ex: ----=_NextPart_000_001B_01CC157B.96F808A0).
-    const size_t randomValuesLength = 10;
-    char randomValues[randomValuesLength];
-    cryptographicallyRandomValues(&randomValues, randomValuesLength);
+    constexpr size_t randomValuesLength = 10;
+    std::array<uint8_t, randomValuesLength> randomValues;
+    cryptographicallyRandomValues(randomValues);
     StringBuilder stringBuilder;
-    stringBuilder.append("----=_NextPart_000_");
-    for (size_t i = 0; i < randomValuesLength; ++i) {
+    stringBuilder.append("----=_NextPart_000_"_s);
+    for (size_t i = 0; i < randomValues.size(); ++i) {
         if (i == 2)
             stringBuilder.append('_');
         else if (i == 6)
@@ -137,74 +138,69 @@ Ref<FragmentedSharedBuffer> MHTMLArchive::generateMHTMLData(Page* page)
     pageSerializer.serialize(*page);
 
     String boundary = generateRandomBoundary();
-    String endOfResourceBoundary = makeString("--", boundary, "\r\n");
+    String endOfResourceBoundary = makeString("--"_s, boundary, "\r\n"_s);
 
     GregorianDateTime now;
     now.setToCurrentLocalTime();
     String dateString = makeRFC2822DateString(now.weekDay(), now.monthDay(), now.month(), now.year(), now.hour(), now.minute(), now.second(), now.utcOffsetInMinute());
 
     StringBuilder stringBuilder;
-    stringBuilder.append("From: <Saved by WebKit>\r\n");
-    stringBuilder.append("Subject: ");
-    // We replace non ASCII characters with '?' characters to match IE's behavior.
-    stringBuilder.append(replaceNonPrintableCharacters(page->mainFrame().document()->title()));
-    stringBuilder.append("\r\nDate: ");
-    stringBuilder.append(dateString);
-    stringBuilder.append("\r\nMIME-Version: 1.0\r\n");
-    stringBuilder.append("Content-Type: multipart/related;\r\n");
-    stringBuilder.append("\ttype=\"");
-    stringBuilder.append(page->mainFrame().document()->suggestedMIMEType());
-    stringBuilder.append("\";\r\n");
-    stringBuilder.append("\tboundary=\"");
-    stringBuilder.append(boundary);
-    stringBuilder.append("\"\r\n\r\n");
+    stringBuilder.append("From: <Saved by WebKit>\r\n"_s);
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(page->mainFrame());
+    if (localMainFrame) {
+        stringBuilder.append("Subject: "_s);
+        // We replace non ASCII characters with '?' characters to match IE's behavior.
+        stringBuilder.append(replaceNonPrintableCharacters(localMainFrame->document()->title()));
+    }
+    stringBuilder.append("\r\nDate: "_s, dateString,
+        "\r\nMIME-Version: 1.0\r\nContent-Type: multipart/related;\r\n"_s);
+    if (localMainFrame)
+        stringBuilder.append("\ttype=\""_s, localMainFrame->document()->suggestedMIMEType());
+    stringBuilder.append("\";\r\n\tboundary=\""_s, boundary, "\"\r\n\r\n"_s);
 
-    // We use utf8() below instead of ascii() as ascii() replaces CRLFs with ?? (we still only have put ASCII characters in it).
-    ASSERT(stringBuilder.toString().isAllASCII());
+    ASSERT(stringBuilder.toString().containsOnlyASCII());
     CString asciiString = stringBuilder.toString().utf8();
     SharedBufferBuilder mhtmlData;
-    mhtmlData.append(asciiString.data(), asciiString.length());
+    mhtmlData.append(asciiString.span());
 
     for (auto& resource : resources) {
         stringBuilder.clear();
-        stringBuilder.append(endOfResourceBoundary, "Content-Type: ", resource.mimeType);
+        stringBuilder.append(endOfResourceBoundary, "Content-Type: "_s, resource.mimeType);
 
-        const char* contentEncoding = nullptr;
+        ASCIILiteral contentEncoding;
         if (MIMETypeRegistry::isSupportedJavaScriptMIMEType(resource.mimeType) || MIMETypeRegistry::isSupportedNonImageMIMEType(resource.mimeType))
             contentEncoding = quotedPrintable;
         else
             contentEncoding = base64;
 
-        stringBuilder.append("\r\nContent-Transfer-Encoding: ", contentEncoding, "\r\nContent-Location: ", resource.url.string(), "\r\n\r\n");
+        stringBuilder.append("\r\nContent-Transfer-Encoding: "_s, contentEncoding, "\r\nContent-Location: "_s, resource.url.string(), "\r\n\r\n"_s);
 
         asciiString = stringBuilder.toString().utf8();
-        mhtmlData.append(asciiString.data(), asciiString.length());
+        mhtmlData.append(asciiString.span());
 
         // FIXME: ideally we would encode the content as a stream without having to fetch it all.
-        auto* data = resource.data->data();
-        size_t dataLength = resource.data->size();
-        if (!strcmp(contentEncoding, quotedPrintable)) {
-            auto encodedData = quotedPrintableEncode(data, dataLength);
-            mhtmlData.append(encodedData.data(), encodedData.size());
-            mhtmlData.append("\r\n", 2);
+        if (contentEncoding == quotedPrintable) {
+            auto encodedData = quotedPrintableEncode(resource.data->span());
+            mhtmlData.append(encodedData.span());
+            mhtmlData.append("\r\n"_span);
         } else {
-            ASSERT(!strcmp(contentEncoding, base64));
+            ASSERT(contentEncoding == base64);
             // We are not specifying insertLFs = true below as it would cut the lines with LFs and MHTML requires CRLFs.
-            auto encodedData = base64EncodeToVector(data, dataLength);
+            auto encodedData = base64EncodeToVector(resource.data->span());
             const size_t maximumLineLength = 76;
             size_t index = 0;
             size_t encodedDataLength = encodedData.size();
             do {
                 size_t lineLength = std::min(encodedDataLength - index, maximumLineLength);
-                mhtmlData.append(encodedData.data() + index, lineLength);
-                mhtmlData.append("\r\n", 2);
+                mhtmlData.append(encodedData.subspan(index, lineLength));
+                mhtmlData.append("\r\n"_span);
                 index += maximumLineLength;
             } while (index < encodedDataLength);
         }
     }
 
-    asciiString = makeString("--", boundary, "--\r\n").utf8();
-    mhtmlData.append(asciiString.data(), asciiString.length());
+    asciiString = makeString("--"_s, boundary, "--\r\n"_s).utf8();
+    mhtmlData.append(asciiString.span());
 
     return mhtmlData.take();
 }
