@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,6 +26,8 @@
 #include "config.h"
 #include "DiagnosticLoggingKeys.h"
 
+#include "HTMLMediaElement.h"
+
 namespace WebCore {
 
 String DiagnosticLoggingKeys::mediaLoadedKey()
@@ -53,16 +55,6 @@ String DiagnosticLoggingKeys::missingValidatorFieldsKey()
     return "missingValidatorFields"_s;
 }
 
-String DiagnosticLoggingKeys::pluginLoadedKey()
-{
-    return "pluginLoaded"_s;
-}
-
-String DiagnosticLoggingKeys::pluginLoadingFailedKey()
-{
-    return "pluginFailedLoading"_s;
-}
-
 String DiagnosticLoggingKeys::postPageBackgroundingCPUUsageKey()
 {
     return "postPageBackgroundingCPUUsage"_s;
@@ -73,11 +65,6 @@ String DiagnosticLoggingKeys::postPageBackgroundingMemoryUsageKey()
     return "postPageBackgroundingMemoryUsage"_s;
 }
 
-String DiagnosticLoggingKeys::pageHandlesWebGLContextLossKey()
-{
-    return "pageHandlesWebGLContextLoss"_s;
-}
-
 String DiagnosticLoggingKeys::postPageLoadCPUUsageKey()
 {
     return "postPageLoadCPUUsage"_s;
@@ -93,16 +80,6 @@ String DiagnosticLoggingKeys::provisionalLoadKey()
     return "provisionalLoad"_s;
 }
 
-String DiagnosticLoggingKeys::pageContainsPluginKey()
-{
-    return "pageContainsPlugin"_s;
-}
-
-String DiagnosticLoggingKeys::pageContainsAtLeastOnePluginKey()
-{
-    return "pageContainsAtLeastOnePlugin"_s;
-}
-
 String DiagnosticLoggingKeys::pageContainsMediaEngineKey()
 {
     return "pageContainsMediaEngine"_s;
@@ -188,26 +165,11 @@ String DiagnosticLoggingKeys::noKey()
     return "no"_s;
 }
 
-String DiagnosticLoggingKeys::noCacheKey()
-{
-    return "noCache"_s;
-}
-
-String DiagnosticLoggingKeys::noStoreKey()
-{
-    return "noStore"_s;
-}
-
 String DiagnosticLoggingKeys::nonVisibleStateKey()
 {
     return "nonVisibleState"_s;
 }
 
-String DiagnosticLoggingKeys::notInMemoryCacheKey()
-{
-    return "notInMemoryCache"_s;
-}
-
 String DiagnosticLoggingKeys::backForwardCacheKey()
 {
     return "backForwardCache"_s;
@@ -248,11 +210,6 @@ String DiagnosticLoggingKeys::isErrorPageKey()
     return "isErrorPage"_s;
 }
 
-String DiagnosticLoggingKeys::isExpiredKey()
-{
-    return "isExpired"_s;
-}
-
 String DiagnosticLoggingKeys::isReloadIgnoringCacheDataKey()
 {
     return "isReloadIgnoringCacheData"_s;
@@ -263,11 +220,6 @@ String DiagnosticLoggingKeys::loadingKey()
     return "loading"_s;
 }
 
-String DiagnosticLoggingKeys::hasPluginsKey()
-{
-    return "hasPlugins"_s;
-}
-
 String DiagnosticLoggingKeys::httpsNoStoreKey()
 {
     return "httpsNoStore"_s;
@@ -278,16 +230,6 @@ String DiagnosticLoggingKeys::imageKey()
     return "image"_s;
 }
 
-String DiagnosticLoggingKeys::inMemoryCacheKey()
-{
-    return "inMemoryCache"_s;
-}
-
-String DiagnosticLoggingKeys::inactiveKey()
-{
-    return "inactive"_s;
-}
-
 String DiagnosticLoggingKeys::internalErrorKey()
 {
     return "internalError"_s;
@@ -398,16 +340,6 @@ String DiagnosticLoggingKeys::createSharedBufferFailedKey()
     return "createSharedBufferFailed"_s;
 }
 
-String DiagnosticLoggingKeys::activeInForegroundTabKey()
-{
-    return "activeInForegroundTab"_s;
-}
-
-String DiagnosticLoggingKeys::activeInBackgroundTabOnlyKey()
-{
-    return "activeInBackgroundTabOnly"_s;
-}
-
 String DiagnosticLoggingKeys::applicationCacheKey()
 {
     return "applicationCache"_s;
@@ -475,6 +407,16 @@ String DiagnosticLoggingKeys::diskCacheAfterValidationKey()
     return "diskCacheAfterValidation"_s;
 }
 
+String DiagnosticLoggingKeys::memoryCacheKey()
+{
+    return "memoryCache"_s;
+}
+
+String DiagnosticLoggingKeys::memoryCacheAfterValidationKey()
+{
+    return "memoryCacheAfterValidation"_s;
+}
+
 String DiagnosticLoggingKeys::reloadKey()
 {
     return "reload"_s;
@@ -505,11 +447,6 @@ String DiagnosticLoggingKeys::retrievalKey()
     return "retrieval"_s;
 }
 
-String DiagnosticLoggingKeys::revalidatingKey()
-{
-    return "revalidating"_s;
-}
-
 String DiagnosticLoggingKeys::reloadFromOriginKey()
 {
     return "reloadFromOrigin"_s;
@@ -680,41 +617,6 @@ String DiagnosticLoggingKeys::unusedKey()
     return "unused"_s;
 }
 
-String DiagnosticLoggingKeys::unusedReasonCredentialSettingsKey()
-{
-    return "unused.reason.credentialSettings"_s;
-}
-
-String DiagnosticLoggingKeys::unusedReasonErrorKey()
-{
-    return "unused.reason.error"_s;
-}
-
-String DiagnosticLoggingKeys::unusedReasonMustRevalidateNoValidatorKey()
-{
-    return "unused.reason.mustRevalidateNoValidator"_s;
-}
-
-String DiagnosticLoggingKeys::unusedReasonNoStoreKey()
-{
-    return "unused.reason.noStore"_s;
-}
-
-String DiagnosticLoggingKeys::unusedReasonRedirectChainKey()
-{
-    return "unused.reason.redirectChain"_s;
-}
-
-String DiagnosticLoggingKeys::unusedReasonReloadKey()
-{
-    return "unused.reason.reload"_s;
-}
-
-String DiagnosticLoggingKeys::unusedReasonTypeMismatchKey()
-{
-    return "unused.reason.typeMismatch"_s;
-}
-
 String DiagnosticLoggingKeys::usedKey()
 {
     return "used"_s;
@@ -795,11 +697,6 @@ String WebCore::DiagnosticLoggingKeys::notHTTPFamilyKey()
     return "notHTTPFamily"_s;
 }
 
-String WebCore::DiagnosticLoggingKeys::webGLStateKey()
-{
-    return "webGLState"_s;
-}
-
 String DiagnosticLoggingKeys::memoryUsageToDiagnosticLoggingKey(uint64_t memoryUsage)
 {
     if (memoryUsage < 32 * MB)
@@ -859,10 +756,97 @@ String DiagnosticLoggingKeys::backgroundCPUUsageToDiagnosticLoggingKey(double cp
     return "over70"_s;
 }
 
-String DiagnosticLoggingKeys::resourceLoadStatisticsTelemetryKey()
+String DiagnosticLoggingKeys::mediaSourceTypeWatchTimeKey()
 {
-    return "resourceLoadStatisticsTelemetry"_s;
+    return "watchtimeBySource"_s;
 }
-    
+
+String DiagnosticLoggingKeys::mediaVideoCodecWatchTimeKey()
+{
+    return "watchtimeByVideoCodec"_s;
+}
+
+String DiagnosticLoggingKeys::mediaAudioCodecWatchTimeKey()
+{
+    return "watchtimeByAudioCodec"_s;
+}
+
+String DiagnosticLoggingKeys::mediaBufferingWatchTimeKey()
+{
+    return "watchtimeBuffering"_s;
+}
+
+String DiagnosticLoggingKeys::mediaTextTrackWatchTimeKey()
+{
+    return "watchtimeTextTrack"_s;
+}
+
+String DiagnosticLoggingKeys::secondsKey()
+{
+    return "seconds"_s;
+}
+
+String DiagnosticLoggingKeys::sourceTypeKey()
+{
+    return "sourceType"_s;
+}
+
+String DiagnosticLoggingKeys::videoCodecKey()
+{
+    return "videoCodec"_s;
+}
+
+String DiagnosticLoggingKeys::audioCodecKey()
+{
+    return "audioCodec"_s;
+}
+
+String DiagnosticLoggingKeys::textTrackTypeKey()
+{
+    return "textTrackType"_s;
+}
+
+String DiagnosticLoggingKeys::textTrackKindKey()
+{
+    return "textTrackKind"_s;
+}
+
+String DiagnosticLoggingKeys::textTrackModeKey()
+{
+    return "textTrackMode"_s;
+}
+
+String DiagnosticLoggingKeys::mediaPresentationTypeWatchTimeKey()
+{
+    return "watchtimeByPresentationType"_s;
+}
+
+String DiagnosticLoggingKeys::presentationTypeKey()
+{
+    return "presentationType"_s;
+}
+
+#if ENABLE(CONTENT_EXTENSIONS)
+String DiagnosticLoggingKeys::iframeResourceMonitoringKey()
+{
+    return "IFrameResourceMonitoring"_s;
+}
+
+String DiagnosticLoggingKeys::unloadCountKey()
+{
+    return "unloadCount"_s;
+}
+
+String DiagnosticLoggingKeys::unloadPreventedByThrottlerCountKey()
+{
+    return "unloadPreventedByThrottlerCount"_s;
+}
+
+String DiagnosticLoggingKeys::unloadPreventedByStickyActivationCountKey()
+{
+    return "unloadPreventedByStickyActivationCount"_s;
+}
+#endif
+
 } // namespace WebCore
 
