@@ -28,6 +28,7 @@
 #include "GradientColorStop.h"
 #include <algorithm>
 #include <optional>
+#include <ranges>
 #include <wtf/Forward.h>
 #include <wtf/Vector.h>
 
@@ -71,9 +72,7 @@ public:
         if (m_isSorted)
             return;
 
-        std::stable_sort(m_stops.begin(), m_stops.end(), [] (auto& a, auto& b) {
-            return a.offset < b.offset;
-        });
+        std::ranges::stable_sort(m_stops, { }, &GradientColorStop::offset);
         m_isSorted = true;
     }
 
@@ -86,8 +85,8 @@ public:
     size_t size() const { return m_stops.size(); }
     bool isEmpty() const { return m_stops.isEmpty(); }
 
-    StopVector::const_iterator begin() const { return m_stops.begin(); }
-    StopVector::const_iterator end() const { return m_stops.end(); }
+    StopVector::const_iterator begin() const LIFETIME_BOUND { return m_stops.begin(); }
+    StopVector::const_iterator end() const LIFETIME_BOUND { return m_stops.end(); }
 
     template<typename MapFunction> GradientColorStops mapColors(MapFunction&& mapFunction) const
     {
@@ -101,9 +100,6 @@ public:
 
     const StopVector& stops() const { return m_stops; }
 
-    template<typename Encoder> void encode(Encoder&) const;
-    template<typename Decoder> static std::optional<GradientColorStops> decode(Decoder&);
-
 private:
     GradientColorStops(StopVector stops, bool isSorted)
         : m_stops { WTFMove(stops) }
@@ -114,9 +110,7 @@ private:
 #if ASSERT_ENABLED
     bool validateIsSorted() const
     {
-        return std::is_sorted(m_stops.begin(), m_stops.end(), [] (auto& a, auto& b) {
-            return a.offset < b.offset;
-        });
+        return std::ranges::is_sorted(m_stops, { }, &GradientColorStop::offset);
     }
 #endif
 
@@ -124,21 +118,6 @@ private:
     bool m_isSorted;
 };
 
-template<typename Encoder> void GradientColorStops::encode(Encoder& encoder) const
-{
-    encoder << m_stops;
-}
-
-template<typename Decoder> std::optional<GradientColorStops> GradientColorStops::decode(Decoder& decoder)
-{
-    std::optional<StopVector> stops;
-    decoder >> stops;
-    if (!stops)
-        return std::nullopt;
-
-    return {{ WTFMove(*stops) }};
-}
-
 TextStream& operator<<(TextStream&, const GradientColorStops&);
 
 } // namespace WebCore
