@@ -32,34 +32,21 @@
 #include "GStreamerCommon.h"
 #include "TrackPrivateBase.h"
 #include <gst/tag/tag.h>
+#include <wtf/glib/GUniquePtr.h>
 #include <wtf/text/CString.h>
+#include <wtf/text/StringCommon.h>
+#include <wtf/text/StringToIntegerConversion.h>
 
 GST_DEBUG_CATEGORY_EXTERN(webkit_media_player_debug);
 #define GST_CAT_DEFAULT webkit_media_player_debug
 
 namespace WebCore {
 
-AtomString TrackPrivateBaseGStreamer::generateUniquePlaybin2StreamID(TrackType trackType, unsigned index)
-{
-    auto prefix = [trackType]() -> char {
-        switch (trackType) {
-        case TrackPrivateBaseGStreamer::TrackType::Audio:
-            return 'A';
-        case TrackPrivateBaseGStreamer::TrackType::Video:
-            return 'V';
-        case TrackPrivateBaseGStreamer::TrackType::Text:
-            return 'T';
-        default:
-            ASSERT_NOT_REACHED();
-            return 'U';
-        }
-    }();
-
-    return makeAtomString(prefix, index);
-}
-
 static GRefPtr<GstPad> findBestUpstreamPad(GRefPtr<GstPad> pad)
 {
+    if (!pad)
+        return nullptr;
+
     GRefPtr<GstPad> sinkPad = pad;
     GRefPtr<GstPad> peerSrcPad;
 
@@ -85,15 +72,32 @@ TrackPrivateBaseGStreamer::TrackPrivateBaseGStreamer(TrackType type, TrackPrivat
     tagsChanged();
 }
 
+TrackPrivateBaseGStreamer::TrackPrivateBaseGStreamer(TrackType type, TrackPrivateBase* owner, unsigned index, GRefPtr<GstPad>&& pad, TrackID trackId)
+    : m_notifier(MainThreadNotifier<MainThreadNotification>::create())
+    , m_index(index)
+    , m_id(trackId)
+    , m_type(type)
+    , m_owner(owner)
+    , m_shouldUsePadStreamId(false)
+    , m_shouldHandleStreamStartEvent(false)
+{
+    setPad(WTFMove(pad));
+    ASSERT(m_pad);
+
+    // We can't call notifyTrackOfTagsChanged() directly, because we need tagsChanged() to setup m_tags.
+    tagsChanged();
+}
+
 TrackPrivateBaseGStreamer::TrackPrivateBaseGStreamer(TrackType type, TrackPrivateBase* owner, unsigned index, GstStream* stream)
     : m_notifier(MainThreadNotifier<MainThreadNotification>::create())
     , m_index(index)
+    , m_gstStreamId(AtomString(unsafeSpan8(gst_stream_get_stream_id(stream))))
+    , m_id(parseStreamId(m_gstStreamId).value_or(index))
     , m_stream(stream)
     , m_type(type)
     , m_owner(owner)
 {
     ASSERT(m_stream);
-    m_id = AtomString::fromLatin1(gst_stream_get_stream_id(m_stream.get()));
 
     g_signal_connect_swapped(m_stream.get(), "notify::tags", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
         track->tagsChanged();
@@ -105,16 +109,24 @@ TrackPrivateBaseGStreamer::TrackPrivateBaseGStreamer(TrackType type, TrackPrivat
 
 void TrackPrivateBaseGStreamer::setPad(GRefPtr<GstPad>&& pad)
 {
+    ASSERT(isMainThread()); // because this code writes to AtomString members.
+
     if (m_bestUpstreamPad && m_eventProbe)
         gst_pad_remove_probe(m_bestUpstreamPad.get(), m_eventProbe);
 
     m_pad = WTFMove(pad);
     m_bestUpstreamPad = findBestUpstreamPad(m_pad);
-    m_id = generateUniquePlaybin2StreamID(m_type, m_index);
+    m_gstStreamId = AtomString(unsafeSpan8(gst_pad_get_stream_id(m_pad.get())));
 
-    m_eventProbe = gst_pad_add_probe(m_bestUpstreamPad.get(), GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM, [] (GstPad*, GstPadProbeInfo* info, gpointer userData) -> GstPadProbeReturn {
-        auto* track = static_cast<TrackPrivateBaseGStreamer*>(userData);
-        switch (GST_EVENT_TYPE(gst_pad_probe_info_get_event(info))) {
+    if (m_shouldUsePadStreamId)
+        m_id = parseStreamId(m_gstStreamId).value_or(m_index);
+
+    if (!m_bestUpstreamPad)
+        return;
+
+    m_eventProbe = gst_pad_add_probe(m_bestUpstreamPad.get(), GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM, reinterpret_cast<GstPadProbeCallback>(+[](GstPad*, GstPadProbeInfo* info, TrackPrivateBaseGStreamer* track) -> GstPadProbeReturn {
+        auto* event = gst_pad_probe_info_get_event(info);
+        switch (GST_EVENT_TYPE(event)) {
         case GST_EVENT_TAG:
             track->tagsChanged();
             break;
@@ -122,11 +134,21 @@ void TrackPrivateBaseGStreamer::setPad(GRefPtr<GstPad>&& pad)
             if (track->m_shouldHandleStreamStartEvent)
                 track->streamChanged();
             break;
+        case GST_EVENT_CAPS: {
+            track->m_taskQueue.enqueueTask([track, event = GRefPtr<GstEvent>(event)]() {
+                GstCaps* caps;
+                gst_event_parse_caps(event.get(), &caps);
+                if (!caps)
+                    return;
+                track->capsChanged(track->m_id, GRefPtr<GstCaps>(caps));
+            });
+            break;
+        }
         default:
             break;
         }
         return GST_PAD_PROBE_OK;
-    }, this, nullptr);
+    }), this, nullptr);
 }
 
 TrackPrivateBaseGStreamer::~TrackPrivateBaseGStreamer()
@@ -135,6 +157,15 @@ TrackPrivateBaseGStreamer::~TrackPrivateBaseGStreamer()
     m_notifier->invalidate();
 }
 
+GstObject* TrackPrivateBaseGStreamer::objectForLogging() const
+{
+    if (m_stream)
+        return GST_OBJECT_CAST(m_stream.get());
+
+    ASSERT(m_pad);
+    return GST_OBJECT_CAST(m_pad.get());
+}
+
 void TrackPrivateBaseGStreamer::disconnect()
 {
     if (m_stream)
@@ -150,12 +181,15 @@ void TrackPrivateBaseGStreamer::disconnect()
         m_bestUpstreamPad.clear();
     }
 
-    if (m_pad)
+    if (m_pad) {
+        g_signal_handlers_disconnect_matched(m_pad.get(), G_SIGNAL_MATCH_DATA, 0, 0, nullptr, nullptr, this);
         m_pad.clear();
+    }
 }
 
 void TrackPrivateBaseGStreamer::tagsChanged()
 {
+    // May be called by any thread, including the streaming thread.
     GRefPtr<GstTagList> tags;
     if (m_bestUpstreamPad) {
         GRefPtr<GstEvent> tagEvent;
@@ -164,7 +198,7 @@ void TrackPrivateBaseGStreamer::tagsChanged()
         do {
             tagEvent = adoptGRef(gst_pad_get_sticky_event(m_bestUpstreamPad.get(), GST_EVENT_TAG, i));
             if (tagEvent) {
-                GstTagList* tagsFromEvent;
+                GstTagList* tagsFromEvent = nullptr;
                 gst_event_parse_tag(tagEvent.get(), &tagsFromEvent);
                 tags = adoptGRef(gst_tag_list_copy(tagsFromEvent));
                 String language;
@@ -179,7 +213,7 @@ void TrackPrivateBaseGStreamer::tagsChanged()
     if (!tags)
         tags = adoptGRef(gst_tag_list_new_empty());
 
-    GST_DEBUG("Inspecting track at index %d with tags: %" GST_PTR_FORMAT, m_index, tags.get());
+    GST_DEBUG("Inspecting track %" PRIu64 " with tags: %" GST_PTR_FORMAT, m_id, tags.get());
     {
         Locker locker { m_tagMutex };
         m_tags.swap(tags);
@@ -194,8 +228,8 @@ bool TrackPrivateBaseGStreamer::getLanguageCode(GstTagList* tags, AtomString& va
 {
     String language;
     if (getTag(tags, GST_TAG_LANGUAGE_CODE, language)) {
-        AtomString convertedLanguage = AtomString::fromLatin1(gst_tag_get_language_code_iso_639_1(language.utf8().data()));
-        GST_DEBUG("Converted track %d's language code to %s.", m_index, convertedLanguage.string().utf8().data());
+        AtomString convertedLanguage = AtomString(unsafeSpan8(gst_tag_get_language_code_iso_639_1(language.utf8().data())));
+        GST_DEBUG("Converted track %" PRIu64 "'s language code to %s.", m_id, convertedLanguage.string().utf8().data());
         if (convertedLanguage != value) {
             value = WTFMove(convertedLanguage);
             return true;
@@ -209,7 +243,7 @@ bool TrackPrivateBaseGStreamer::getTag(GstTagList* tags, const gchar* tagName, S
 {
     GUniqueOutPtr<gchar> tagValue;
     if (gst_tag_list_get_string(tags, tagName, &tagValue.outPtr())) {
-        GST_DEBUG("Track %d got %s %s.", m_index, tagName, tagValue.get());
+        GST_DEBUG("Track %" PRIu64 " got %s %s.", m_id, tagName, tagValue.get());
         value = StringType { String::fromLatin1(tagValue.get()) };
         return true;
     }
@@ -218,8 +252,7 @@ bool TrackPrivateBaseGStreamer::getTag(GstTagList* tags, const gchar* tagName, S
 
 void TrackPrivateBaseGStreamer::notifyTrackOfTagsChanged()
 {
-    TrackPrivateBaseClient* client = m_owner->client();
-
+    ASSERT(isMainThread()); // because this code writes to AtomString members.
     GRefPtr<GstTagList> tags;
     {
         Locker locker { m_tagMutex };
@@ -229,8 +262,13 @@ void TrackPrivateBaseGStreamer::notifyTrackOfTagsChanged()
     if (!tags)
         return;
 
-    if (getTag(tags.get(), GST_TAG_TITLE, m_label) && client)
-        client->labelChanged(m_label);
+    tagsChanged(GRefPtr<GstTagList>(tags));
+
+    if (getTag(tags.get(), GST_TAG_TITLE, m_label)) {
+        m_owner->notifyMainThreadClient([&](auto& client) {
+            client.labelChanged(m_label);
+        });
+    }
 
     AtomString language;
     if (!getLanguageCode(tags.get(), language))
@@ -240,18 +278,25 @@ void TrackPrivateBaseGStreamer::notifyTrackOfTagsChanged()
         return;
 
     m_language = language;
-    if (client)
-        client->languageChanged(m_language);
+    m_owner->notifyMainThreadClient([&](auto& client) {
+        client.languageChanged(m_language);
+    });
 }
 
 void TrackPrivateBaseGStreamer::notifyTrackOfStreamChanged()
 {
-    GUniquePtr<char> streamId(gst_pad_get_stream_id(m_pad.get()));
+    if (!m_pad)
+        return;
+
+    auto gstStreamId = AtomString(unsafeSpan8(gst_pad_get_stream_id(m_pad.get())));
+    auto streamId = parseStreamId(gstStreamId);
     if (!streamId)
         return;
 
-    GST_INFO("Track %d got stream start for stream %s.", m_index, streamId.get());
-    m_id = AtomString::fromLatin1(streamId.get());
+    ASSERT(isMainThread()); // because this code writes to AtomString members.
+    m_gstStreamId = gstStreamId;
+    m_id = streamId.value();
+    GST_INFO("Track %" PRIu64 " got stream start. GStreamer stream-id: %s", m_id, m_gstStreamId.string().utf8().data());
 }
 
 void TrackPrivateBaseGStreamer::streamChanged()
@@ -261,6 +306,85 @@ void TrackPrivateBaseGStreamer::streamChanged()
     });
 }
 
+void TrackPrivateBaseGStreamer::installUpdateConfigurationHandlers()
+{
+    if (m_pad) {
+        g_signal_connect_swapped(m_pad.get(), "notify::caps", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
+            if (!track->m_pad)
+                return;
+            auto caps = adoptGRef(gst_pad_get_current_caps(track->m_pad.get()));
+            // We will receive a synchronous notification for caps being unset during pipeline teardown.
+            if (!caps)
+                return;
+
+            track->m_taskQueue.enqueueTask([track, caps = WTFMove(caps)]() mutable {
+                track->capsChanged(getStreamIdFromPad(track->m_pad.get()).value_or(track->m_index), WTFMove(caps));
+            });
+        }), this);
+        g_signal_connect_swapped(m_pad.get(), "notify::tags", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
+            track->m_taskQueue.enqueueTask([track]() {
+                if (!track->m_pad)
+                    return;
+                track->updateConfigurationFromTags(getAllTags(track->m_pad));
+            });
+        }), this);
+    } else if (m_stream) {
+        g_signal_connect_swapped(m_stream.get(), "notify::caps", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
+            track->m_taskQueue.enqueueTask([track]() {
+                auto caps = adoptGRef(gst_stream_get_caps(track->m_stream.get()));
+                track->capsChanged(getStreamIdFromStream(track->m_stream.get()).value_or(track->m_index), WTFMove(caps));
+            });
+        }), this);
+
+        // This signal can be triggered from the main thread
+        // (CanvasCaptureMediaStreamTrack::Source::captureCanvas() triggering the mediastreamsrc
+        // InternalSource::videoFrameAvailable() which can update the stream tags.)
+        g_signal_connect_swapped(m_stream.get(), "notify::tags", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
+            if (isMainThread()) {
+                auto tags = adoptGRef(gst_stream_get_tags(track->m_stream.get()));
+                track->updateConfigurationFromTags(WTFMove(tags));
+                return;
+            }
+            track->m_taskQueue.enqueueTask([track]() {
+                auto tags = adoptGRef(gst_stream_get_tags(track->m_stream.get()));
+                track->updateConfigurationFromTags(WTFMove(tags));
+            });
+        }), this);
+    }
+}
+
+GRefPtr<GstTagList> TrackPrivateBaseGStreamer::getAllTags(const GRefPtr<GstPad>& pad)
+{
+    auto allTags = adoptGRef(gst_tag_list_new_empty());
+    GstTagList* taglist = nullptr;
+    for (guint i = 0;; i++) {
+        GRefPtr<GstEvent> tagsEvent = adoptGRef(gst_pad_get_sticky_event(pad.get(), GST_EVENT_TAG, i));
+        if (!tagsEvent)
+            break;
+        gst_event_parse_tag(tagsEvent.get(), &taglist);
+        allTags = adoptGRef(gst_tag_list_merge(allTags.get(), taglist, GST_TAG_MERGE_APPEND));
+    }
+    return allTags;
+}
+
+bool TrackPrivateBaseGStreamer::updateTrackIDFromTags(const GRefPtr<GstTagList>& tags)
+{
+    ASSERT(isMainThread()); // because this code writes to AtomString members.
+    GUniqueOutPtr<char> trackIDString;
+    if (!gst_tag_list_get_string(tags.get(), "container-specific-track-id", &trackIDString.outPtr()))
+        return false;
+
+    auto trackID = WTF::parseInteger<TrackID>(StringView { unsafeSpan(trackIDString.get()) });
+    if (trackID && *trackID != m_trackID.value_or(0)) {
+        m_trackID = *trackID;
+        ASSERT(m_trackID);
+        return true;
+    }
+    return false;
+}
+
+#undef GST_CAT_DEFAULT
+
 } // namespace WebCore
 
 #endif // ENABLE(VIDEO) && USE(GSTREAMER)
