@@ -21,6 +21,7 @@
 
 #include "DOMMimeType.h"
 #include "Navigator.h"
+#include <ranges>
 #include <wtf/TZoneMallocInlines.h>
 #include <wtf/text/AtomString.h>
 
@@ -38,7 +39,7 @@ static Vector<Ref<DOMMimeType>> makeMimeTypes(Navigator& navigator, const Plugin
     auto types = info.mimes.map([&](auto& type) {
         return DOMMimeType::create(navigator, type, self);
     });
-    std::sort(types.begin(), types.end(), [](const Ref<DOMMimeType>& a, const Ref<DOMMimeType>& b) {
+    std::ranges::sort(types, [](auto& a, auto& b) {
         return codePointCompareLessThan(a->type(), b->type());
     });
 
