@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2006, 2008, 2011-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2024 Apple Inc. All rights reserved.
+ * Copyright (C) 2014 Google Inc. All rights reserved.
  * Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies)
  *
  * This library is free software; you can redistribute it and/or
@@ -23,11 +24,11 @@
 #include "HitTestResult.h"
 
 #include "CachedImage.h"
+#include "ContainerNodeInlines.h"
 #include "DocumentMarkerController.h"
 #include "Editor.h"
 #include "ElementInlines.h"
 #include "File.h"
-#include "Frame.h"
 #include "FrameSelection.h"
 #include "HTMLAnchorElement.h"
 #include "HTMLAttachmentElement.h"
@@ -35,10 +36,13 @@
 #include "HTMLImageElement.h"
 #include "HTMLInputElement.h"
 #include "HTMLObjectElement.h"
-#include "HTMLParserIdioms.h"
 #include "HTMLTextAreaElement.h"
 #include "HTMLVideoElement.h"
 #include "ImageOverlay.h"
+#include "LocalFrame.h"
+#include "NodeInlines.h"
+#include "OriginAccessPatterns.h"
+#include "Page.h"
 #include "PseudoElement.h"
 #include "Range.h"
 #include "RenderBlockFlow.h"
@@ -53,6 +57,7 @@
 #include "UserGestureIndicator.h"
 #include "VisibleUnits.h"
 #include "XLinkNames.h"
+#include <wtf/TZoneMallocInlines.h>
 
 #if ENABLE(SERVICE_CONTROLS)
 #include "ImageControlsMac.h"
@@ -62,6 +67,8 @@ namespace WebCore {
 
 using namespace HTMLNames;
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(HitTestResult);
+
 static inline void appendToNodeSet(const HitTestResult::NodeSet& source, HitTestResult::NodeSet& destination)
 {
     for (auto& node : source)
@@ -152,16 +159,14 @@ void HitTestResult::setToNonUserAgentShadowAncestor()
 
 void HitTestResult::setInnerNode(Node* node)
 {
-    if (is<PseudoElement>(node))
-        node = downcast<PseudoElement>(*node).hostElement();
-    m_innerNode = node;
+    auto* pseudoElement = dynamicDowncast<PseudoElement>(node);
+    m_innerNode = pseudoElement ? pseudoElement->hostElement() : node;
 }
     
 void HitTestResult::setInnerNonSharedNode(Node* node)
 {
-    if (is<PseudoElement>(node))
-        node = downcast<PseudoElement>(*node).hostElement();
-    m_innerNonSharedNode = node;
+    auto* pseudoElement = dynamicDowncast<PseudoElement>(node);
+    m_innerNonSharedNode = pseudoElement ? pseudoElement->hostElement() : node;
 }
 
 void HitTestResult::setURLElement(Element* n) 
@@ -169,12 +174,12 @@ void HitTestResult::setURLElement(Element* n)
     m_innerURLElement = n; 
 }
 
-void HitTestResult::setScrollbar(Scrollbar* s)
+void HitTestResult::setScrollbar(RefPtr<Scrollbar>&& scrollbar)
 {
-    m_scrollbar = s;
+    m_scrollbar = WTFMove(scrollbar);
 }
 
-Frame* HitTestResult::innerNodeFrame() const
+LocalFrame* HitTestResult::innerNodeFrame() const
 {
     if (m_innerNonSharedNode)
         return m_innerNonSharedNode->document().frame();
@@ -183,16 +188,24 @@ Frame* HitTestResult::innerNodeFrame() const
     return 0;
 }
 
-Frame* HitTestResult::targetFrame() const
+LocalFrame* HitTestResult::frame() const
+{
+    if (m_innerNonSharedNode)
+        return m_innerNonSharedNode->document().frame();
+
+    return nullptr;
+}
+
+LocalFrame* HitTestResult::targetFrame() const
 {
     if (!m_innerURLElement)
         return nullptr;
 
-    Frame* frame = m_innerURLElement->document().frame();
+    auto* frame = m_innerURLElement->document().frame();
     if (!frame)
         return nullptr;
 
-    return frame->tree().find(m_innerURLElement->target(), *frame);
+    return dynamicDowncast<LocalFrame>(frame->tree().findBySpecifiedName(m_innerURLElement->target(), *frame));
 }
 
 bool HitTestResult::isSelected() const
@@ -200,19 +213,53 @@ bool HitTestResult::isSelected() const
     if (!m_innerNonSharedNode)
         return false;
 
-    Frame* frame = m_innerNonSharedNode->document().frame();
+    auto* frame = m_innerNonSharedNode->document().frame();
     if (!frame)
         return false;
 
     return frame->selection().contains(m_hitTestLocation.point());
 }
 
+bool HitTestResult::allowsFollowingLink() const
+{
+    auto linkURL = absoluteLinkURL();
+    if (linkURL.isEmpty())
+        return false;
+
+    RefPtr innerFrame = innerNodeFrame();
+    if (!innerFrame)
+        return false;
+
+    RefPtr document = innerFrame->document();
+    if (!document)
+        return false;
+
+    return document->protectedSecurityOrigin()->canDisplay(linkURL, OriginAccessPatternsForWebProcess::singleton());
+}
+
+bool HitTestResult::allowsFollowingImageURL() const
+{
+    auto linkURL = absoluteImageURL();
+    if (linkURL.isEmpty())
+        return false;
+
+    RefPtr innerFrame = innerNodeFrame();
+    if (!innerFrame)
+        return false;
+
+    RefPtr document = innerFrame->document();
+    if (!document)
+        return false;
+
+    return document->protectedSecurityOrigin()->canDisplay(linkURL, OriginAccessPatternsForWebProcess::singleton());
+}
+
 String HitTestResult::selectedText() const
 {
     if (!m_innerNonSharedNode)
         return emptyString();
 
-    Frame* frame = m_innerNonSharedNode->document().frame();
+    auto* frame = m_innerNonSharedNode->document().frame();
     if (!frame)
         return emptyString();
 
@@ -239,12 +286,15 @@ String HitTestResult::spellingToolTip(TextDirection& dir) const
     if (!m_innerNonSharedNode)
         return String();
     
-    DocumentMarker* marker = m_innerNonSharedNode->document().markers().markerContainingPoint(m_hitTestLocation.point(), DocumentMarker::Grammar);
+    CheckedPtr markers = m_innerNonSharedNode->document().markersIfExists();
+    if (!markers)
+        return String();
+    WeakPtr marker = markers->markerContainingPoint(m_hitTestLocation.point(), DocumentMarkerType::Grammar);
     if (!marker)
         return String();
 
-    if (auto renderer = m_innerNonSharedNode->renderer())
-        dir = renderer->style().direction();
+    if (CheckedPtr renderer = m_innerNonSharedNode->renderer())
+        dir = renderer->writingMode().computedTextDirection();
     return marker->description();
 }
 
@@ -254,8 +304,11 @@ String HitTestResult::replacedString() const
     // and is used for generating a contextual menu item that allows it to easily be changed back if desired.
     if (!m_innerNonSharedNode)
         return String();
-    
-    DocumentMarker* marker = m_innerNonSharedNode->document().markers().markerContainingPoint(m_hitTestLocation.point(), DocumentMarker::Replacement);
+
+    CheckedPtr markers = m_innerNonSharedNode->document().markersIfExists();
+    if (!markers)
+        return String();
+    WeakPtr marker = markers->markerContainingPoint(m_hitTestLocation.point(), DocumentMarkerType::Replacement);
     if (!marker)
         return String();
     
@@ -268,12 +321,11 @@ String HitTestResult::title(TextDirection& dir) const
     // Find the title in the nearest enclosing DOM node.
     // For <area> tags in image maps, walk the tree for the <area>, not the <img> using it.
     for (Node* titleNode = m_innerNode.get(); titleNode; titleNode = titleNode->parentInComposedTree()) {
-        if (is<Element>(*titleNode)) {
-            Element& titleElement = downcast<Element>(*titleNode);
-            String title = titleElement.title();
-            if (!title.isEmpty()) {
-                if (auto renderer = titleElement.renderer())
-                    dir = renderer->style().direction();
+        if (RefPtr titleElement = dynamicDowncast<Element>(*titleNode)) {
+            auto title = titleElement->title();
+            if (!title.isNull()) {
+                if (auto renderer = titleElement->renderer())
+                    dir = renderer->writingMode().computedTextDirection();
                 return title;
             }
         }
@@ -283,23 +335,21 @@ String HitTestResult::title(TextDirection& dir) const
 
 String HitTestResult::innerTextIfTruncated(TextDirection& dir) const
 {
-    for (Node* truncatedNode = m_innerNode.get(); truncatedNode; truncatedNode = truncatedNode->parentInComposedTree()) {
-        if (!is<Element>(*truncatedNode))
+    for (auto* truncatedNode = m_innerNode.get(); truncatedNode; truncatedNode = truncatedNode->parentInComposedTree()) {
+        auto* element = dynamicDowncast<Element>(*truncatedNode);
+        if (!element)
             continue;
 
-        if (auto renderer = downcast<Element>(*truncatedNode).renderer()) {
-            if (is<RenderBlockFlow>(*renderer)) {
-                RenderBlockFlow& block = downcast<RenderBlockFlow>(*renderer);
-                if (block.style().textOverflow() == TextOverflow::Ellipsis) {
-                    for (auto* line = block.firstRootBox(); line; line = line->nextRootBox()) {
-                        if (line->hasEllipsisBox()) {
-                            dir = block.style().direction();
-                            return downcast<Element>(*truncatedNode).innerText();
-                        }
+        if (auto* block = dynamicDowncast<RenderBlockFlow>(element->renderer())) {
+            if (block->style().textOverflow() == TextOverflow::Ellipsis) {
+                for (auto lineBox = InlineIterator::firstLineBoxFor(*block); lineBox; lineBox.traverseNext()) {
+                    if (lineBox->hasEllipsis()) {
+                        dir = block->writingMode().computedTextDirection();
+                        return element->innerText();
                     }
                 }
-                break;
             }
+            break;
         }
     }
 
@@ -318,16 +368,12 @@ String HitTestResult::altDisplayString() const
 {
     if (!m_innerNonSharedNode)
         return String();
-    
-    if (is<HTMLImageElement>(*m_innerNonSharedNode)) {
-        HTMLImageElement& image = downcast<HTMLImageElement>(*m_innerNonSharedNode);
-        return displayString(image.attributeWithoutSynchronization(altAttr), m_innerNonSharedNode.get());
-    }
-    
-    if (is<HTMLInputElement>(*m_innerNonSharedNode)) {
-        HTMLInputElement& input = downcast<HTMLInputElement>(*m_innerNonSharedNode);
-        return displayString(input.alt(), m_innerNonSharedNode.get());
-    }
+
+    if (RefPtr image = dynamicDowncast<HTMLImageElement>(*m_innerNonSharedNode))
+        return displayString(image->attributeWithoutSynchronization(altAttr), m_innerNonSharedNode.get());
+
+    if (RefPtr input = dynamicDowncast<HTMLInputElement>(*m_innerNonSharedNode))
+        return displayString(input->attributeWithoutSynchronization(altAttr), m_innerNonSharedNode.get());
 
     return String();
 }
@@ -354,10 +400,9 @@ Image* HitTestResult::image() const
     if (!imageNode)
         return nullptr;
 
-    if (auto renderer = imageNode->renderer(); is<RenderImage>(renderer)) {
-        auto& image = downcast<RenderImage>(*renderer);
-        if (image.cachedImage() && !image.cachedImage()->errorOccurred())
-            return image.cachedImage()->imageForRenderer(&image);
+    if (auto* image = dynamicDowncast<RenderImage>(imageNode->renderer())) {
+        if (image->cachedImage() && !image->cachedImage()->errorOccurred())
+            return image->cachedImage()->imageForRenderer(image);
     }
 
     return nullptr;
@@ -375,6 +420,22 @@ IntRect HitTestResult::imageRect() const
     return imageNode->renderBox()->absoluteContentQuad().enclosingBoundingBox();
 }
 
+bool HitTestResult::hasEntireImage() const
+{
+    auto imageURL = absoluteImageURL();
+    if (imageURL.isEmpty() || imageRect().isEmpty())
+        return false;
+
+    auto* innerFrame = innerNodeFrame();
+    if (!innerFrame)
+        return false;
+
+    if (RefPtr page = innerFrame->page())
+        return page->hasLocalDataForURL(imageURL);
+
+    return false;
+}
+
 URL HitTestResult::absoluteImageURL() const
 {
     auto imageNode = nodeForImageData();
@@ -385,12 +446,17 @@ URL HitTestResult::absoluteImageURL() const
     if (!renderer || !renderer->isImage())
         return { };
 
-    if (is<HTMLEmbedElement>(*imageNode)
-        || is<HTMLImageElement>(*imageNode)
-        || is<HTMLInputElement>(*imageNode)
-        || is<HTMLObjectElement>(*imageNode)
-        || is<SVGImageElement>(*imageNode))
-        return imageNode->document().completeURL(downcast<Element>(*imageNode).imageSourceURL());
+    if (RefPtr element = dynamicDowncast<Element>(*imageNode); element
+        && (is<HTMLEmbedElement>(*element)
+        || is<HTMLImageElement>(*element)
+        || is<HTMLInputElement>(*element)
+        || is<HTMLObjectElement>(*element)
+        || is<SVGImageElement>(*element))) {
+        auto imageURL = imageNode->document().completeURL(element->imageSourceURL());
+        if (RefPtr page = imageNode->document().page())
+            return page->applyLinkDecorationFiltering(imageURL, LinkDecorationFilteringTrigger::Unspecified);
+        return imageURL;
+    }
 
     return { };
 }
@@ -400,15 +466,15 @@ URL HitTestResult::absolutePDFURL() const
     if (!m_innerNonSharedNode)
         return URL();
 
-    if (!is<HTMLEmbedElement>(*m_innerNonSharedNode) && !is<HTMLObjectElement>(*m_innerNonSharedNode))
+    RefPtr element = dynamicDowncast<HTMLPlugInImageElement>(*m_innerNonSharedNode);
+    if (!element)
         return URL();
 
-    HTMLPlugInImageElement& element = downcast<HTMLPlugInImageElement>(*m_innerNonSharedNode);
-    URL url = m_innerNonSharedNode->document().completeURL(stripLeadingAndTrailingHTMLSpaces(element.url()));
+    auto url = m_innerNonSharedNode->document().completeURL(element->url());
     if (!url.isValid())
         return URL();
 
-    if (element.serviceType() == "application/pdf" || (element.serviceType().isEmpty() && url.path().endsWithIgnoringASCIICase(".pdf")))
+    if (element->serviceType() == "application/pdf"_s || (element->serviceType().isEmpty() && url.path().endsWithIgnoringASCIICase(".pdf"_s)))
         return url;
     return URL();
 }
@@ -416,22 +482,23 @@ URL HitTestResult::absolutePDFURL() const
 URL HitTestResult::absoluteMediaURL() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
-        return mediaElt->currentSrc();
-    return URL();
-#else
-    return URL();
+    if (RefPtr element = mediaElement()) {
+        auto sourceURL = element->currentSrc();
+        if (RefPtr page = element->document().page())
+            return page->applyLinkDecorationFiltering(sourceURL, LinkDecorationFilteringTrigger::Unspecified);
+        return sourceURL;
+    }
 #endif
+    return { };
 }
 
 bool HitTestResult::mediaSupportsFullscreen() const
 {
 #if ENABLE(VIDEO)
-    HTMLMediaElement* mediaElt(mediaElement());
-    return is<HTMLVideoElement>(mediaElt) && mediaElt->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModeStandard);
-#else
-    return false;
+    if (RefPtr element = mediaElement())
+        return is<HTMLVideoElement>(*element) && element->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModeStandard);
 #endif
+    return false;
 }
 
 #if ENABLE(VIDEO)
@@ -440,19 +507,26 @@ HTMLMediaElement* HitTestResult::mediaElement() const
     if (!m_innerNonSharedNode)
         return nullptr;
 
-    if (!(m_innerNonSharedNode->renderer() && m_innerNonSharedNode->renderer()->isMedia()))
+    if (!(m_innerNonSharedNode->renderer() && m_innerNonSharedNode->renderer()->isRenderMedia()))
         return nullptr;
 
-    if (is<HTMLMediaElement>(*m_innerNonSharedNode))
-        return downcast<HTMLMediaElement>(m_innerNonSharedNode.get());
-    return nullptr;
+    return dynamicDowncast<HTMLMediaElement>(*m_innerNonSharedNode);
 }
 #endif
 
+bool HitTestResult::hasMediaElement() const
+{
+#if ENABLE(VIDEO)
+    return !!mediaElement();
+#else
+    return false;
+#endif
+}
+
 void HitTestResult::toggleMediaControlsDisplay() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         mediaElt->setControls(!mediaElt->controls());
 #endif
 }
@@ -460,15 +534,23 @@ void HitTestResult::toggleMediaControlsDisplay() const
 void HitTestResult::toggleMediaLoopPlayback() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         mediaElt->setLoop(!mediaElt->loop());
 #endif
 }
 
+void HitTestResult::toggleShowMediaStats() const
+{
+#if ENABLE(VIDEO)
+    if (RefPtr mediaElt = mediaElement())
+        mediaElt->setShowingStats(!mediaElt->showingStats());
+#endif
+}
+
 bool HitTestResult::mediaIsInFullscreen() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElement = this->mediaElement())
+    if (RefPtr mediaElement = this->mediaElement())
         return mediaElement->isVideo() && mediaElement->isStandardFullscreen();
 #endif
     return false;
@@ -477,9 +559,9 @@ bool HitTestResult::mediaIsInFullscreen() const
 void HitTestResult::toggleMediaFullscreenState() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElement = this->mediaElement()) {
+    if (RefPtr mediaElement = this->mediaElement()) {
         if (mediaElement->isVideo() && mediaElement->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModeStandard)) {
-            UserGestureIndicator indicator(ProcessingUserGesture, &mediaElement->document());
+            UserGestureIndicator indicator(IsProcessingUserGesture::Yes, &mediaElement->document());
             mediaElement->toggleStandardFullscreenState();
         }
     }
@@ -489,21 +571,44 @@ void HitTestResult::toggleMediaFullscreenState() const
 void HitTestResult::enterFullscreenForVideo() const
 {
 #if ENABLE(VIDEO)
-    HTMLMediaElement* mediaElement(this->mediaElement());
-    if (is<HTMLVideoElement>(mediaElement)) {
-        HTMLVideoElement& videoElement = downcast<HTMLVideoElement>(*mediaElement);
-        if (!videoElement.isFullscreen() && mediaElement->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModeStandard)) {
-            UserGestureIndicator indicator(ProcessingUserGesture, &mediaElement->document());
-            videoElement.webkitEnterFullscreen();
+    RefPtr mediaElement(this->mediaElement());
+    if (RefPtr videoElement = dynamicDowncast<HTMLVideoElement>(mediaElement)) {
+        if (!videoElement->isFullscreen() && mediaElement->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModeStandard)) {
+            UserGestureIndicator indicator(IsProcessingUserGesture::Yes, &mediaElement->document());
+            videoElement->webkitEnterFullscreen();
         }
     }
 #endif
 }
 
+bool HitTestResult::mediaIsInVideoViewer() const
+{
+#if PLATFORM(MAC) && ENABLE(VIDEO) && ENABLE(VIDEO_PRESENTATION_MODE)
+    if (RefPtr mediaElt = mediaElement())
+        return is<HTMLVideoElement>(mediaElt) && mediaElt->fullscreenMode() == HTMLMediaElementEnums::VideoFullscreenModeInWindow;
+#endif
+    return false;
+}
+
+void HitTestResult::toggleVideoViewer() const
+{
+#if PLATFORM(MAC) && ENABLE(VIDEO) && ENABLE(VIDEO_PRESENTATION_MODE)
+    RefPtr mediaElement(this->mediaElement());
+    RefPtr videoElement = dynamicDowncast<HTMLVideoElement>(mediaElement);
+    if (!videoElement || !mediaElement->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModeInWindow))
+        return;
+
+    UserGestureIndicator indicator(IsProcessingUserGesture::Yes, &mediaElement->document());
+    auto newMode = videoElement->webkitPresentationMode() == HTMLVideoElement::VideoPresentationMode::InWindow ? HTMLVideoElement::VideoPresentationMode::Inline : HTMLVideoElement::VideoPresentationMode::InWindow;
+
+    videoElement->webkitSetPresentationMode(newMode);
+#endif
+}
+
 bool HitTestResult::mediaControlsEnabled() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElement = this->mediaElement())
+    if (RefPtr mediaElement = this->mediaElement())
         return mediaElement->controls();
 #endif
     return false;
@@ -512,16 +617,25 @@ bool HitTestResult::mediaControlsEnabled() const
 bool HitTestResult::mediaLoopEnabled() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         return mediaElt->loop();
 #endif
     return false;
 }
 
+bool HitTestResult::mediaStatsShowing() const
+{
+#if ENABLE(VIDEO)
+    if (RefPtr mediaElt = mediaElement())
+        return mediaElt->showingStats();
+#endif
+    return false;
+}
+
 bool HitTestResult::mediaPlaying() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         return !mediaElt->paused();
 #endif
     return false;
@@ -530,7 +644,7 @@ bool HitTestResult::mediaPlaying() const
 void HitTestResult::toggleMediaPlayState() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         mediaElt->togglePlayState();
 #endif
 }
@@ -538,7 +652,7 @@ void HitTestResult::toggleMediaPlayState() const
 bool HitTestResult::mediaHasAudio() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         return mediaElt->hasAudio();
 #endif
     return false;
@@ -547,7 +661,7 @@ bool HitTestResult::mediaHasAudio() const
 bool HitTestResult::mediaIsVideo() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         return is<HTMLVideoElement>(*mediaElt);
 #endif
     return false;
@@ -556,7 +670,7 @@ bool HitTestResult::mediaIsVideo() const
 bool HitTestResult::mediaMuted() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         return mediaElt->muted();
 #endif
     return false;
@@ -565,7 +679,7 @@ bool HitTestResult::mediaMuted() const
 void HitTestResult::toggleMediaMuteState() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         mediaElt->setMuted(!mediaElt->muted());
 #endif
 }
@@ -573,7 +687,7 @@ void HitTestResult::toggleMediaMuteState() const
 bool HitTestResult::isDownloadableMedia() const
 {
 #if ENABLE(VIDEO)
-    if (HTMLMediaElement* mediaElt = mediaElement())
+    if (RefPtr mediaElt = mediaElement())
         return mediaElt->canSaveMediaData();
 #endif
 
@@ -582,22 +696,19 @@ bool HitTestResult::isDownloadableMedia() const
 
 bool HitTestResult::isOverTextInsideFormControlElement() const
 {
-    Node* node = innerNode();
-    if (!node)
+    RefPtr element = dynamicDowncast<Element>(innerNode());
+    if (!element || !element->isTextField())
         return false;
 
-    if (!is<Element>(*node) || !downcast<Element>(*node).isTextField())
-        return false;
-
-    Frame* frame = node->document().frame();
+    auto* frame = element->document().frame();
     if (!frame)
         return false;
 
-    IntPoint framePoint = roundedPointInInnerNodeFrame();
+    auto framePoint = roundedPointInInnerNodeFrame();
     if (!frame->rangeForPoint(framePoint))
         return false;
 
-    VisiblePosition position = frame->visiblePositionForPoint(framePoint);
+    auto position = frame->visiblePositionForPoint(framePoint);
     if (position.isNull())
         return false;
 
@@ -607,9 +718,26 @@ bool HitTestResult::isOverTextInsideFormControlElement() const
 
 URL HitTestResult::absoluteLinkURL() const
 {
-    if (m_innerURLElement)
-        return m_innerURLElement->absoluteLinkURL();
-    return URL();
+    if (!m_innerURLElement)
+        return { };
+
+    auto url = m_innerURLElement->absoluteLinkURL();
+    if (RefPtr page = m_innerURLElement->document().page())
+        return page->applyLinkDecorationFiltering(url, LinkDecorationFilteringTrigger::Unspecified);
+
+    return url;
+}
+
+bool HitTestResult::hasLocalDataForLinkURL() const
+{
+    auto linkURL = absoluteLinkURL();
+    if (linkURL.isEmpty())
+        return false;
+
+    if (RefPtr page = m_innerURLElement->document().page())
+        return page->hasLocalDataForURL(linkURL);
+
+    return false;
 }
 
 bool HitTestResult::isOverLink() const
@@ -644,8 +772,8 @@ bool HitTestResult::isContentEditable() const
     if (is<HTMLTextAreaElement>(*m_innerNonSharedNode))
         return true;
 
-    if (is<HTMLInputElement>(*m_innerNonSharedNode))
-        return downcast<HTMLInputElement>(*m_innerNonSharedNode).isTextField();
+    if (RefPtr input = dynamicDowncast<HTMLInputElement>(*m_innerNonSharedNode))
+        return input->isTextField();
 
     return m_innerNonSharedNode->hasEditableStyle();
 }
@@ -723,48 +851,47 @@ Vector<String> HitTestResult::dictationAlternatives() const
     if (!m_innerNonSharedNode)
         return Vector<String>();
 
-    DocumentMarker* marker = m_innerNonSharedNode->document().markers().markerContainingPoint(pointInInnerNodeFrame(), DocumentMarker::DictationAlternatives);
+    CheckedPtr markers = m_innerNonSharedNode->document().markersIfExists();
+    if (!markers)
+        return Vector<String>();
+
+    WeakPtr marker = markers->markerContainingPoint(pointInInnerNodeFrame(), DocumentMarkerType::DictationAlternatives);
     if (!marker)
         return Vector<String>();
 
-    Frame* frame = innerNonSharedNode()->document().frame();
+    RefPtr frame = innerNonSharedNode()->document().frame();
     if (!frame)
         return Vector<String>();
 
     return frame->editor().dictationAlternativesForMarker(*marker);
 }
 
-Node* HitTestResult::targetNode() const
+RefPtr<Node> HitTestResult::protectedTargetNode() const
 {
-    Node* node = innerNode();
-    if (!node)
-        return nullptr;
-    if (node->isConnected())
-        return node;
-
-    Element* element = node->parentElement();
-    if (element && element->isConnected())
-        return element;
-
-    return node;
+    return innerNode();
 }
 
 Element* HitTestResult::targetElement() const
 {
     for (Node* node = m_innerNode.get(); node; node = node->parentInComposedTree()) {
-        if (is<Element>(*node))
-            return downcast<Element>(node);
+        if (auto* element = dynamicDowncast<Element>(*node))
+            return element;
     }
     return nullptr;
 }
 
+RefPtr<Element> HitTestResult::protectedTargetElement() const
+{
+    return targetElement();
+}
+
 Element* HitTestResult::innerNonSharedElement() const
 {
-    Node* node = m_innerNonSharedNode.get();
+    auto* node = m_innerNonSharedNode.get();
     if (!node)
         return nullptr;
-    if (is<Element>(*node))
-        return downcast<Element>(node);
+    if (auto* element = dynamicDowncast<Element>(*node))
+        return element;
     return node->parentElement();
 }
 
@@ -779,37 +906,70 @@ String HitTestResult::linkSuggestedFilename() const
 bool HitTestResult::mediaSupportsEnhancedFullscreen() const
 {
 #if PLATFORM(MAC) && ENABLE(VIDEO) && ENABLE(VIDEO_PRESENTATION_MODE)
-    HTMLMediaElement* mediaElt(mediaElement());
-    return is<HTMLVideoElement>(mediaElt) && mediaElt->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModePictureInPicture);
-#else
-    return false;
+    if (RefPtr mediaElt = mediaElement())
+        return is<HTMLVideoElement>(mediaElt) && mediaElt->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModePictureInPicture);
 #endif
+    return false;
 }
 
 bool HitTestResult::mediaIsInEnhancedFullscreen() const
 {
 #if PLATFORM(MAC) && ENABLE(VIDEO) && ENABLE(VIDEO_PRESENTATION_MODE)
-    HTMLMediaElement* mediaElt(mediaElement());
-    return is<HTMLVideoElement>(mediaElt) && mediaElt->fullscreenMode() == HTMLMediaElementEnums::VideoFullscreenModePictureInPicture;
-#else
-    return false;
+    if (RefPtr mediaElt = mediaElement())
+        return is<HTMLVideoElement>(mediaElt) && mediaElt->fullscreenMode() == HTMLMediaElementEnums::VideoFullscreenModePictureInPicture;
 #endif
+    return false;
 }
 
 void HitTestResult::toggleEnhancedFullscreenForVideo() const
 {
 #if PLATFORM(MAC) && ENABLE(VIDEO) && ENABLE(VIDEO_PRESENTATION_MODE)
-    HTMLMediaElement* mediaElement(this->mediaElement());
-    if (!is<HTMLVideoElement>(mediaElement) || !mediaElement->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModePictureInPicture))
+    RefPtr mediaElement(this->mediaElement());
+    RefPtr videoElement = dynamicDowncast<HTMLVideoElement>(mediaElement);
+    if (!mediaElement || !videoElement || !mediaElement->supportsFullscreen(HTMLMediaElementEnums::VideoFullscreenModePictureInPicture))
         return;
 
-    HTMLVideoElement& videoElement = downcast<HTMLVideoElement>(*mediaElement);
-    UserGestureIndicator indicator(ProcessingUserGesture, &mediaElement->document());
-    if (videoElement.webkitPresentationMode() == HTMLVideoElement::VideoPresentationMode::PictureInPicture)
-        videoElement.webkitSetPresentationMode(HTMLVideoElement::VideoPresentationMode::Inline);
+    UserGestureIndicator indicator(IsProcessingUserGesture::Yes, &mediaElement->document());
+    if (videoElement->webkitPresentationMode() == HTMLVideoElement::VideoPresentationMode::PictureInPicture)
+        videoElement->webkitSetPresentationMode(HTMLVideoElement::VideoPresentationMode::Inline);
     else
-        videoElement.webkitSetPresentationMode(HTMLVideoElement::VideoPresentationMode::PictureInPicture);
+        videoElement->webkitSetPresentationMode(HTMLVideoElement::VideoPresentationMode::PictureInPicture);
 #endif
 }
 
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+HTMLImageElement* HitTestResult::imageElement() const
+{
+    if (auto* imageElement = dynamicDowncast<HTMLImageElement>(m_innerNonSharedNode.get()))
+        return imageElement;
+    return nullptr;
+}
+
+bool HitTestResult::isAnimating() const
+{
+    if (auto* imageElement = this->imageElement())
+        return imageElement->allowsAnimation();
+    return false;
+}
+
+void HitTestResult::playAnimation() const
+{
+    setAllowsAnimation(true);
+}
+
+void HitTestResult::pauseAnimation() const
+{
+    setAllowsAnimation(false);
+}
+
+void HitTestResult::setAllowsAnimation(bool allowAnimation) const
+{
+    if (auto* imageElement = this->imageElement()) {
+        imageElement->setAllowsAnimation(allowAnimation);
+        if (auto* renderer = m_innerNonSharedNode->renderer())
+            renderer->repaint();
+    }
+}
+#endif // ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+
 } // namespace WebCore
