@@ -20,10 +20,12 @@
 #include "config.h"
 #include "RenderSVGTextPath.h"
 
+#include "ContainerNodeInlines.h"
 #include "FloatQuad.h"
 #include "RenderBlock.h"
 #include "RenderBoxModelObjectInlines.h"
 #include "RenderLayer.h"
+#include "RenderObjectInlines.h"
 #include "RenderSVGInlineInlines.h"
 #include "RenderSVGShape.h"
 #include "SVGElementTypeHelpers.h"
