@@ -21,20 +21,16 @@
 
 #pragma once
 
-#if ENABLE(SVG_FONTS)
-
 #include "SVGElement.h"
 
 namespace WebCore {
 
 class SVGGlyphElement final : public SVGElement {
-    WTF_MAKE_ISO_ALLOCATED(SVGGlyphElement);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(SVGGlyphElement);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(SVGGlyphElement);
 public:
     static Ref<SVGGlyphElement> create(const QualifiedName&, Document&);
 
-    // Helper function used by SVGFont
-    static String querySVGFontLanguage(const SVGElement*);
-
 private:
     SVGGlyphElement(const QualifiedName&, Document&);
 
@@ -42,5 +38,3 @@ private:
 };
 
 } // namespace WebCore
-
-#endif // ENABLE(SVG_FONTS)
