@@ -34,12 +34,15 @@
 #include <wtf/CompletionHandler.h>
 #include <wtf/LoggerHelper.h>
 #include <wtf/NeverDestroyed.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/UUID.h>
 #include <wtf/text/StringHash.h>
 #include <wtf/text/StringView.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(MockMediaSessionCoordinator);
+
 Ref<MockMediaSessionCoordinator> MockMediaSessionCoordinator::create(ScriptExecutionContext& context, RefPtr<StringCallback>&& listener)
 {
     return adoptRef(*new MockMediaSessionCoordinator(context, WTFMove(listener)));
@@ -54,7 +57,7 @@ MockMediaSessionCoordinator::MockMediaSessionCoordinator(ScriptExecutionContext&
 std::optional<Exception> MockMediaSessionCoordinator::result() const
 {
     if (m_failCommands)
-        return Exception { InvalidStateError };
+        return Exception { ExceptionCode::InvalidStateError };
 
     return std::nullopt;
 }
