@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2013-2017 Apple Inc. All Rights Reserved.
+ * Copyright (C) 2008, 2013-2017 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -339,7 +339,7 @@ WI.DataGridNode = class DataGridNode extends WI.Object
             this._scheduledRefreshIdentifier = undefined;
         }
 
-        this._cachedFilterableData = null;
+        this.clearCachedFilterableData();
         this._needsRefresh = false;
 
         this._element.removeChildren();
@@ -378,6 +378,7 @@ WI.DataGridNode = class DataGridNode extends WI.Object
             if (column["icon"]) {
                 let iconElement = document.createElement("div");
                 iconElement.classList.add("icon");
+                iconElement.title = this.generateIconTitle(columnIdentifier);
                 div.insertBefore(iconElement, div.firstChild);
             }
         }
@@ -404,6 +405,12 @@ WI.DataGridNode = class DataGridNode extends WI.Object
         return typeof data === "number" ? data.maxDecimals(2).toLocaleString() : data;
     }
 
+    generateIconTitle(columnIdentifier)
+    {
+        // Overridden by subclasses if needed.
+        return "";
+    }
+
     elementWithColumnIdentifier(columnIdentifier)
     {
         if (!this.dataGrid)
@@ -454,8 +461,6 @@ WI.DataGridNode = class DataGridNode extends WI.Object
         for (var i = 0; i < this.children.length; ++i)
             this.children[i].revealed = false;
 
-        this.dispatchEventToListeners("collapsed");
-
         if (this.dataGrid) {
             this.dataGrid.dispatchEventToListeners(WI.DataGrid.Event.CollapsedNode, {dataGridNode: this});
             this.dataGrid._noteRowsChanged();
@@ -485,7 +490,7 @@ WI.DataGridNode = class DataGridNode extends WI.Object
             for (var i = 0; i < this.children.length; ++i)
                 this.children[i]._detach();
 
-            this.dispatchEventToListeners("populate");
+            this.dispatchEventToListeners(DataGridNode.Event.Populate);
 
             if (this._attached) {
                 for (var i = 0; i < this.children.length; ++i) {
@@ -504,8 +509,6 @@ WI.DataGridNode = class DataGridNode extends WI.Object
 
         this._expanded = true;
 
-        this.dispatchEventToListeners("expanded");
-
         if (this.dataGrid) {
             this.dataGrid.dispatchEventToListeners(WI.DataGrid.Event.ExpandedNode, {dataGridNode: this});
             this.dataGrid._noteRowsChanged();
@@ -557,8 +560,6 @@ WI.DataGridNode = class DataGridNode extends WI.Object
         }
 
         this.dataGrid.updateVisibleRows(this);
-
-        this.dispatchEventToListeners("revealed");
     }
 
     select(suppressSelectedEvent)
@@ -566,18 +567,10 @@ WI.DataGridNode = class DataGridNode extends WI.Object
         if (!this.dataGrid || !this.selectable || this.selected)
             return;
 
-        let oldSelectedNode = this.dataGrid.selectedNode;
-        if (oldSelectedNode)
-            oldSelectedNode.deselect(true);
-
         this._selected = true;
-        this.dataGrid.selectedNode = this;
+        this._element?.classList.add("selected");
 
-        if (this._element)
-            this._element.classList.add("selected");
-
-        if (!suppressSelectedEvent)
-            this.dataGrid.dispatchEventToListeners(WI.DataGrid.Event.SelectedNodeChanged, {oldSelectedNode});
+        this.dataGrid.selectDataGridNodeInternal(this, suppressSelectedEvent);
     }
 
     revealAndSelect(suppressSelectedEvent)
@@ -588,23 +581,19 @@ WI.DataGridNode = class DataGridNode extends WI.Object
 
     deselect(suppressDeselectedEvent)
     {
-        if (!this.dataGrid || this.dataGrid.selectedNode !== this || !this.selected)
+        if (!this.dataGrid || !this.selectable || !this.selected)
             return;
 
         this._selected = false;
-        this.dataGrid.selectedNode = null;
+        this._element?.classList.remove("selected");
 
-        if (this._element)
-            this._element.classList.remove("selected");
-
-        if (!suppressDeselectedEvent)
-            this.dataGrid.dispatchEventToListeners(WI.DataGrid.Event.SelectedNodeChanged, {oldSelectedNode: this});
+        this.dataGrid.deselectDataGridNodeInternal(this, suppressDeselectedEvent);
     }
 
     traverseNextNode(skipHidden, stayWithin, dontPopulate, info)
     {
         if (!dontPopulate && this.hasChildren)
-            this.dispatchEventToListeners("populate");
+            this.dispatchEventToListeners(DataGridNode.Event.Populate);
 
         if (info)
             info.depthChange = 0;
@@ -640,11 +629,11 @@ WI.DataGridNode = class DataGridNode extends WI.Object
     {
         var node = (!skipHidden || this.revealed) ? this.previousSibling : null;
         if (!dontPopulate && node && node.hasChildren)
-            node.dispatchEventToListeners("populate");
+            node.dispatchEventToListeners(DataGridNode.Event.Populate);
 
         while (node && ((!skipHidden || (node.revealed && node.expanded)) ? node.children.lastValue : null)) {
             if (!dontPopulate && node.hasChildren)
-                node.dispatchEventToListeners("populate");
+                node.dispatchEventToListeners(DataGridNode.Event.Populate);
             node = (!skipHidden || (node.revealed && node.expanded)) ? node.children.lastValue : null;
         }
 
@@ -666,12 +655,11 @@ WI.DataGridNode = class DataGridNode extends WI.Object
         if (!cell || !cell.classList.contains("disclosure"))
             return false;
 
-        let computedStyle = window.getComputedStyle(cell);
         let start = 0;
         if (WI.resolvedLayoutDirection() === WI.LayoutDirection.RTL)
-            start += cell.totalOffsetRight - computedStyle.getPropertyCSSValue("padding-right").getFloatValue(CSSPrimitiveValue.CSS_PX) - this.disclosureToggleWidth;
+            start += cell.totalOffsetRight - cell.getComputedCSSPropertyNumberValue("padding-right") - this.disclosureToggleWidth;
         else
-            start += cell.totalOffsetLeft + computedStyle.getPropertyCSSValue("padding-left").getFloatValue(CSSPrimitiveValue.CSS_PX);
+            start += cell.totalOffsetLeft + cell.getComputedCSSPropertyNumberValue("padding-left");
         return event.pageX >= start && event.pageX <= start + this.disclosureToggleWidth;
     }
 
@@ -685,12 +673,12 @@ WI.DataGridNode = class DataGridNode extends WI.Object
         let insertionIndex = -1;
 
         if (!this.isPlaceholderNode) {
-            var previousGridNode = this.traversePreviousNode(true, true);
-            insertionIndex = this.dataGrid._rows.indexOf(previousGridNode);
-            if (insertionIndex === -1)
+            let previousGridNode = this.traversePreviousNode(true, true);
+            if (previousGridNode) {
+                // If the previousGridNode isn't in the list, we'll increment the not found value of `-1` to `0`.
+                insertionIndex = this.dataGrid._rows.lastIndexOf(previousGridNode) + 1;
+            } else
                 insertionIndex = 0;
-            else
-                insertionIndex++;
         }
 
         if (insertionIndex === -1)
@@ -760,12 +748,23 @@ WI.DataGridNode = class DataGridNode extends WI.Object
         return typeof value === "string" ? value : null;
     }
 
+    clearCachedFilterableData()
+    {
+        this._cachedFilterableData = null;
+
+        this.dataGrid?._applyFiltersToNodeAndDispatchEvent(this);
+    }
+
     didResizeColumn(columnIdentifier)
     {
         // Override by subclasses.
     }
 };
 
+WI.DataGridNode.Event = {
+    Populate: "data-grid-node-populate",
+};
+
 // Used to create a new table row when entering new data by editing cells.
 WI.PlaceholderDataGridNode = class PlaceholderDataGridNode extends WI.DataGridNode
 {
