@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2019-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,19 +26,22 @@
 #include "config.h"
 #include "NetworkSocketChannel.h"
 
+#include "MessageSenderInlines.h"
 #include "NetworkConnectionToWebProcess.h"
 #include "NetworkProcess.h"
 #include "NetworkSession.h"
-#include "WebCoreArgumentCoders.h"
 #include "WebSocketChannelMessages.h"
 #include "WebSocketTask.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebKit {
 using namespace WebCore;
 
-std::unique_ptr<NetworkSocketChannel> NetworkSocketChannel::create(NetworkConnectionToWebProcess& connection, PAL::SessionID sessionID, const ResourceRequest& request, const String& protocol, WebSocketIdentifier identifier, WebPageProxyIdentifier webPageProxyID, const WebCore::ClientOrigin& clientOrigin, bool hadMainFrameMainResourcePrivateRelayed)
+WTF_MAKE_TZONE_ALLOCATED_IMPL(NetworkSocketChannel);
+
+RefPtr<NetworkSocketChannel> NetworkSocketChannel::create(NetworkConnectionToWebProcess& connection, PAL::SessionID sessionID, const ResourceRequest& request, const String& protocol, WebSocketIdentifier identifier, WebPageProxyIdentifier webPageProxyID, std::optional<FrameIdentifier> frameID, std::optional<PageIdentifier> pageID, const WebCore::ClientOrigin& clientOrigin, bool hadMainFrameMainResourcePrivateRelayed, bool allowPrivacyProxy, OptionSet<AdvancedPrivacyProtections> advancedPrivacyProtections, WebCore::StoredCredentialsPolicy storedCredentialsPolicy)
 {
-    auto result = makeUnique<NetworkSocketChannel>(connection, connection.networkProcess().networkSession(sessionID), request, protocol, identifier, webPageProxyID, clientOrigin, hadMainFrameMainResourcePrivateRelayed);
+    Ref result = adoptRef(*new NetworkSocketChannel(connection, connection.networkProcess().networkSession(sessionID), request, protocol, identifier, webPageProxyID, frameID, pageID, clientOrigin, hadMainFrameMainResourcePrivateRelayed, allowPrivacyProxy, advancedPrivacyProtections, storedCredentialsPolicy));
     if (!result->m_socket) {
         result->didClose(0, "Cannot create a web socket task"_s);
         return nullptr;
@@ -46,40 +49,50 @@ std::unique_ptr<NetworkSocketChannel> NetworkSocketChannel::create(NetworkConnec
     return result;
 }
 
-NetworkSocketChannel::NetworkSocketChannel(NetworkConnectionToWebProcess& connection, NetworkSession* session, const ResourceRequest& request, const String& protocol, WebSocketIdentifier identifier, WebPageProxyIdentifier webPageProxyID, const WebCore::ClientOrigin& clientOrigin, bool hadMainFrameMainResourcePrivateRelayed)
+NetworkSocketChannel::NetworkSocketChannel(NetworkConnectionToWebProcess& connection, NetworkSession* session, const ResourceRequest& request, const String& protocol, WebSocketIdentifier identifier, WebPageProxyIdentifier webPageProxyID, std::optional<FrameIdentifier> frameID, std::optional<PageIdentifier> pageID, const WebCore::ClientOrigin& clientOrigin, bool hadMainFrameMainResourcePrivateRelayed, bool allowPrivacyProxy, OptionSet<AdvancedPrivacyProtections> advancedPrivacyProtections, WebCore::StoredCredentialsPolicy storedCredentialsPolicy)
     : m_connectionToWebProcess(connection)
     , m_identifier(identifier)
     , m_session(session)
     , m_errorTimer(*this, &NetworkSocketChannel::sendDelayedError)
     , m_webPageProxyID(webPageProxyID)
 {
-    if (!m_session)
+    relaxAdoptionRequirement();
+    if (!session)
         return;
 
-    m_socket = m_session->createWebSocketTask(webPageProxyID, *this, request, protocol, clientOrigin, hadMainFrameMainResourcePrivateRelayed);
-    if (m_socket) {
-        m_session->addWebSocketTask(webPageProxyID, *m_socket);
-        m_socket->resume();
+    m_socket = session->createWebSocketTask(webPageProxyID, frameID, pageID, *this, request, protocol, clientOrigin, hadMainFrameMainResourcePrivateRelayed, allowPrivacyProxy, advancedPrivacyProtections, storedCredentialsPolicy);
+    if (CheckedPtr socket = m_socket.get()) {
+#if PLATFORM(COCOA)
+        session->addWebSocketTask(webPageProxyID, *socket);
+#endif
+        socket->resume();
     }
 }
 
 NetworkSocketChannel::~NetworkSocketChannel()
 {
-    if (m_socket) {
-        if (m_session && m_socket->sessionSet())
-            m_session->removeWebSocketTask(*m_socket->sessionSet(), *m_socket);
-        m_socket->cancel();
+    if (CheckedPtr socket = m_socket.get()) {
+#if PLATFORM(COCOA)
+        if (RefPtr sessionSet = m_session ? socket->sessionSet() : nullptr)
+            CheckedRef { *m_session }->removeWebSocketTask(*sessionSet, *socket);
+#endif
+        socket->cancel();
     }
 }
 
-void NetworkSocketChannel::sendString(const IPC::DataReference& message, CompletionHandler<void()>&& callback)
+Ref<NetworkConnectionToWebProcess> NetworkSocketChannel::protectedConnectionToWebProcess()
 {
-    m_socket->sendString(message, WTFMove(callback));
+    return m_connectionToWebProcess.get();
 }
 
-void NetworkSocketChannel::sendData(const IPC::DataReference& data, CompletionHandler<void()>&& callback)
+void NetworkSocketChannel::sendString(std::span<const uint8_t> message, CompletionHandler<void()>&& callback)
 {
-    m_socket->sendData(data, WTFMove(callback));
+    checkedSocket()->sendString(message, WTFMove(callback));
+}
+
+void NetworkSocketChannel::sendData(std::span<const uint8_t> data, CompletionHandler<void()>&& callback)
+{
+    checkedSocket()->sendData(data, WTFMove(callback));
 }
 
 void NetworkSocketChannel::finishClosingIfPossible()
@@ -90,12 +103,12 @@ void NetworkSocketChannel::finishClosingIfPossible()
     }
     ASSERT(m_state == State::Closing);
     m_state = State::Closed;
-    m_connectionToWebProcess.removeSocketChannel(m_identifier);
+    protectedConnectionToWebProcess()->removeSocketChannel(m_identifier);
 }
 
 void NetworkSocketChannel::close(int32_t code, const String& reason)
 {
-    m_socket->close(code, reason);
+    checkedSocket()->close(code, reason);
     finishClosingIfPossible();
 }
 
@@ -109,9 +122,9 @@ void NetworkSocketChannel::didReceiveText(const String& text)
     send(Messages::WebSocketChannel::DidReceiveText { text });
 }
 
-void NetworkSocketChannel::didReceiveBinaryData(const uint8_t* data, size_t length)
+void NetworkSocketChannel::didReceiveBinaryData(std::span<const uint8_t> data)
 {
-    send(Messages::WebSocketChannel::DidReceiveBinaryData { { data, length } });
+    send(Messages::WebSocketChannel::DidReceiveBinaryData { data });
 }
 
 void NetworkSocketChannel::didClose(unsigned short code, const String& reason)
@@ -152,12 +165,17 @@ void NetworkSocketChannel::didReceiveHandshakeResponse(ResourceResponse&& respon
 
 IPC::Connection* NetworkSocketChannel::messageSenderConnection() const
 {
-    return &m_connectionToWebProcess.connection();
+    return &m_connectionToWebProcess->connection();
 }
 
-NetworkSession* NetworkSocketChannel::session()
+NetworkSession* NetworkSocketChannel::session() const
 {
     return m_session.get();
 }
 
+CheckedPtr<WebSocketTask> NetworkSocketChannel::checkedSocket()
+{
+    return m_socket.get();
+}
+
 } // namespace WebKit
