@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -149,9 +149,7 @@ enum class CacheOption : uint8_t {
     // In testing mode we try to eliminate sources of randomness. Cache does not shrink and there are no read timeouts.
     TestingMode = 1 << 0,
     RegisterNotify = 1 << 1,
-#if ENABLE(NETWORK_CACHE_SPECULATIVE_REVALIDATION)
     SpeculativeRevalidation = 1 << 2,
-#endif
 };
 
 class Cache : public RefCountedAndCanMakeWeakPtr<Cache> {
@@ -198,18 +196,13 @@ public:
 
     String recordsPathIsolatedCopy() const;
 
-#if ENABLE(NETWORK_CACHE_SPECULATIVE_REVALIDATION)
     SpeculativeLoadManager* speculativeLoadManager() { return m_speculativeLoadManager.get(); }
-#endif
 
-#if ENABLE(NETWORK_CACHE_STALE_WHILE_REVALIDATE)
     void startAsyncRevalidationIfNeeded(const WebCore::ResourceRequest&, const NetworkCache::Key&, std::unique_ptr<Entry>&&, const GlobalFrameID&, std::optional<NavigatingToAppBoundDomain>, bool allowPrivacyProxy, OptionSet<WebCore::AdvancedPrivacyProtections>);
-#endif
 
     void browsingContextRemoved(WebPageProxyIdentifier, WebCore::PageIdentifier, WebCore::FrameIdentifier);
 
     NetworkProcess& networkProcess() { return m_networkProcess.get(); }
-    Ref<NetworkProcess> protectedNetworkProcess();
     PAL::SessionID sessionID() const { return m_sessionID; }
     const String& storageDirectory() const { return m_storageDirectory; }
     void fetchData(bool shouldComputeSize, CompletionHandler<void(Vector<WebsiteData::Entry>&&)>&&);
@@ -233,19 +226,15 @@ private:
     const Ref<Storage> m_storage;
     const Ref<NetworkProcess> m_networkProcess;
 
-#if ENABLE(NETWORK_CACHE_SPECULATIVE_REVALIDATION)
     bool shouldUseSpeculativeLoadManager() const;
     void updateSpeculativeLoadManagerEnabledState();
 
     std::unique_ptr<WebCore::LowPowerModeNotifier> m_lowPowerModeNotifier;
     std::unique_ptr<WebCore::ThermalMitigationNotifier> m_thermalMitigationNotifier;
     std::unique_ptr<SpeculativeLoadManager> m_speculativeLoadManager;
-#endif
 
-#if ENABLE(NETWORK_CACHE_STALE_WHILE_REVALIDATE)
     HashMap<Key, Ref<AsyncRevalidation>> m_pendingAsyncRevalidations;
     HashMap<GlobalFrameID, WeakHashSet<AsyncRevalidation>> m_pendingAsyncRevalidationByPage;
-#endif
 
     unsigned m_traverseCount { 0 };
     PAL::SessionID m_sessionID;
