@@ -32,7 +32,6 @@
 #include "GestureTypes.h"
 #endif
 #include "MessageNames.h"
-#include "Plugin.h"
 #include <WebCore/KeyboardEvent.h>
 #include <WebCore/PlatformLayerIdentifier.h>
 #include <WebCore/PluginData.h>
@@ -74,17 +73,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit LoadURL(const String& url)
-        : m_arguments(url)
+        : m_url(url)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_url;
     }
 
 private:
-    std::tuple<const String&> m_arguments;
+    const String& m_url;
 };
 
 #if ENABLE(TOUCH_EVENTS)
@@ -99,17 +99,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit LoadSomething(const String& url)
-        : m_arguments(url)
+        : m_url(url)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_url;
     }
 
 private:
-    std::tuple<const String&> m_arguments;
+    const String& m_url;
 };
 #endif
 
@@ -125,17 +126,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit TouchEvent(const WebKit::WebTouchEvent& event)
-        : m_arguments(event)
+        : m_event(event)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_event;
     }
 
 private:
-    std::tuple<const WebKit::WebTouchEvent&> m_arguments;
+    SUPPRESS_FORWARD_DECL_MEMBER const WebKit::WebTouchEvent& m_event;
 };
 #endif
 
@@ -151,17 +153,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit AddEvent(const WebKit::WebTouchEvent& event)
-        : m_arguments(event)
+        : m_event(event)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_event;
     }
 
 private:
-    std::tuple<const WebKit::WebTouchEvent&> m_arguments;
+    SUPPRESS_FORWARD_DECL_MEMBER const WebKit::WebTouchEvent& m_event;
 };
 #endif
 
@@ -177,17 +180,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit LoadSomethingElse(const String& url)
-        : m_arguments(url)
+        : m_url(url)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_url;
     }
 
 private:
-    std::tuple<const String&> m_arguments;
+    const String& m_url;
 };
 #endif
 
@@ -202,17 +206,24 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     DidReceivePolicyDecision(uint64_t frameID, uint64_t listenerID, uint32_t policyAction)
-        : m_arguments(frameID, listenerID, policyAction)
+        : m_frameID(frameID)
+        , m_listenerID(listenerID)
+        , m_policyAction(policyAction)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_frameID;
+        encoder << m_listenerID;
+        encoder << m_policyAction;
     }
 
 private:
-    std::tuple<uint64_t, uint64_t, uint32_t> m_arguments;
+    uint64_t m_frameID;
+    uint64_t m_listenerID;
+    uint32_t m_policyAction;
 };
 
 class Close {
@@ -225,13 +236,16 @@ public:
     static constexpr bool replyCanDispatchOutOfOrder = false;
     static constexpr bool deferSendingIfSuspended = false;
 
-    auto&& arguments()
+    Close()
+    {
+    }
+
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
     }
 
 private:
-    std::tuple<> m_arguments;
 };
 
 class PreferencesDidChange {
@@ -245,17 +259,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit PreferencesDidChange(const WebKit::WebPreferencesStore& store)
-        : m_arguments(store)
+        : m_store(store)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_store;
     }
 
 private:
-    std::tuple<const WebKit::WebPreferencesStore&> m_arguments;
+    SUPPRESS_FORWARD_DECL_MEMBER const WebKit::WebPreferencesStore& m_store;
 };
 
 class SendDoubleAndFloat {
@@ -269,17 +284,21 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     SendDoubleAndFloat(double d, float f)
-        : m_arguments(d, f)
+        : m_d(d)
+        , m_f(f)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_d;
+        encoder << m_f;
     }
 
 private:
-    std::tuple<double, float> m_arguments;
+    double m_d;
+    float m_f;
 };
 
 class SendInts {
@@ -293,17 +312,21 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     SendInts(const Vector<uint64_t>& ints, const Vector<Vector<uint64_t>>& intVectors)
-        : m_arguments(ints, intVectors)
+        : m_ints(ints)
+        , m_intVectors(intVectors)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_ints;
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_intVectors;
     }
 
 private:
-    std::tuple<const Vector<uint64_t>&, const Vector<Vector<uint64_t>>&> m_arguments;
+    SUPPRESS_FORWARD_DECL_MEMBER const Vector<uint64_t>& m_ints;
+    SUPPRESS_FORWARD_DECL_MEMBER const Vector<Vector<uint64_t>>& m_intVectors;
 };
 
 class CreatePlugin {
@@ -322,17 +345,21 @@ public:
     using Reply = CompletionHandler<void(bool)>;
     using Promise = WTF::NativePromise<bool, IPC::Error>;
     CreatePlugin(uint64_t pluginInstanceID, const WebKit::Plugin::Parameters& parameters)
-        : m_arguments(pluginInstanceID, parameters)
+        : m_pluginInstanceID(pluginInstanceID)
+        , m_parameters(parameters)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_pluginInstanceID;
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_parameters;
     }
 
 private:
-    std::tuple<uint64_t, const WebKit::Plugin::Parameters&> m_arguments;
+    uint64_t m_pluginInstanceID;
+    SUPPRESS_FORWARD_DECL_MEMBER const WebKit::Plugin::Parameters& m_parameters;
 };
 
 class RunJavaScriptAlert {
@@ -351,17 +378,21 @@ public:
     using Reply = CompletionHandler<void()>;
     using Promise = WTF::NativePromise<void, IPC::Error>;
     RunJavaScriptAlert(uint64_t frameID, const String& message)
-        : m_arguments(frameID, message)
+        : m_frameID(frameID)
+        , m_message(message)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_frameID;
+        encoder << m_message;
     }
 
 private:
-    std::tuple<uint64_t, const String&> m_arguments;
+    uint64_t m_frameID;
+    const String& m_message;
 };
 
 class GetPlugins {
@@ -380,17 +411,18 @@ public:
     using Reply = CompletionHandler<void(Vector<WebCore::PluginInfo>&&)>;
     using Promise = WTF::NativePromise<Vector<WebCore::PluginInfo>, IPC::Error>;
     explicit GetPlugins(bool refresh)
-        : m_arguments(refresh)
+        : m_refresh(refresh)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_refresh;
     }
 
 private:
-    std::tuple<bool> m_arguments;
+    bool m_refresh;
 };
 
 class GetPluginProcessConnection {
@@ -407,17 +439,18 @@ public:
     using ReplyArguments = std::tuple<IPC::Connection::Handle>;
     using Reply = CompletionHandler<void(IPC::Connection::Handle&&)>;
     explicit GetPluginProcessConnection(const String& pluginPath)
-        : m_arguments(pluginPath)
+        : m_pluginPath(pluginPath)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_pluginPath;
     }
 
 private:
-    std::tuple<const String&> m_arguments;
+    const String& m_pluginPath;
 };
 
 class TestMultipleAttributes {
@@ -433,13 +466,16 @@ public:
     static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
     using ReplyArguments = std::tuple<>;
     using Reply = CompletionHandler<void()>;
-    auto&& arguments()
+    TestMultipleAttributes()
+    {
+    }
+
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
     }
 
 private:
-    std::tuple<> m_arguments;
 };
 
 class TestParameterAttributes {
@@ -453,17 +489,24 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     TestParameterAttributes(uint64_t foo, double bar, double baz)
-        : m_arguments(foo, bar, baz)
+        : m_foo(foo)
+        , m_bar(bar)
+        , m_baz(baz)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_foo;
+        encoder << m_bar;
+        encoder << m_baz;
     }
 
 private:
-    std::tuple<uint64_t, double, double> m_arguments;
+    uint64_t m_foo;
+    double m_bar;
+    double m_baz;
 };
 
 class TemplateTest {
@@ -477,17 +520,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit TemplateTest(const HashMap<String, std::pair<String, uint64_t>>& a)
-        : m_arguments(a)
+        : m_a(a)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_a;
     }
 
 private:
-    std::tuple<const HashMap<String, std::pair<String, uint64_t>>&> m_arguments;
+    SUPPRESS_FORWARD_DECL_MEMBER const HashMap<String, std::pair<String, uint64_t>>& m_a;
 };
 
 class SetVideoLayerID {
@@ -501,17 +545,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit SetVideoLayerID(const WebCore::PlatformLayerIdentifier& videoLayerID)
-        : m_arguments(videoLayerID)
+        : m_videoLayerID(videoLayerID)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_videoLayerID;
     }
 
 private:
-    std::tuple<const WebCore::PlatformLayerIdentifier&> m_arguments;
+    const WebCore::PlatformLayerIdentifier& m_videoLayerID;
 };
 
 #if PLATFORM(MAC)
@@ -526,17 +571,21 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     DidCreateWebProcessConnection(MachSendRight&& connectionIdentifier, const OptionSet<WebKit::SelectionFlags>& flags)
-        : m_arguments(WTFMove(connectionIdentifier), flags)
+        : m_connectionIdentifier(WTFMove(connectionIdentifier))
+        , m_flags(flags)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << WTFMove(m_connectionIdentifier);
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_flags;
     }
 
 private:
-    std::tuple<MachSendRight&&, const OptionSet<WebKit::SelectionFlags>&> m_arguments;
+    MachSendRight&& m_connectionIdentifier;
+    SUPPRESS_FORWARD_DECL_MEMBER const OptionSet<WebKit::SelectionFlags>& m_flags;
 };
 #endif
 
@@ -557,17 +606,18 @@ public:
     using Reply = CompletionHandler<void(Vector<WebCore::KeypressCommand>&&)>;
     using Promise = WTF::NativePromise<Vector<WebCore::KeypressCommand>, IPC::Error>;
     explicit InterpretKeyEvent(uint32_t type)
-        : m_arguments(type)
+        : m_type(type)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        encoder << m_type;
     }
 
 private:
-    std::tuple<uint32_t> m_arguments;
+    uint32_t m_type;
 };
 #endif
 
@@ -583,17 +633,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit DeprecatedOperation(const IPC::DummyType& dummy)
-        : m_arguments(dummy)
+        : m_dummy(dummy)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_dummy;
     }
 
 private:
-    std::tuple<const IPC::DummyType&> m_arguments;
+    SUPPRESS_FORWARD_DECL_MEMBER const IPC::DummyType& m_dummy;
 };
 #endif
 
@@ -609,17 +660,18 @@ public:
     static constexpr bool deferSendingIfSuspended = false;
 
     explicit ExperimentalOperation(const IPC::DummyType& dummy)
-        : m_arguments(dummy)
+        : m_dummy(dummy)
     {
     }
 
-    auto&& arguments()
+    template<typename Encoder>
+    void encode(Encoder& encoder)
     {
-        return WTFMove(m_arguments);
+        SUPPRESS_FORWARD_DECL_ARG encoder << m_dummy;
     }
 
 private:
-    std::tuple<const IPC::DummyType&> m_arguments;
+    SUPPRESS_FORWARD_DECL_MEMBER const IPC::DummyType& m_dummy;
 };
 #endif
 
