@@ -24,10 +24,7 @@
 #include <wtf/glib/GRefPtr.h>
 
 namespace WTF {
-
-template <> WebKitWebViewBackend* refGPtr(WebKitWebViewBackend* ptr);
-template <> void derefGPtr(WebKitWebViewBackend* ptr);
-
+WTF_DECLARE_GREF_TRAITS(WebKitWebViewBackend)
 }
 
 void webkitWebViewBackendUnref(WebKitWebViewBackend*);
