@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2010-2021 Apple Inc. All rights reserved.
- * Portions Copyright (c) 2011 Motorola Mobility, Inc.  All rights reserved.
+ * Copyright (C) 2010-2025 Apple Inc. All rights reserved.
+ * Portions Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,11 +29,15 @@
 
 #include "APIInspectorClient.h"
 #include "APINavigation.h"
+#include "APIPageConfiguration.h"
 #include "APIProcessPoolConfiguration.h"
 #include "APIUIClient.h"
 #include "InspectorBrowserAgent.h"
+#include "MessageSenderInlines.h"
+#include "PageClient.h"
 #include "WebAutomationSession.h"
 #include "WebFrameProxy.h"
+#include "WebInspectorBackendProxyMessages.h"
 #include "WebInspectorInterruptDispatcherMessages.h"
 #include "WebInspectorMessages.h"
 #include "WebInspectorUIExtensionControllerProxy.h"
@@ -51,6 +55,10 @@
 #include <pal/text/TextEncoding.h>
 #include <wtf/SetForScope.h>
 
+#if ENABLE(INSPECTOR_EXTENSIONS)
+#include "WebExtensionController.h"
+#endif
+
 #if PLATFORM(GTK)
 #include "WebInspectorUIProxyClient.h"
 #endif
@@ -65,14 +73,15 @@ const unsigned WebInspectorUIProxy::initialWindowWidth = 1000;
 const unsigned WebInspectorUIProxy::initialWindowHeight = 650;
 
 WebInspectorUIProxy::WebInspectorUIProxy(WebPageProxy& inspectedPage)
-    : m_inspectedPage(&inspectedPage)
+    : m_backend(adoptRef(new WebInspectorBackendProxy(*this)))
+    , m_inspectedPage(inspectedPage)
     , m_inspectorClient(makeUnique<API::InspectorClient>())
     , m_inspectedPageIdentifier(inspectedPage.identifier())
 #if PLATFORM(MAC)
-    , m_closeFrontendAfterInactivityTimer(RunLoop::main(), this, &WebInspectorUIProxy::closeFrontendAfterInactivityTimerFired)
+    , m_closeFrontendAfterInactivityTimer(RunLoop::mainSingleton(), "WebInspectorUIProxy::CloseFrontendAfterInactivityTimer"_s, this, &WebInspectorUIProxy::closeFrontendAfterInactivityTimerFired)
 #endif
 {
-    m_inspectedPage->process().addMessageReceiver(Messages::WebInspectorUIProxy::messageReceiverName(), m_inspectedPage->webPageID(), *this);
+    protectedInspectedPage()->protectedLegacyMainFrameProcess()->addMessageReceiver(Messages::WebInspectorBackendProxy::messageReceiverName(), m_inspectedPage->webPageIDInMainFrameProcess(), *m_backend);
 }
 
 WebInspectorUIProxy::~WebInspectorUIProxy()
@@ -91,13 +100,18 @@ void WebInspectorUIProxy::setInspectorClient(std::unique_ptr<API::InspectorClien
 
 unsigned WebInspectorUIProxy::inspectionLevel() const
 {
-    return inspectorLevelForPage(inspectedPage());
+    return inspectorLevelForPage(protectedInspectedPage().get());
 }
 
 WebPreferences& WebInspectorUIProxy::inspectorPagePreferences() const
 {
     ASSERT(m_inspectorPage);
-    return m_inspectorPage->pageGroup().preferences();
+    return protectedInspectorPage()->pageGroup().preferences();
+}
+
+Ref<WebPreferences> WebInspectorUIProxy::protectedInspectorPagePreferences() const
+{
+    return inspectorPagePreferences();
 }
 
 void WebInspectorUIProxy::invalidate()
@@ -110,10 +124,11 @@ void WebInspectorUIProxy::invalidate()
 
 void WebInspectorUIProxy::sendMessageToFrontend(const String& message)
 {
-    if (!m_inspectorPage)
+    RefPtr inspectorPage = m_inspectorPage.get();
+    if (!inspectorPage)
         return;
 
-    m_inspectorPage->send(Messages::WebInspectorUI::SendMessageToFrontend(message));
+    inspectorPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::SendMessageToFrontend(message), m_inspectorPage->webPageIDInMainFrameProcess());
 }
 
 // Public APIs
@@ -127,10 +142,11 @@ bool WebInspectorUIProxy::isFront()
 
 void WebInspectorUIProxy::connect()
 {
-    if (!m_inspectedPage)
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
         return;
 
-    if (!m_inspectedPage->preferences().developerExtrasEnabled())
+    if (!inspectedPage->protectedPreferences()->developerExtrasEnabled())
         return;
 
     if (m_showMessageSent)
@@ -141,8 +157,14 @@ void WebInspectorUIProxy::connect()
 
     createFrontendPage();
 
-    m_inspectedPage->send(Messages::WebInspectorInterruptDispatcher::NotifyNeedDebuggerBreak(), 0);
-    m_inspectedPage->send(Messages::WebInspector::Show());
+    Ref legacyMainFrameProcess = inspectedPage->legacyMainFrameProcess();
+    legacyMainFrameProcess->send(Messages::WebInspectorInterruptDispatcher::NotifyNeedDebuggerBreak(), 0);
+    legacyMainFrameProcess->sendWithAsyncReply(
+        Messages::WebInspector::Show(),
+        [this, protectedThis = Ref { *this }] {
+            openLocalInspectorFrontend();
+        },
+        m_inspectedPage->webPageIDInMainFrameProcess());
 }
 
 void WebInspectorUIProxy::show()
@@ -173,10 +195,11 @@ void WebInspectorUIProxy::hide()
 
 void WebInspectorUIProxy::close()
 {
-    if (!m_inspectedPage)
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
         return;
 
-    m_inspectedPage->send(Messages::WebInspector::Close());
+    inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::Close(), m_inspectedPage->webPageIDInMainFrameProcess());
 
     closeFrontendPageAndWindow();
 }
@@ -204,19 +227,20 @@ void WebInspectorUIProxy::resetState()
     if (!m_inspectedPage || !m_inspectorPage)
         return;
 
-    inspectorPagePreferences().deleteInspectorAttachedHeight();
-    inspectorPagePreferences().deleteInspectorAttachedWidth();
-    inspectorPagePreferences().deleteInspectorAttachmentSide();
-    inspectorPagePreferences().deleteInspectorStartsAttached();
-    inspectorPagePreferences().deleteInspectorWindowFrame();
+    Ref preferences = inspectorPagePreferences();
+    preferences->deleteInspectorAttachedHeight();
+    preferences->deleteInspectorAttachedWidth();
+    preferences->deleteInspectorAttachmentSide();
+    preferences->deleteInspectorStartsAttached();
+    preferences->deleteInspectorWindowFrame();
 
     platformResetState();
 }
 
 void WebInspectorUIProxy::reset()
 {
-    if (m_inspectedPage) {
-        m_inspectedPage->process().removeMessageReceiver(Messages::WebInspectorUIProxy::messageReceiverName(), m_inspectedPage->webPageID());
+    if (RefPtr inspectedPage = m_inspectedPage.get()) {
+        inspectedPage->protectedLegacyMainFrameProcess()->removeMessageReceiver(Messages::WebInspectorBackendProxy::messageReceiverName(), inspectedPage->webPageIDInMainFrameProcess());
         m_inspectedPage = nullptr;
     }
 }
@@ -225,51 +249,55 @@ void WebInspectorUIProxy::updateForNewPageProcess(WebPageProxy& inspectedPage)
 {
     ASSERT(!m_inspectedPage);
 
-    m_inspectedPage = &inspectedPage;
-    m_inspectedPageIdentifier = m_inspectedPage->identifier();
+    m_inspectedPage = inspectedPage;
+    m_inspectedPageIdentifier = inspectedPage.identifier();
 
-    m_inspectedPage->process().addMessageReceiver(Messages::WebInspectorUIProxy::messageReceiverName(), m_inspectedPage->webPageID(), *this);
+    protectedInspectedPage()->protectedLegacyMainFrameProcess()->addMessageReceiver(Messages::WebInspectorBackendProxy::messageReceiverName(), m_inspectedPage->webPageIDInMainFrameProcess(), *m_backend);
 
-    if (m_inspectorPage)
-        m_inspectorPage->send(Messages::WebInspectorUI::UpdateConnection());
+    if (RefPtr inspectorPage = m_inspectorPage.get())
+        inspectorPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::UpdateConnection(), m_inspectorPage->webPageIDInMainFrameProcess());
 }
 
-void WebInspectorUIProxy::setFrontendConnection(IPC::Attachment connectionIdentifier)
+void WebInspectorUIProxy::setFrontendConnection(IPC::Connection::Handle&& connectionIdentifier)
 {
+    RefPtr inspectedPage = m_inspectedPage.get();
     if (!m_inspectedPage)
         return;
 
-    m_inspectedPage->send(Messages::WebInspector::SetFrontendConnection(connectionIdentifier));
+    inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::SetFrontendConnection(WTFMove(connectionIdentifier)), inspectedPage->webPageIDInMainFrameProcess());
 }
 
 void WebInspectorUIProxy::showConsole()
 {
-    if (!m_inspectedPage)
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
         return;
 
-    createFrontendPage();
+    show();
 
-    m_inspectedPage->send(Messages::WebInspector::ShowConsole());
+    inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::ShowConsole(), inspectedPage->webPageIDInMainFrameProcess());
 }
 
 void WebInspectorUIProxy::showResources()
 {
-    if (!m_inspectedPage)
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
         return;
 
-    createFrontendPage();
+    show();
 
-    m_inspectedPage->send(Messages::WebInspector::ShowResources());
+    inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::ShowResources(), inspectedPage->webPageIDInMainFrameProcess());
 }
 
-void WebInspectorUIProxy::showMainResourceForFrame(WebFrameProxy* frame)
+void WebInspectorUIProxy::showMainResourceForFrame(WebCore::FrameIdentifier frameID)
 {
-    if (!m_inspectedPage)
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
         return;
 
-    createFrontendPage();
+    show();
 
-    m_inspectedPage->send(Messages::WebInspector::ShowMainResourceForFrame(frame->frameID()));
+    inspectedPage->sendToProcessContainingFrame(frameID, Messages::WebInspector::ShowMainResourceForFrame(frameID));
 }
 
 void WebInspectorUIProxy::attachBottom()
@@ -296,28 +324,31 @@ void WebInspectorUIProxy::attach(AttachmentSide side)
     m_isAttached = true;
     m_attachmentSide = side;
 
-    inspectorPagePreferences().setInspectorAttachmentSide(static_cast<uint32_t>(side));
+    Ref preferences = inspectorPagePreferences();
+    preferences->setInspectorAttachmentSide(static_cast<uint32_t>(side));
 
     if (m_isVisible)
-        inspectorPagePreferences().setInspectorStartsAttached(true);
+        preferences->setInspectorStartsAttached(true);
 
-    m_inspectedPage->send(Messages::WebInspector::SetAttached(true));
+    protectedInspectedPage()->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::SetAttached(true), m_inspectedPage->webPageIDInMainFrameProcess());
 
     switch (m_attachmentSide) {
     case AttachmentSide::Bottom:
-        m_inspectorPage->send(Messages::WebInspectorUI::AttachedBottom());
+        protectedInspectorPage()->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::AttachedBottom(), m_inspectorPage->webPageIDInMainFrameProcess());
         break;
 
     case AttachmentSide::Right:
-        m_inspectorPage->send(Messages::WebInspectorUI::AttachedRight());
+        protectedInspectorPage()->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::AttachedRight(), m_inspectorPage->webPageIDInMainFrameProcess());
         break;
 
     case AttachmentSide::Left:
-        m_inspectorPage->send(Messages::WebInspectorUI::AttachedLeft());
+        protectedInspectorPage()->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::AttachedLeft(), m_inspectorPage->webPageIDInMainFrameProcess());
         break;
     }
 
     platformAttach();
+
+    dispatchDidChangeLocalInspectorAttachment();
 }
 
 void WebInspectorUIProxy::detach()
@@ -328,12 +359,14 @@ void WebInspectorUIProxy::detach()
     m_isAttached = false;
 
     if (m_isVisible)
-        inspectorPagePreferences().setInspectorStartsAttached(false);
+        protectedInspectorPagePreferences()->setInspectorStartsAttached(false);
 
-    m_inspectedPage->send(Messages::WebInspector::SetAttached(false));
-    m_inspectorPage->send(Messages::WebInspectorUI::Detached());
+    protectedInspectedPage()->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::SetAttached(false), m_inspectedPage->webPageIDInMainFrameProcess());
+    protectedInspectorPage()->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::Detached(), m_inspectorPage->webPageIDInMainFrameProcess());
 
     platformDetach();
+
+    dispatchDidChangeLocalInspectorAttachment();
 }
 
 void WebInspectorUIProxy::setAttachedWindowHeight(unsigned height)
@@ -342,7 +375,7 @@ void WebInspectorUIProxy::setAttachedWindowHeight(unsigned height)
     if (!m_inspectorPage)
         return;
 
-    inspectorPagePreferences().setInspectorAttachedHeight(height);
+    protectedInspectorPagePreferences()->setInspectorAttachedHeight(height);
     platformSetAttachedWindowHeight(height);
 }
 
@@ -352,7 +385,7 @@ void WebInspectorUIProxy::setAttachedWindowWidth(unsigned width)
     if (!m_inspectorPage)
         return;
 
-    inspectorPagePreferences().setInspectorAttachedWidth(width);
+    protectedInspectorPagePreferences()->setInspectorAttachedWidth(width);
     platformSetAttachedWindowWidth(width);
 }
 
@@ -371,10 +404,13 @@ void WebInspectorUIProxy::togglePageProfiling()
     if (!m_inspectedPage)
         return;
 
+    show();
+
+    RefPtr inspectedPage = m_inspectedPage.get();
     if (m_isProfilingPage)
-        m_inspectedPage->send(Messages::WebInspector::StopPageProfiling());
+        inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::StopPageProfiling(), inspectedPage->webPageIDInMainFrameProcess());
     else
-        m_inspectedPage->send(Messages::WebInspector::StartPageProfiling());
+        inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::StartPageProfiling(), inspectedPage->webPageIDInMainFrameProcess());
 }
 
 void WebInspectorUIProxy::toggleElementSelection()
@@ -382,12 +418,13 @@ void WebInspectorUIProxy::toggleElementSelection()
     if (!m_inspectedPage)
         return;
 
+    RefPtr inspectedPage = m_inspectedPage.get();
     if (m_elementSelectionActive) {
         m_ignoreElementSelectionChange = true;
-        m_inspectedPage->send(Messages::WebInspector::StopElementSelection());
+        inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::StopElementSelection(), inspectedPage->webPageIDInMainFrameProcess());
     } else {
         connect();
-        m_inspectedPage->send(Messages::WebInspector::StartElementSelection());
+        inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::StartElementSelection(), inspectedPage->webPageIDInMainFrameProcess());
     }
 }
 
@@ -412,84 +449,108 @@ void WebInspectorUIProxy::createFrontendPage()
     if (m_inspectorPage)
         return;
 
-    m_inspectorPage = platformCreateFrontendPage();
-    ASSERT(m_inspectorPage);
-    if (!m_inspectorPage)
+    RefPtr inspectorPage = platformCreateFrontendPage();
+    m_inspectorPage = inspectorPage.get();
+    ASSERT(inspectorPage);
+    if (!inspectorPage)
         return;
 
-    trackInspectorPage(m_inspectorPage, m_inspectedPage);
+    trackInspectorPage(inspectorPage.get(), protectedInspectedPage().get());
 
     // Make sure the inspected page has a running WebProcess so we can inspect it.
-    m_inspectedPage->launchInitialProcessIfNecessary();
+    protectedInspectedPage()->launchInitialProcessIfNecessary();
 
-    m_inspectorPage->process().addMessageReceiver(Messages::WebInspectorUIProxy::messageReceiverName(), m_inspectedPageIdentifier, *this);
+    inspectorPage->protectedLegacyMainFrameProcess()->addMessageReceiver(Messages::WebInspectorUIProxy::messageReceiverName(), m_inspectedPageIdentifier, *this);
 
 #if ENABLE(INSPECTOR_EXTENSIONS)
-    m_extensionController = WebInspectorUIExtensionControllerProxy::create(*m_inspectorPage);
+    m_extensionController = WebInspectorUIExtensionControllerProxy::create(*inspectorPage);
 #endif
 }
 
-void WebInspectorUIProxy::openLocalInspectorFrontend(bool canAttach, bool underTest)
+void WebInspectorUIProxy::requestOpenLocalInspectorFrontend()
 {
-    if (!m_inspectedPage)
+    // Prevent a compromised malicious web page from opening Web Inspector at will.
+    if (!m_underTest && inspectionLevel() < 2)
         return;
 
-    if (!m_inspectedPage->preferences().developerExtrasEnabled())
+    openLocalInspectorFrontend();
+}
+
+void WebInspectorUIProxy::openLocalInspectorFrontend()
+{
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
         return;
 
-    if (m_inspectedPage->inspectorController().hasLocalFrontend()) {
+    if (!inspectedPage->protectedPreferences()->developerExtrasEnabled())
+        return;
+
+    if (inspectedPage->inspectorController().hasLocalFrontend()) {
         show();
         return;
     }
 
-    m_underTest = underTest;
     createFrontendPage();
 
-    ASSERT(m_inspectorPage);
-    if (!m_inspectorPage)
+    RefPtr inspectorPage = m_inspectorPage.get();
+    ASSERT(inspectorPage);
+    if (!inspectorPage)
         return;
 
-    m_inspectorPage->send(Messages::WebInspectorUI::EstablishConnection(m_inspectedPageIdentifier, infoForLocalDebuggable(), m_underTest, inspectionLevel()));
+    inspectorPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::EstablishConnection(m_inspectedPageIdentifier, infoForLocalDebuggable(), m_underTest, inspectionLevel()), m_inspectorPage->webPageIDInMainFrameProcess());
 
     ASSERT(!m_isActiveFrontend);
     m_isActiveFrontend = true;
-    m_inspectedPage->inspectorController().connectFrontend(*this);
+    inspectedPage->inspectorController().connectFrontend(*this);
 
     if (!m_underTest) {
-        m_canAttach = platformCanAttach(canAttach);
+        // FIXME <https://webkit.org/b/283435>: Remove the webProcessCanAttach argument from platformCanAttach.
+        // The value canAttach in the web process is no longer used or respected.
+        const bool webProcessCanAttach = false;
+        m_canAttach = platformCanAttach(webProcessCanAttach);
         m_isAttached = shouldOpenAttached();
-        m_attachmentSide = static_cast<AttachmentSide>(inspectorPagePreferences().inspectorAttachmentSide());
+        m_attachmentSide = static_cast<AttachmentSide>(protectedInspectorPagePreferences()->inspectorAttachmentSide());
 
-        m_inspectedPage->send(Messages::WebInspector::SetAttached(m_isAttached));
+        inspectedPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspector::SetAttached(m_isAttached), inspectedPage->webPageIDInMainFrameProcess());
 
+        Ref inspectorPageProcess = inspectorPage->legacyMainFrameProcess();
         if (m_isAttached) {
             switch (m_attachmentSide) {
             case AttachmentSide::Bottom:
-                m_inspectorPage->send(Messages::WebInspectorUI::AttachedBottom());
+                inspectorPageProcess->send(Messages::WebInspectorUI::AttachedBottom(), inspectorPage->webPageIDInMainFrameProcess());
                 break;
 
             case AttachmentSide::Right:
-                m_inspectorPage->send(Messages::WebInspectorUI::AttachedRight());
+                inspectorPageProcess->send(Messages::WebInspectorUI::AttachedRight(), inspectorPage->webPageIDInMainFrameProcess());
                 break;
 
             case AttachmentSide::Left:
-                m_inspectorPage->send(Messages::WebInspectorUI::AttachedLeft());
+                inspectorPageProcess->send(Messages::WebInspectorUI::AttachedLeft(), inspectorPage->webPageIDInMainFrameProcess());
                 break;
             }
         } else
-            m_inspectorPage->send(Messages::WebInspectorUI::Detached());
+            inspectorPageProcess->send(Messages::WebInspectorUI::Detached(), inspectorPage->webPageIDInMainFrameProcess());
 
-        m_inspectorPage->send(Messages::WebInspectorUI::SetDockingUnavailable(!m_canAttach));
+        inspectorPageProcess->send(Messages::WebInspectorUI::SetDockingUnavailable(!m_canAttach), inspectorPage->webPageIDInMainFrameProcess());
+
+        dispatchDidChangeLocalInspectorAttachment();
     }
 
-    // Notify WebKit client when a local inspector attaches so that it may install delegates prior to the _WKInspector loading its frontend.
-    m_inspectedPage->uiClient().didAttachLocalInspector(*m_inspectedPage, *this);
+    // Notify clients when a local inspector attaches so that it may install delegates prior to the _WKInspector loading its frontend.
+
+#if ENABLE(INSPECTOR_EXTENSIONS)
+    if (RefPtr webExtensionController = inspectedPage->webExtensionController())
+        webExtensionController->inspectorWillOpen(*this, *inspectedPage);
+#endif
+
+    inspectedPage->uiClient().didAttachLocalInspector(*inspectedPage, *this);
 
     // Bail out if the client closed the inspector from the delegate method.
-    if (!m_inspectorPage)
+
+    if (!inspectorPage)
         return;
 
-    m_inspectorPage->loadRequest(URL { m_underTest ? WebInspectorUIProxy::inspectorTestPageURL() : WebInspectorUIProxy::inspectorPageURL() });
+    inspectorPage->loadRequest(URL { m_underTest ? WebInspectorUIProxy::inspectorTestPageURL() : WebInspectorUIProxy::inspectorPageURL() });
 }
 
 void WebInspectorUIProxy::open()
@@ -503,7 +564,7 @@ void WebInspectorUIProxy::open()
     SetForScope isOpening(m_isOpening, true);
 
     m_isVisible = true;
-    m_inspectorPage->send(Messages::WebInspectorUI::SetIsVisible(m_isVisible));
+    protectedInspectorPage()->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::SetIsVisible(m_isVisible), m_inspectorPage->webPageIDInMainFrameProcess());
 
     if (m_isAttached && platformCanAttach(m_canAttach))
         platformAttach();
@@ -513,6 +574,8 @@ void WebInspectorUIProxy::open()
     }
 
     platformBringToFront();
+
+    dispatchDidChangeLocalInspectorAttachment();
 }
 
 void WebInspectorUIProxy::didClose()
@@ -531,22 +594,30 @@ void WebInspectorUIProxy::closeFrontendPageAndWindow()
     
     SetForScope reentrancyProtector(m_closing, true);
     
-    // Notify WebKit client when a local inspector closes so it can clear _WKInspectorDelegate and perform other cleanup.
-    if (m_inspectedPage)
-        m_inspectedPage->uiClient().willCloseLocalInspector(*m_inspectedPage, *this);
+    // Notify clients when a local inspector closes so it can clear _WKInspectorDelegate and perform other cleanup.
+    if (RefPtr inspectedPage = m_inspectedPage.get()) {
+#if ENABLE(INSPECTOR_EXTENSIONS)
+        if (RefPtr webExtensionController = inspectedPage->webExtensionController())
+            webExtensionController->inspectorWillClose(*this, *inspectedPage);
+#endif
+
+        inspectedPage->uiClient().willCloseLocalInspector(*inspectedPage, *this);
+    }
 
     m_isVisible = false;
     m_isProfilingPage = false;
     m_showMessageSent = false;
     m_ignoreFirstBringToFront = false;
 
-    untrackInspectorPage(m_inspectorPage);
+    RefPtr inspectorPage = m_inspectorPage.get();
+    untrackInspectorPage(inspectorPage.get());
 
-    m_inspectorPage->send(Messages::WebInspectorUI::SetIsVisible(m_isVisible));
-    m_inspectorPage->process().removeMessageReceiver(Messages::WebInspectorUIProxy::messageReceiverName(), m_inspectedPageIdentifier);
+    Ref inspectorPageProcess = inspectorPage->legacyMainFrameProcess();
+    inspectorPageProcess->send(Messages::WebInspectorUI::SetIsVisible(m_isVisible), inspectorPage->webPageIDInMainFrameProcess());
+    inspectorPageProcess->removeMessageReceiver(Messages::WebInspectorUIProxy::messageReceiverName(), m_inspectedPageIdentifier);
 
-    if (m_inspectedPage && m_isActiveFrontend)
-        m_inspectedPage->inspectorController().disconnectFrontend(*this);
+    if (RefPtr inspectedPage = m_inspectedPage.get(); inspectedPage && m_isActiveFrontend)
+        inspectedPage->inspectorController().disconnectFrontend(*this);
 
     m_isActiveFrontend = false;
 
@@ -556,7 +627,7 @@ void WebInspectorUIProxy::closeFrontendPageAndWindow()
 #if ENABLE(INSPECTOR_EXTENSIONS)
     // This extension controller may be kept alive by the IPC dispatcher beyond the point
     // when m_inspectorPage is cleared below. Notify the controller so it can clean up before then.
-    m_extensionController->inspectorFrontendWillClose();
+    protectedExtensionController()->inspectorFrontendWillClose();
     m_extensionController = nullptr;
 #endif
     
@@ -565,30 +636,33 @@ void WebInspectorUIProxy::closeFrontendPageAndWindow()
 
     m_isAttached = false;
     m_canAttach = false;
-    m_underTest = false;
 
     platformCloseFrontendPageAndWindow();
+
+    dispatchDidChangeLocalInspectorAttachment();
 }
 
 void WebInspectorUIProxy::sendMessageToBackend(const String& message)
 {
-    if (!m_inspectedPage)
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
         return;
 
-    m_inspectedPage->inspectorController().dispatchMessageFromFrontend(message);
+    inspectedPage->inspectorController().dispatchMessageFromFrontend(message);
 }
 
 void WebInspectorUIProxy::frontendLoaded()
 {
-    if (!m_inspectedPage)
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
         return;
 
-    if (auto* automationSession = m_inspectedPage->process().processPool().automationSession())
-        automationSession->inspectorFrontendLoaded(*m_inspectedPage);
-    
+    if (RefPtr automationSession = inspectedPage->configuration().processPool().automationSession())
+        automationSession->inspectorFrontendLoaded(*inspectedPage);
+
 #if ENABLE(INSPECTOR_EXTENSIONS)
-    if (m_extensionController)
-        m_extensionController->inspectorFrontendLoaded();
+    if (RefPtr extensionController = m_extensionController)
+        extensionController->inspectorFrontendLoaded();
 #endif
 
     if (m_inspectorClient)
@@ -620,8 +694,8 @@ void WebInspectorUIProxy::attachAvailabilityChanged(bool available)
     if (previousCanAttach == m_canAttach)
         return;
 
-    if (m_inspectorPage && !m_underTest)
-        m_inspectorPage->send(Messages::WebInspectorUI::SetDockingUnavailable(!m_canAttach));
+    if (RefPtr inspectorPage = m_inspectorPage.get(); inspectorPage && !m_underTest)
+        inspectorPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::SetDockingUnavailable(!m_canAttach), inspectorPage->webPageIDInMainFrameProcess());
 
     platformAttachAvailabilityChanged(m_canAttach);
 }
@@ -631,6 +705,19 @@ void WebInspectorUIProxy::setForcedAppearance(InspectorFrontendClient::Appearanc
     platformSetForcedAppearance(appearance);
 }
 
+void WebInspectorUIProxy::effectiveAppearanceDidChange(InspectorFrontendClient::Appearance appearance)
+{
+#if ENABLE(INSPECTOR_EXTENSIONS)
+    if (!m_extensionController)
+        return;
+
+    ASSERT(appearance == WebCore::InspectorFrontendClient::Appearance::Dark || appearance == WebCore::InspectorFrontendClient::Appearance::Light);
+    auto extensionAppearance = appearance == WebCore::InspectorFrontendClient::Appearance::Dark ? Inspector::ExtensionAppearance::Dark : Inspector::ExtensionAppearance::Light;
+
+    protectedExtensionController()->effectiveAppearanceDidChange(extensionAppearance);
+#endif
+}
+
 void WebInspectorUIProxy::openURLExternally(const String& url)
 {
     if (m_inspectorClient)
@@ -652,6 +739,15 @@ void WebInspectorUIProxy::showCertificate(const CertificateInfo& certificateInfo
     platformShowCertificate(certificateInfo);
 }
 
+void WebInspectorUIProxy::setInspectorPageDeveloperExtrasEnabled(bool enabled)
+{
+    RefPtr inspectorPage = m_inspectorPage.get();
+    if (!inspectorPage)
+        return;
+
+    inspectorPage->protectedPreferences()->setDeveloperExtrasEnabled(enabled);
+}
+
 void WebInspectorUIProxy::elementSelectionChanged(bool active)
 {
     m_elementSelectionActive = active;
@@ -674,25 +770,30 @@ void WebInspectorUIProxy::timelineRecordingChanged(bool active)
     m_isProfilingPage = active;
 }
 
-void WebInspectorUIProxy::setDeveloperPreferenceOverride(WebCore::InspectorClient::DeveloperPreference developerPreference, std::optional<bool> overrideValue)
+void WebInspectorUIProxy::setDeveloperPreferenceOverride(WebCore::InspectorBackendClient::DeveloperPreference developerPreference, std::optional<bool> overrideValue)
 {
     switch (developerPreference) {
-    case InspectorClient::DeveloperPreference::PrivateClickMeasurementDebugModeEnabled:
-        if (m_inspectedPage)
-            m_inspectedPage->websiteDataStore().setPrivateClickMeasurementDebugMode(overrideValue && overrideValue.value());
+    case InspectorBackendClient::DeveloperPreference::PrivateClickMeasurementDebugModeEnabled:
+        if (RefPtr inspectedPage = m_inspectedPage.get())
+            inspectedPage->protectedWebsiteDataStore()->setPrivateClickMeasurementDebugMode(overrideValue && overrideValue.value());
         return;
 
-    case InspectorClient::DeveloperPreference::ITPDebugModeEnabled:
-        if (m_inspectedPage)
-            m_inspectedPage->websiteDataStore().setResourceLoadStatisticsDebugMode(overrideValue && overrideValue.value());
+    case InspectorBackendClient::DeveloperPreference::ITPDebugModeEnabled:
+        if (RefPtr inspectedPage = m_inspectedPage.get())
+            inspectedPage->protectedWebsiteDataStore()->setResourceLoadStatisticsDebugMode(overrideValue && overrideValue.value());
         return;
 
-    case InspectorClient::DeveloperPreference::MockCaptureDevicesEnabled:
+    case InspectorBackendClient::DeveloperPreference::MockCaptureDevicesEnabled:
 #if ENABLE(MEDIA_STREAM)
-        if (m_inspectedPage)
-            m_inspectedPage->setMockCaptureDevicesEnabledOverride(overrideValue);
+        if (RefPtr inspectedPage = m_inspectedPage.get())
+            inspectedPage->setMockCaptureDevicesEnabledOverride(overrideValue);
 #endif // ENABLE(MEDIA_STREAM)
         return;
+
+    case InspectorBackendClient::DeveloperPreference::NeedsSiteSpecificQuirks:
+        if (RefPtr inspectedPage = m_inspectedPage.get())
+            inspectedPage->protectedPreferences()->setNeedsSiteSpecificQuirksInspectorOverride(overrideValue);
+        return;
     }
 
     ASSERT_NOT_REACHED();
@@ -702,8 +803,8 @@ void WebInspectorUIProxy::setDeveloperPreferenceOverride(WebCore::InspectorClien
 
 void WebInspectorUIProxy::setEmulatedConditions(std::optional<int64_t>&& bytesPerSecondLimit)
 {
-    if (m_inspectedPage)
-        m_inspectedPage->websiteDataStore().setEmulatedConditions(WTFMove(bytesPerSecondLimit));
+    if (auto inspectedPage = this->inspectedPage())
+        inspectedPage->websiteDataStore().setEmulatedConditions(WTFMove(bytesPerSecondLimit));
 }
 
 #endif // ENABLE(INSPECTOR_NETWORK_THROTTLING)
@@ -711,7 +812,7 @@ void WebInspectorUIProxy::setEmulatedConditions(std::optional<int64_t>&& bytesPe
 void WebInspectorUIProxy::setDiagnosticLoggingAvailable(bool available)
 {
 #if ENABLE(INSPECTOR_TELEMETRY)
-    m_inspectorPage->send(Messages::WebInspectorUI::SetDiagnosticLoggingAvailable(available));
+    protectedInspectorPage()->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::SetDiagnosticLoggingAvailable(available), m_inspectorPage->webPageIDInMainFrameProcess());
 #else
     UNUSED_PARAM(available);
 #endif
@@ -719,7 +820,7 @@ void WebInspectorUIProxy::setDiagnosticLoggingAvailable(bool available)
 
 void WebInspectorUIProxy::save(Vector<InspectorFrontendClient::SaveData>&& saveDatas, bool forceSaveAs)
 {
-    if (!m_inspectedPage->preferences().developerExtrasEnabled())
+    if (!protectedInspectedPage()->protectedPreferences()->developerExtrasEnabled())
         return;
 
     ASSERT(!saveDatas.isEmpty());
@@ -735,7 +836,7 @@ void WebInspectorUIProxy::save(Vector<InspectorFrontendClient::SaveData>&& saveD
 
 void WebInspectorUIProxy::load(const String& path, CompletionHandler<void(const String&)>&& completionHandler)
 {
-    if (!m_inspectedPage->preferences().developerExtrasEnabled())
+    if (!protectedInspectedPage()->protectedPreferences()->developerExtrasEnabled())
         return;
 
     ASSERT(!path.isEmpty());
@@ -747,7 +848,7 @@ void WebInspectorUIProxy::load(const String& path, CompletionHandler<void(const
 
 void WebInspectorUIProxy::pickColorFromScreen(CompletionHandler<void(const std::optional<WebCore::Color> &)>&& completionHandler)
 {
-    if (!m_inspectedPage->preferences().developerExtrasEnabled()) {
+    if (!protectedInspectedPage()->protectedPreferences()->developerExtrasEnabled()) {
         completionHandler({ });
         return;
     }
@@ -757,22 +858,43 @@ void WebInspectorUIProxy::pickColorFromScreen(CompletionHandler<void(const std::
 
 bool WebInspectorUIProxy::shouldOpenAttached()
 {
-    return inspectorPagePreferences().inspectorStartsAttached() && canAttach();
+    return protectedInspectorPagePreferences()->inspectorStartsAttached() && canAttach();
 }
 
 void WebInspectorUIProxy::evaluateInFrontendForTesting(const String& expression)
 {
-    if (!m_inspectorPage)
+    RefPtr inspectorPage = m_inspectorPage.get();
+    if (!inspectorPage)
         return;
 
-    m_inspectorPage->send(Messages::WebInspectorUI::EvaluateInFrontendForTesting(expression));
+    inspectorPage->protectedLegacyMainFrameProcess()->send(Messages::WebInspectorUI::EvaluateInFrontendForTesting(expression), inspectorPage->webPageIDInMainFrameProcess());
+}
+
+#if ENABLE(INSPECTOR_EXTENSIONS)
+RefPtr<WebInspectorUIExtensionControllerProxy> WebInspectorUIProxy::protectedExtensionController() const
+{
+    return extensionController();
+}
+#endif
+
+void WebInspectorUIProxy::dispatchDidChangeLocalInspectorAttachment()
+{
+    RefPtr inspectedPage = m_inspectedPage.get();
+    if (!inspectedPage)
+        return;
+
+    RefPtr pageClient = inspectedPage->pageClient();
+    if (!pageClient)
+        return;
+
+    pageClient->didChangeLocalInspectorAttachment();
 }
 
 // Unsupported configurations can use the stubs provided here.
 
-#if !PLATFORM(MAC) && !PLATFORM(GTK) && !PLATFORM(WIN)
+#if !PLATFORM(MAC) && !PLATFORM(GTK) && !PLATFORM(WIN) && !ENABLE(WPE_PLATFORM)
 
-WebPageProxy* WebInspectorUIProxy::platformCreateFrontendPage()
+RefPtr<WebPageProxy> WebInspectorUIProxy::platformCreateFrontendPage()
 {
     notImplemented();
     return nullptr;
