@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2020-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,28 +28,31 @@
 
 #if ENABLE(INSPECTOR_EXTENSIONS)
 
+#include "JavaScriptEvaluationResult.h"
 #include "Logging.h"
 #include "WebInspectorUI.h"
 #include "WebInspectorUIExtensionControllerMessages.h"
-#include "WebInspectorUIExtensionControllerMessagesReplies.h"
 #include "WebInspectorUIExtensionControllerProxyMessages.h"
 #include "WebPage.h"
 #include "WebProcess.h"
 #include <JavaScriptCore/APICast.h>
 #include <JavaScriptCore/JSCInlines.h>
+#include <JavaScriptCore/JSContextRef.h>
 #include <WebCore/ExceptionDetails.h>
 #include <WebCore/InspectorFrontendAPIDispatcher.h>
+#include <WebCore/JSDOMGlobalObject.h>
 #include <WebCore/SerializedScriptValue.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebKit {
 
-WebInspectorUIExtensionController::WebInspectorUIExtensionController(WebCore::InspectorFrontendClient& inspectorFrontend)
-    : m_frontendClient(inspectorFrontend)
-{
-    auto* page = inspectorFrontend.frontendPage();
-    ASSERT(page);
+WTF_MAKE_TZONE_ALLOCATED_IMPL(WebInspectorUIExtensionController);
 
-    m_inspectorPageIdentifier = WebPage::fromCorePage(*page).identifier();
+WebInspectorUIExtensionController::WebInspectorUIExtensionController(WebCore::InspectorFrontendClient& inspectorFrontend, WebCore::PageIdentifier pageIdentifier)
+    : m_frontendClient(inspectorFrontend)
+    , m_inspectorPageIdentifier(pageIdentifier)
+{
+    ASSERT(inspectorFrontend.frontendPage() && WebPage::fromCorePage(*inspectorFrontend.frontendPage())->identifier() == pageIdentifier);
 
     WebProcess::singleton().addMessageReceiver(Messages::WebInspectorUIExtensionController::messageReceiverName(), m_inspectorPageIdentifier, *this);
 }
@@ -59,7 +62,7 @@ WebInspectorUIExtensionController::~WebInspectorUIExtensionController()
     WebProcess::singleton().removeMessageReceiver(Messages::WebInspectorUIExtensionController::messageReceiverName(), m_inspectorPageIdentifier);
 }
 
-std::optional<Inspector::ExtensionError> WebInspectorUIExtensionController::parseExtensionErrorFromEvaluationResult(InspectorFrontendAPIDispatcher::EvaluationResult result) const
+std::optional<Inspector::ExtensionError> WebInspectorUIExtensionController::parseExtensionErrorFromEvaluationResult(WebCore::InspectorFrontendAPIDispatcher::EvaluationResult result) const
 {
     if (!result) {
         switch (result.error()) {
@@ -159,7 +162,7 @@ void WebInspectorUIExtensionController::unregisterExtension(const Inspector::Ext
     });
 }
 
-JSC::JSObject* WebInspectorUIExtensionController::unwrapEvaluationResultAsObject(InspectorFrontendAPIDispatcher::EvaluationResult result) const
+JSC::JSObject* WebInspectorUIExtensionController::unwrapEvaluationResultAsObject(WebCore::InspectorFrontendAPIDispatcher::EvaluationResult result) const
 {
     if (!result)
         return nullptr;
@@ -184,7 +187,7 @@ void WebInspectorUIExtensionController::createTabForExtension(const Inspector::E
         JSON::Value::create(tabIconURL.string()),
         JSON::Value::create(sourceURL.string()),
     };
-    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("createTabForExtension"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
+    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("createTabForExtension"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](WebCore::InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
         if (!weakThis || !result) {
             completionHandler(makeUnexpected(Inspector::ExtensionError::ContextDestroyed));
             return;
@@ -214,10 +217,10 @@ void WebInspectorUIExtensionController::createTabForExtension(const Inspector::E
     });
 }
 
-void WebInspectorUIExtensionController::evaluateScriptForExtension(const Inspector::ExtensionID& extensionID, const String& scriptSource, const std::optional<URL>& frameURL, const std::optional<URL>& contextSecurityOrigin, const std::optional<bool>& useContentScriptContext, CompletionHandler<void(const IPC::DataReference&, const std::optional<WebCore::ExceptionDetails>&, const std::optional<Inspector::ExtensionError>&)>&& completionHandler)
+void WebInspectorUIExtensionController::evaluateScriptForExtension(const Inspector::ExtensionID& extensionID, const String& scriptSource, const std::optional<URL>& frameURL, const std::optional<URL>& contextSecurityOrigin, const std::optional<bool>& useContentScriptContext, CompletionHandler<void(Expected<WebKit::JavaScriptEvaluationResult, std::optional<WebCore::ExceptionDetails>>&&, const std::optional<Inspector::ExtensionError>&)>&& completionHandler)
 {
     if (!m_frontendClient) {
-        completionHandler({ }, std::nullopt, Inspector::ExtensionError::InvalidRequest);
+        completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::InvalidRequest);
         return;
     }
 
@@ -235,15 +238,15 @@ void WebInspectorUIExtensionController::evaluateScriptForExtension(const Inspect
         WTFMove(optionalArguments)
     };
 
-    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("evaluateScriptForExtension"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
+    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("evaluateScriptForExtension"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](WebCore::InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
         if (!weakThis) {
-            completionHandler({ }, std::nullopt, Inspector::ExtensionError::ContextDestroyed);
+            completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::ContextDestroyed);
             return;
         }
 
         auto* frontendGlobalObject = weakThis->m_frontendClient->frontendAPIDispatcher().frontendGlobalObject();
         if (!frontendGlobalObject) {
-            completionHandler({ }, std::nullopt, Inspector::ExtensionError::ContextDestroyed);
+            completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::ContextDestroyed);
             return;
         }
         
@@ -256,7 +259,7 @@ void WebInspectorUIExtensionController::evaluateScriptForExtension(const Inspect
             } else
                 LOG(Inspector, "Internal error encountered while evaluating upon the frontend: %s", extensionErrorToString(parsedError.value()).utf8().data());
 
-            completionHandler({ }, std::nullopt, parsedError);
+            completionHandler(makeUnexpected(std::nullopt), parsedError);
             return;
         }
 
@@ -264,7 +267,7 @@ void WebInspectorUIExtensionController::evaluateScriptForExtension(const Inspect
         auto objectResult = weakThis->unwrapEvaluationResultAsObject(result);
         if (!objectResult) {
             LOG(Inspector, "Unexpected non-object value returned from InspectorFrontendAPI.evaluateScriptForExtension().");
-            completionHandler({ }, std::nullopt, Inspector::ExtensionError::InternalError);
+            completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::InternalError);
             return;
         }
         ASSERT(result.value());
@@ -272,22 +275,20 @@ void WebInspectorUIExtensionController::evaluateScriptForExtension(const Inspect
         JSC::JSValue errorPayload = objectResult->get(frontendGlobalObject, JSC::Identifier::fromString(frontendGlobalObject->vm(), "error"_s));
         if (!errorPayload.isUndefined()) {
             if (!errorPayload.isString()) {
-                completionHandler({ }, std::nullopt, Inspector::ExtensionError::InternalError);
+                completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::InternalError);
                 return;
             }
 
-            completionHandler({ }, ExceptionDetails { errorPayload.toWTFString(frontendGlobalObject) }, std::nullopt);
+            completionHandler(makeUnexpected(WebCore::ExceptionDetails { errorPayload.toWTFString(frontendGlobalObject) }), std::nullopt);
             return;
         }
 
         JSC::JSValue resultPayload = objectResult->get(frontendGlobalObject, JSC::Identifier::fromString(frontendGlobalObject->vm(), "result"_s));
-        RefPtr<SerializedScriptValue> serializedResultValue = SerializedScriptValue::create(*frontendGlobalObject, resultPayload);
-        if (!serializedResultValue) {
-            completionHandler({ }, std::nullopt, Inspector::ExtensionError::InternalError);
-            return;
-        }
 
-        completionHandler(serializedResultValue->data(), std::nullopt, std::nullopt);
+        if (auto extracted = JavaScriptEvaluationResult::extract(JSContextGetGlobalContext(toRef(frontendGlobalObject)), toRef(frontendGlobalObject, resultPayload)))
+            completionHandler(WTFMove(*extracted), std::nullopt);
+        else
+            completionHandler(makeUnexpected(std::nullopt), std::nullopt);
     });
 }
 
@@ -311,7 +312,7 @@ void WebInspectorUIExtensionController::reloadForExtension(const Inspector::Exte
         WTFMove(optionalArguments)
     };
 
-    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("reloadForExtension"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
+    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("reloadForExtension"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](WebCore::InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
         if (!weakThis) {
             completionHandler(Inspector::ExtensionError::ContextDestroyed);
             return;
@@ -344,7 +345,7 @@ void WebInspectorUIExtensionController::showExtensionTab(const Inspector::Extens
         JSON::Value::create(extensionTabIdentifier),
     };
 
-    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("showExtensionTab"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
+    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("showExtensionTab"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](WebCore::InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
         if (!weakThis) {
             completionHandler(makeUnexpected(Inspector::ExtensionError::ContextDestroyed));
             return;
@@ -374,12 +375,45 @@ void WebInspectorUIExtensionController::showExtensionTab(const Inspector::Extens
     });
 }
 
-// WebInspectorUIExtensionController IPC messages for testing.
-
-void WebInspectorUIExtensionController::evaluateScriptInExtensionTab(const Inspector::ExtensionTabID& extensionTabID, const String& scriptSource, CompletionHandler<void(const IPC::DataReference&, const std::optional<WebCore::ExceptionDetails>&, const std::optional<Inspector::ExtensionError>&)>&& completionHandler)
+void WebInspectorUIExtensionController::navigateTabForExtension(const Inspector::ExtensionTabID& extensionTabIdentifier, const URL& sourceURL, WTF::CompletionHandler<void(const std::optional<Inspector::ExtensionError>&)>&& completionHandler)
 {
     if (!m_frontendClient) {
-        completionHandler({ }, std::nullopt, Inspector::ExtensionError::InvalidRequest);
+        completionHandler(Inspector::ExtensionError::InvalidRequest);
+        return;
+    }
+
+    Vector<Ref<JSON::Value>> arguments {
+        JSON::Value::create(extensionTabIdentifier),
+        JSON::Value::create(sourceURL.string()),
+    };
+    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("navigateTabForExtension"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](WebCore::InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
+        if (!weakThis) {
+            completionHandler(Inspector::ExtensionError::ContextDestroyed);
+            return;
+        }
+
+        auto* frontendGlobalObject = weakThis->m_frontendClient->frontendAPIDispatcher().frontendGlobalObject();
+        if (!frontendGlobalObject) {
+            completionHandler(Inspector::ExtensionError::ContextDestroyed);
+            return;
+        }
+
+        if (auto parsedError = weakThis->parseExtensionErrorFromEvaluationResult(result)) {
+            LOG(Inspector, "Internal error encountered while evaluating upon the frontend: %s", Inspector::extensionErrorToString(*parsedError).utf8().data());
+            completionHandler(parsedError);
+            return;
+        }
+
+        completionHandler(std::nullopt);
+    });
+}
+
+// WebInspectorUIExtensionController IPC messages for testing.
+
+void WebInspectorUIExtensionController::evaluateScriptInExtensionTab(const Inspector::ExtensionTabID& extensionTabID, const String& scriptSource, CompletionHandler<void(Expected<JavaScriptEvaluationResult, std::optional<WebCore::ExceptionDetails>>&&, const std::optional<Inspector::ExtensionError>&)>&& completionHandler)
+{
+    if (!m_frontendClient) {
+        completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::InvalidRequest);
         return;
     }
 
@@ -388,15 +422,15 @@ void WebInspectorUIExtensionController::evaluateScriptInExtensionTab(const Inspe
         JSON::Value::create(scriptSource),
     };
 
-    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("evaluateScriptInExtensionTab"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
+    m_frontendClient->frontendAPIDispatcher().dispatchCommandWithResultAsync("evaluateScriptInExtensionTab"_s, WTFMove(arguments), [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)](WebCore::InspectorFrontendAPIDispatcher::EvaluationResult&& result) mutable {
         if (!weakThis) {
-            completionHandler({ }, std::nullopt, Inspector::ExtensionError::ContextDestroyed);
+            completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::ContextDestroyed);
             return;
         }
 
         auto* frontendGlobalObject = weakThis->m_frontendClient->frontendAPIDispatcher().frontendGlobalObject();
         if (!frontendGlobalObject) {
-            completionHandler({ }, std::nullopt, Inspector::ExtensionError::ContextDestroyed);
+            completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::ContextDestroyed);
             return;
         }
 
@@ -407,7 +441,7 @@ void WebInspectorUIExtensionController::evaluateScriptInExtensionTab(const Inspe
             } else
                 LOG(Inspector, "Internal error encountered while evaluating upon the frontend.");
 
-            completionHandler({ }, std::nullopt, parsedError);
+            completionHandler(makeUnexpected(std::nullopt), parsedError);
             return;
         }
 
@@ -415,7 +449,7 @@ void WebInspectorUIExtensionController::evaluateScriptInExtensionTab(const Inspe
         auto objectResult = weakThis->unwrapEvaluationResultAsObject(result);
         if (!objectResult) {
             LOG(Inspector, "Unexpected non-object value returned from InspectorFrontendAPI.evaluateScriptInExtensionTab().");
-            completionHandler({ }, std::nullopt, Inspector::ExtensionError::InternalError);
+            completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::InternalError);
             return;
         }
         ASSERT(result.has_value());
@@ -423,22 +457,20 @@ void WebInspectorUIExtensionController::evaluateScriptInExtensionTab(const Inspe
         JSC::JSValue errorPayload = objectResult->get(frontendGlobalObject, JSC::Identifier::fromString(frontendGlobalObject->vm(), "error"_s));
         if (!errorPayload.isUndefined()) {
             if (!errorPayload.isString()) {
-                completionHandler({ }, std::nullopt, Inspector::ExtensionError::InternalError);
+                completionHandler(makeUnexpected(std::nullopt), Inspector::ExtensionError::InternalError);
                 return;
             }
 
-            completionHandler({ }, ExceptionDetails { errorPayload.toWTFString(frontendGlobalObject) }, std::nullopt);
+            completionHandler(makeUnexpected(WebCore::ExceptionDetails { errorPayload.toWTFString(frontendGlobalObject) }), std::nullopt);
             return;
         }
 
         JSC::JSValue resultPayload = objectResult->get(frontendGlobalObject, JSC::Identifier::fromString(frontendGlobalObject->vm(), "result"_s));
-        auto serializedResultValue = SerializedScriptValue::create(*frontendGlobalObject, resultPayload);
-        if (!serializedResultValue) {
-            completionHandler({ }, std::nullopt, Inspector::ExtensionError::InternalError);
-            return;
-        }
 
-        completionHandler(serializedResultValue->data(), std::nullopt, std::nullopt);
+        if (auto extracted = JavaScriptEvaluationResult::extract(JSContextGetGlobalContext(toRef(frontendGlobalObject)), toRef(frontendGlobalObject, resultPayload)))
+            completionHandler(WTFMove(*extracted), std::nullopt);
+        else
+            completionHandler(makeUnexpected(std::nullopt), std::nullopt);
     });
 }
 
