@@ -26,6 +26,7 @@
 #include "config.h"
 #include "SharedBufferTest.h"
 
+#include <wtf/FileHandle.h>
 #include <wtf/FileSystem.h>
 #include <wtf/MainThread.h>
 
@@ -36,13 +37,12 @@ void FragmentedSharedBufferTest::SetUp()
     WTF::initializeMainThread();
 
     // create temp file
-    FileSystem::PlatformFileHandle handle;
-    m_tempFilePath = FileSystem::openTemporaryFile("tempTestFile"_s, handle);
-    FileSystem::writeToFile(handle, testData(), strlen(testData()));
-    FileSystem::closeFile(handle);
+    auto result = FileSystem::openTemporaryFile("tempTestFile"_s);
+    m_tempFilePath = result.first;
+    result.second.write(testData().span8());
+    result.second = { };
 
-    m_tempEmptyFilePath = FileSystem::openTemporaryFile("tempEmptyTestFile"_s, handle);
-    FileSystem::closeFile(handle);
+    m_tempEmptyFilePath = FileSystem::createTemporaryFile("tempEmptyTestFile"_s);
 }
 
 void FragmentedSharedBufferTest::TearDown()
