@@ -28,7 +28,7 @@
 #include <memory>
 #include <wpe/wpe.h>
 
-#if defined(ENABLE_ACCESSIBILITY) && ENABLE_ACCESSIBILITY
+#if defined(USE_ATK) && USE_ATK
 typedef struct _AtkObject AtkObject;
 #endif
 
@@ -50,7 +50,7 @@ public:
         virtual bool dispatchTouchEvent(struct wpe_input_touch_event*) { return false; }
     };
     void setInputClient(std::unique_ptr<InputClient>&&);
-#if defined(ENABLE_ACCESSIBILITY) && ENABLE_ACCESSIBILITY
+#if defined(USE_ATK) && USE_ATK
     void setAccessibleChild(AtkObject*);
 #endif
 
@@ -62,9 +62,7 @@ protected:
 
     void initializeAccessibility();
     void updateAccessibilityState(uint32_t);
-#if defined(ENABLE_ACCESSIBILITY) && ENABLE_ACCESSIBILITY
     static void notifyAccessibilityKeyEventListeners(struct wpe_input_keyboard_event* event);
-#endif
 
     void dispatchInputPointerEvent(struct wpe_input_pointer_event*);
     void dispatchInputAxisEvent(struct wpe_input_axis_event*);
